# 0 前言

ISO C++标准委员会在2020年9月通过C++20标准。

C++编程语言适合用来构建高性能的关键业务和关键任务计算系统，包括：

- 操作系统
- 实时系统
- 嵌入式系统
- 游戏系统
- 银行系统
- 空中管制系统
- 通信系统
- ······

## 0.1 着眼于现代C++

所谓“现代C++”，主要指以下四个最新的C++标准：

- C++11
- C++14
- C++17
- C++20

以及即将正式发布的C++23。

重点关注：

- 如何以一种全新的或改进的方法，用C++语言编写代码
- 什么才是C++编程的最佳实践
- 当前专业软件开发的惯例
- 聚焦性能、安全性和软件工程

## 0.2 保持话题性

想要掌握C++语言的发展动态，并以此改变开发人员使用C++语言编写代码的方式，需要搜集足够丰富的信息。它们来自最新的：

- 专业书籍
- 技术文章
- 研究论文
- 白皮书
- 文档
- 博客帖子
- 论坛帖子
- 视频教程
- ······

## 0.3 C++版本

从低版本到高版本，C++的发展历程通常都是从低级别的细节，逐步演变为更容易使用的高级别的表达形式。版本的演进趋势将越来越有利于：

- 缩短开发周期
- 改善运行性能
- 提高安全性
- 增强可维护性

## 0.4 目标受众

本课程适合以下四类学习者：

- 希望学习最新C++20特性的C++软件工程师
- 希望学习C++语言的其它语言软件工程师
- 学习并使用过C++语言的早期版本，希望更新知识储备的在读学生
- 计划开设C++20课程的职业培训教师

## 0.5 代码下载

https://github.com/pdeitel/cplusplus20forprogrammers

CPlusPlus20ForProgrammers-master.zip

## 0.6 三种工业强度的编译器

三大主流平台上支持或部分支持C++20的编译器：

- Windows平台：Visual C++
- Linux平台：GUN C++
- macOS平台：Clang C++

## 0.7 编程技巧与关键特性

- 软件工程：针对大型系统的架构和设计问题
- 安全提示：增强程序的安全性以抵御攻击
- 性能提示：使程序运行得更快，占用内存更少
- 错误提示：演示不正确做法的反面示例
- 核心准则：规则性建议
- 模块：C++20的模块特性
- 概念：C++20的概念特性

## 0.8 对象自然

### 0.8.1 何为对象自然

在不自己定义类的前提下，只使用现成的类创建对象，实现程序的功能。这就是所谓的“对象自然”之法。

C++社区已经提供了大量的免费且开源的类库。

### 0.8.2 免费的类

尽可能使用C++生态系统中现有的类，而不要轻易创造类。它们通常有以下几个来源：

- C++标准库
- 操作系统特有的库
- 免费的第三方库
- 其它开发者贡献的库

GitHub上有大量免费且开源的C++代码实例可供参考。

### 0.8.3 Boost项目

Boost项目提供了多达168个开源的C++库。C++标准库中的一大部分都演进自Boost项目。

### 0.8.4 学习案例

通过如下案例学习对象自然编程：

- 字符串处理
- 任意大小的整数
- 压缩与解压缩
- 加密与解密
- 向量容器
- 连续容器元素的视图
- 读取和解析CSV文件
- 正则表达式
- JSON序列化与反序列化

基于对象自然之法编程，完全无需定义任何类，仅只使用现成的类创建对象，在完全不知道这些类如何实现的前提下，即能很好地操作这些对象，完成程序的功能。

## 0.9 课程概览

![](images/toc.png)

本课程涵盖了C++20的几种编程模型：

- 过程式编程
- 对象式编程
- 函数式编程
- 泛型编程
- 模板元编程

### 0.9.1 第Ⅰ部分：编程基础

- 第1章：开发环境
  - 简要介绍
  - 摩尔定律、多核处理器、标准化的并发编程
  - 面向对象、术语
  - 试运行
    - Visual C++、GUN C++、Clang C++
    - Docker、WSL
- 第2章：基础知识
  - 关键语言特性：输入输出、基本数据类型、算术操作符及其优先级、决策
  - 对象自然案例：借助string类的对象处理字符串
- 第3章：控制语句
  - if-else条件分支语句
  - while循环语句
  - 递增、递减、赋值操作符
  - 对象自然案例：使用第三方库创建任意大小的整数
- 第4章：其它控制语句
  - for循环语句
  - do-while循环语句
  - switch-case开关分支语句
  - break和continue语句
  - 逻辑操作符
  - 对象自然案例：利用miniz-cpp库读写ZIP文件

- 第5章：自定义函数
  - 随机数生成器
  - 函数间的信息传递
  - 函数间的递归调用
  - 对象自然案例：私钥加密


### 0.9.2 第Ⅱ部分：数组、指针和字符串

- 第6章：容器、迭代器和算法
  - array容器
  - 数组的定义、初始化、元素访问和传参
  - 对数组进行排序和查找
  - 多维数组
  - 匿名函数、范围、函数式编程
  - 对象自然案例：综合使用array和vector容器

- 第7章：指针
  - 内置指针、基于指针的数组、基于指针的字符串
  - 现代C++对指针的淡化
    - array和vector容器
    - span
    - string_view

  - 内置数组及其在现代C++中的替代物
  - 对象自然案例：借助span，在不直接使用指针的前提下，查看和操作连续容器

- 第8章：string
  - 读写纯文本文件和CSV文件
  - 基于正则表达式的字符串模式匹配
  - 用string取代C风格的基于指针的字符串
  - string_view
  - 对象自然案例之一：读取分析流行数据集
  - 对象自然案例之二：正则匹配和文本替换


### 0.9.3 第Ⅲ部分：面向对象程序设计

- 第9章：自定义类
  - 每个自定义类都是一种新的数据类型，可用于创建对象，C++是一种可扩展的语言
  - 对象自然案例：基于JSON的对象序列化和反序列化

- 第10章：继承和多态
  - 运行时多态的实现机制：虚函数和动态绑定
  - 运行时多态的其它机制：非虚接口惯用法、variant/visit
  - 面向接口而非实现编程

- 第11章：操作符重载、智能指针、动态内存管理
  - 智能指针提供了内置指针所不具备的额外功能
  - unique_ptr
  - string类的操作符重载
  - 为自定义类重载操作符
  - 重载了操作符的MyArray类
  - 拷贝语义和移动语义
    - 拷贝构造函数
    - 拷贝赋值操作符函数
    - 移动构造函数
    - 移动赋值操作符函数
    - 析构函数

  - 三路比较操作符
  - 自定义转换操作符

- 第12章：异常
  - 何时使用异常
  - 异常的安全保证
  - 构造和析构函数中的异常
  - 如何处理动态内存分配失败
  - 为什么有的项目不使用异常
  - 契约


### 0.9.4 第Ⅳ部分：标准库容器、迭代器和算法

- 第13章：容器及其迭代器
  - 容器、容器适配器、近似容器
  - 常用的数据结构
  - 不同容器支持不同种类的迭代器
  - 利用范围简化代码

- 第14章：泛型算法
  - 通过算法操作容器和容器中的元素
    - 用值填充容器、生成值
    - 比较容器中的元素或整个容器
    - 从容器中移除元素、替换容器中的元素
    - 对容器中的元素做数学运算
    - 在容器中查找特定元素、对容器中的元素做排序
    - 容器间的交换、拷贝和合并
    - 集合操作、确定边界、计算最值
    - 根据算法对迭代器的最低要求，确定哪些容器能使用哪些算法

  - 通过概念指定对范围算法的要求
  - 通过范围和视图拓展函数式编程


### 0.9.5 第Ⅴ部分：高级编程主题

- 第15章：模板
  - 自定义类模板
  - 概念、创建自己的概念
  - 模板化MyArray类
  - 可变参数模板
  - 模板元编程

- 第16章：模块
  - 确定向用户公开哪些声明，同时封装实现细节
  - 提高大型软件系统的构建效率，并为其赋予更佳的可伸缩性
  - 改善代码的整洁性，消除对预处理器的需求，提高编译速度
  - 目前几种主流编译器尚不能完整支持模块功能

- 第17章：多任务
  - 通过为应用程序赋予并发执行多任务的能力，提高其运行性能和可响应性
  - 借助预打包并行算法创建多线程程序
  - 利用chrono库分析并行计算在多核系统上的性能提升
  - 基于并发元语实现生产者——消费者模型
    - latch
    - barrier
    - semaphore

  - 尽量使用高级并发元语、用低级元语实现高级元语

- 第18章：协程
  - 能够暂停当前执行并在稍后由程序的另一部分继续执行的函数
  - co_await、co_yield和co_return关键字
  - 以简单的顺序编码实现并发编程
  - 高级协程支持库

- 第19章：I/O及格式化
  - 基于iomanip的文本格式化
  - 全新的文本格式化特性

- 第20章：其它主题、C++23的新特性

### 0.9.6 附录

- 附录A：按优先级从高到低的顺序排列操作符
- 附录B：字符及其编码
- 附录C：二进制、八进制、十进制和十六进制
- 附录D：编译预处理
- 附录E：位操作符、位域

### 0.9.7 其它

https://deitel.com/c-plus-plus-20-for-programmers

## 0.10 C++核心准则

https://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines

- Bjarne Stroustrup
- Herb Sutter

### 0.10.1 准则支持库

准测支持库包括为各种建议提供支持的辅助类和函数。

微软在GitHub上开源了一个该库的实现：https://github.com/microsoft/GSL

### 0.10.2 静态代码分析工具

静态代码分析工具用于快速检查代码是否存在常见的错误和安全隐患，并提供改进意见：

- clang-tidy：https://clang.llvm.org/extra/clang-tidy
- cppcheck：https://cppcheck.sourceforge.io
- Microsoft的静态代码分析工具内置于Visual C++的静态代码分析器中

以上三种工具可就以下几个方面对代码进行检查：

- 是否遵循了C++核心准则？
- 是否遵循了C++编码标准？
- 是否遵循了现代C++惯例？
- 是否存在潜在的安全隐患？
- 是否存在常见的Bug？
- 是否存在可能的性能缺陷？
- 是否具有良好的可读性？
- ······

在使用GUN C++和Clang C++编译器编译代码时，建议加上-Wall编译选项，以给出全部警告信息。

## 0.11 教学方法

- 实时编码，标准规范
- 主题鲜明，重点突出
- 目标明确，提纲挈领
- 表格罗列，图文并茂
- 编程技巧，关键特性

## 0.12 开发者资源

### 0.12.1 StackOverflow

StackOverflow（https://stackoverflow.com）是最受欢迎的面向开发人员的技术问答网站之一。

### 0.12.2 GitHub

在GitHub（https://github.com）上有超过47000个C++代码仓库，是寻找免费的开源代码，并将其集成到自己项目中的绝佳去处。

2018年，微软斥资75亿美元收购了GitHub。来自世界各地的使用各种编程语言的软件开发人员，每天都在GitHub上研究他人的开源代码，并贡献自己的代码。

### 0.12.3 Docker

为了使用一个软件，往往需要先安装该软件赖以运行的操作系统，并在该系统上安装包括各种库在内的运行环境，然后才能安装该软件本身，之后可能还需要进行一些列复杂的配置。

Docker（https://www.docker.com）是一种将软件打包成容器的工具。只需下载现成的，免费的Docker容器，就能直接运行其中的软件。避免了复杂的安装和配置问题。

将自己的工作封装在自定义的Docker容器中，令自己或他人随时随地都能再现工作的成果。可重复性在包括科学研究在内的很多领域尤其重要。

## 0.13 重要的C++资源

### 0.13.1 文档

- 由C++标准委员会通过的C++20标准文档最终草案：https://timsong-cpp.github.io/cppwp/n4861
- C++参考手册：https://zh.cppreference.com
- C++ FAQ：https://isocpp.org/faq
- Microsoft的C++语言文档：https://learn.microsoft.com/zh-cn/cpp/cpp
- GCC在线文档：https://gcc.gnu.org/onlinedocs

### 0.13.2 博客

- Sutter’s Mill：https://herbsutter.com
- Microsoft C++团队：https://devblogs.microsoft.com/cppblog
- Marius Bancila：https://mariusbancila.ro/blog
- Jonathan Boccara：https://www.fluentcpp.com
- Bartlomiej Filipek：https://www.cppstories.com
- Rainer Grimm：https://www.modernescpp.com
- Arthur O'Dwyer：https://quuxplusone.github.io/blog

### 0.13.3 其它

- C++之父（Bjarne Stroustrup）：https://stroustrup.com
- Standard C++ Foundation：https://isocpp.org
- C++标准委员会：https://www.open-std.org/jtc1/sc22/wg21

## 0.14 自己寻找答案

- 与C++相关的常规技术论坛：

  - https://stackoverflow.com

  - https://www.reddit.com/r/cpp

  - https://groups.google.com/g/comp.lang.c++

  - https://www.dreamincode.net/forums/forum/15-c-and-c

  - https://www.geeksforgeeks.org/stuck-in-programming-get-the-solution-from-these-10-best-websites

- 工具和库的开发商论坛

- 工具和库在GitHub上的“issues（议题）”标签

## 0.15 安装编译器

### 0.15.1 在Windows系统上安装Visual Studio社区版

打开https://visualstudio.microsoft.com/zh-hans/downloads，点击“社区”下面的“免费下载”。安装过程中选择“使用C++的桌面开发”。

### 0.15.2 在macOS系统上安装Xcode

在App Store中搜索Xcode。

### 0.15.3 在Ubuntu系统上安装GNU C++

执行如下命令：

```bash
sudo apt-get update
sudo apt-get install build-essential gdb
```

### 0.15.4 在Docker中安装GNU C++和Clang C++

从https://www.docker.com/get-started下载并安装Docker Desktop。

以管理员身份在PowerShell中执行如下命令：

```bash
Enable-WindowsOptionalFeature -Online -FeatureName $("Microsoft-Hyper-V", "Containers") -All
```

重启系统。

在https://hub.docker.com注册一个Docker Hub账户：

- 邮箱：johnmin74@hotmail.com
- 户名：johnmin74
- 密码：Bedknbsklwt17908

执行如下命令，下载GNU C++容器：

```bash
docker pull gcc:latest
```

执行如下命令，下载Clang C++容器：

```bash
docker pull teeks99/clang-ubuntu:latest
```

## 0.16 安装静态代码分析工具

### 0.16.1 在Ubuntu系统上安装clang-tidy

执行如下命令：

```bash
sudo apt-get update
sudo apt-get install clang-tidy
```

### 0.16.2 在Ubuntu系统上安装cppcheck

执行如下命令：

```bash
sudo apt-get update
sudo apt-get install cppcheck
```

### 0.16.3 在Visual C++中配置静态代码分析工具

1. 右键点击项目名称，选择“属性”；
2. 选择“代码分析”下的“常规”，将“生成时启用代码分析”设为“是”；
3. 选择“代码分析”下的“Microsoft”，通过“<选择多个规则集...>”选择所有以“C++ Core Check”开头的规则集；
4. 点击“另存为...”，指定规则集名称，点击“保存”；
5. 最后点击“应用”，完成配置。

## 0.17 联系我们

在课程学习过程中遇到任何问题，可以通过minwei@tedu.cn联系我们，我们会及时作出答复。