## 1.2 编译并运行C++20应用程序

第一个C++应用程序——猜数字游戏：

- 产生一个介于1到1000之间的随机数，提示用户猜测
  - 猜对，换另一个数继续猜，除非用户选择退出
  - 猜错，提示所猜数字比正确答案偏高还是偏低，重猜，直到猜对为止
- 重猜次数没有限制

尝试在不同平台上使用不同编译器编译这个程序：

- 在Windows上用Visual C++编译
- 在macOS上用Clang C++编译
- 在Ubuntu上用GNU C++编译
- 在Docker中用GNU C++编译
- 在Docker中用Clang C++编译

### 1.2.1 在Windows上用VS2022编译并运行C++20应用程序

...\Ch01\GuessNumber\GuessNumber.cpp：

```c++
// GuessNumber.cpp
//
// 产生一个介于1到1000之间的随机数，让用户猜
//
#include <iostream>
#include <random> // C++11提供的随机数生成器
using namespace std;

bool isCorrect(int, int); // 函数原型

int main() {
    // 使用默认随机数生成引擎，产生一个介于1到1000之间的，服从均匀分布的伪随机数
    default_random_engine engine{ random_device{}() };
    uniform_int_distribution<int> randomInt{ 1, 1000 };

    char response = 'n'; // 是否继续猜

    // 循环，直到用户输入“n”退出游戏
    do {
        // 产生一个大于等于1且小于等于1000的随机数
        const int answer{ randomInt(engine) };

        // 提示用户猜测
        cout << "我有一个介于1到1000之间的数字。\n"
             << "你能猜中我的数字吗？\n"
             << "请输入你的首次猜测：";
        int guess;
        cin >> guess;

        // 循环，直到猜对为止
        while (!isCorrect(guess, answer)) {
            cin >> guess;
        }

        // 提示进入下一轮
        cout << "\n太棒了！你猜对了！\n"
             << "你想换个数接着猜吗？(y/n) ";
        cin >> response;

        cout << endl;
    }   while (response == 'y');

    return 0; // 成功退出
}

// 猜对返回true，猜错显示提示并返回false
bool isCorrect(int guess, int answer) {
    // 猜对
    if (guess == answer) {
        return true;
    }

    // 猜错，显示提示
    if (guess < answer) {
        cout << "太小了，再试一次：";
    }
    else {
        cout << "太大了，再试一次：";
    }

    return false;
}   // isCorrect函数结束
```

右键点击项目名称，选择“属性”，将“C/C++”下“语言”中的“C++语言标准”设为“ISO C++20 标准”，点击“确定”。

按快捷键<Ctrl+F5>编译运行：

```
我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太小了，再试一次：600
太小了，再试一次：700
太大了，再试一次：650
太大了，再试一次：640
太小了，再试一次：645
太大了，再试一次：644
太大了，再试一次：643
太大了，再试一次：642

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) y

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太小了，再试一次：600
太小了，再试一次：700
太小了，再试一次：800
太小了，再试一次：900
太小了，再试一次：910
太小了，再试一次：920
太小了，再试一次：930
太大了，再试一次：929
太大了，再试一次：928
太大了，再试一次：927
太大了，再试一次：926
太大了，再试一次：925

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) n
```

### 1.2.2 在macOS上用Xcode编译并运行C++20应用程序

······

### 1.2.3 在Linux上用gcc编译并运行C++20应用程序

在.../Ch01/GuessNumber目录下编译：

```bash
g++ -std=c++20 GuessNumber.cpp -o ../x64/Debug/GuessNumber
```

在.../Ch01/x64/Debug目录下运行：

```
./GuessNumber

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太大了，再试一次：400
太小了，再试一次：450
太小了，再试一次：460
太小了，再试一次：470
太小了，再试一次：480
太小了，再试一次：490

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) y

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太大了，再试一次：400
太大了，再试一次：300
太大了，再试一次：200
太小了，再试一次：250
太大了，再试一次：240
太大了，再试一次：230
太大了，再试一次：220
太大了，再试一次：210
太大了，再试一次：205
太大了，再试一次：204
太大了，再试一次：203
太大了，再试一次：202
太大了，再试一次：201

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) n
```

### 1.2.4 在Docker容器中用gcc编译并运行C++20应用程序

在...\Ch01目录下启动GNU C++容器：

```bash
docker run --rm -it -v "%cd%":/usr/src gcc:latest
```

在/usr/src/GuessNumber目录下编译：

```bash
g++ -std=c++20 GuessNumber.cpp -o ../x64/Debug/GuessNumber
```

在/usr/src/x64/Debug目录下运行：

```
./GuessNumber

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太小了，再试一次：600
太小了，再试一次：700
太小了，再试一次：800
太大了，再试一次：750
太小了，再试一次：760
太小了，再试一次：770
太小了，再试一次：780
太小了，再试一次：790

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) y

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太小了，再试一次：600
太小了，再试一次：700
太小了，再试一次：800
太大了，再试一次：750
太大了，再试一次：740
太小了，再试一次：745
太大了，再试一次：744

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) n
```

### 1.2.5 在Docker容器中用clang++编译并运行C++20应用程序

在...\Ch01目录下启动Clang C++容器：

```bash
docker run --rm -it -v "%cd%":/usr/src teeks99/clang-ubuntu:latest
```

在/usr/src/GuessNumber目录下编译：

```bash
clang++-16 -std=c++20 GuessNumber.cpp -o ../x64/Debug/GuessNumber
```

在/usr/src/x64/Debug目录下运行：

```
./GuessNumber

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太大了，再试一次：400
太大了，再试一次：300
太小了，再试一次：350
太大了，再试一次：340
太大了，再试一次：330
太大了，再试一次：320
太大了，再试一次：310
太小了，再试一次：311
太小了，再试一次：312
太小了，再试一次：313
太小了，再试一次：314
太小了，再试一次：315
太小了，再试一次：316
太小了，再试一次：317
太小了，再试一次：318

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) y

我有一个介于1到1000之间的数字。
你能猜中我的数字吗？
请输入你的首次猜测：500
太大了，再试一次：400
太大了，再试一次：300
太大了，再试一次：200
太小了，再试一次：250
太大了，再试一次：240
太大了，再试一次：230
太大了，再试一次：220
太大了，再试一次：210
太小了，再试一次：211
太小了，再试一次：212
太小了，再试一次：213
太小了，再试一次：214
太小了，再试一次：215
太小了，再试一次：216

太棒了！你猜对了！
你想换个数接着猜吗？(y/n) n
```