## 1.4 面向对象简单回顾

几乎任何名词都可以通过属性（如名称、大小、颜色）和行为（如移动、计算、通信）的组合，被合理地表示为一个软件对象。用于缔造对象的类本质上是可重用的软件组件。面向对象的程序设计，不仅令开发团队的效率更高，而且这样的程序代码通常也更容易理解、修改和维护。

### 1.4.1 汽车对象

汽车要想真正开动起来，必须：

- 先有人绘制设计图纸
- 再有人把它制造出来
- 最后还需要人来驾驶

即使是一个对汽车工作原理一无所知的人也能够轻松驾驶汽车，因为：

- 油门踏板向司机隐藏了使汽车加速的复杂机制
- 制动踏板向司机隐藏了使汽车减速的复杂机制
- 转向机构向司机隐藏了使汽车转向的复杂机制
- ······

而所有这一切皆源自对汽车的设计。

### 1.4.2 函数、成员函数和类

函数中包含执行特定任务所需要的一系列语句，它向调用者隐藏了这些语句。C++的类就是这样一种程序单元，它容纳了执行该类任务所需要的一组函数。这组函数被称为该类的成员函数。类就象汽车的设计图纸，它包含了油门踏板、制动踏板、转向机构等的设计。

```mermaid
graph LR
  subgraph 类
    func1(成员函数)
    func2(成员函数)
    func3(成员函数)
  end
  subgraph 汽车设计图
    act1(油门踏板)
    act2(制动踏板)
    act3(转向机构)
  end
  func1---act1
  func2---act2
  func3---act3
```

### 1.4.3 实例化

汽车要想开起来必须有人根据它的设计图把它制造出来。类的成员函数所定义的任务要想被执行，也必须先根据类创建出一个对象。这个过程称为实例化，对象即类的实例。

```mermaid
graph LR
  subgraph 描述
	map(汽车设计图)
	cls(类)
  end
  subgraph 实体
	car(汽车)
	obj(对象)
  end
  map--制造-->car
  cls--实例化-->obj
```

### 1.4.4 重用

汽车设计图可以重复使用多次，以批量地生产汽车。

```mermaid
graph TB
  map(汽车设计图)
  car1(汽车)
  car2(汽车)
  car3(汽车)
  map--生产-->car1
  map--生产-->car2
  map--生产-->car3
```

类也同样可以重复使用多次，以批量地创建对象。

```mermaid
graph TB
  cls(类)
  obj1(对象)
  obj2(对象)
  obj3(对象)
  cls--创建-->obj1
  cls--创建-->obj2
  cls--创建-->obj3
```

重用有助于高效地构建可靠的系统，因为现有的类通常都已经过广泛地测试、调试和优化。

### 1.4.5 消息和成员函数调用

司机操作汽车的油门踏板、制动踏板和转向机构，即向汽车发送了消息，汽车针对每种操作做出不同的响应。

```mermaid
graph LR
  driver(司机)
  subgraph 汽车
    act1(油门踏板)
    act2(制动踏板)
    act3(转向机构)
  end
  driver--操作-->act1
  driver--操作-->act2
  driver--操作-->act3
```

用户调用对象的成员函数，即向对象发送了消息，对象针对每种调用执行不同的处理过程。

```mermaid
graph LR
  user(用户)
  subgraph 对象
    func1(成员函数)
    func2(成员函数)
    func3(成员函数)
  end
  user--调用-->func1
  user--调用-->func2
  user--调用-->func3
```

### 1.4.6 属性和数据成员

汽车除了拥有油门踏板、制动踏板、转向机构等与行为相关的特征外，还同时具备诸如车身颜色、车门数量、油箱油量、行驶速度、行驶里程等与属性相关的特征。汽车的属性和汽车的行为一样，也是其设计图的一部分。类似地，对象也有属性，在描述对象的类中，表现为成员变量或数据成员。

```mermaid
graph LR
  subgraph 类
    func1(成员函数)
    func2(成员函数)
    func3(成员函数)
    data1(数据成员)
    data2(数据成员)
    data3(数据成员)
    data4(数据成员)
    data5(数据成员)
end
  subgraph 汽车设计图
    act1(油门踏板)
    act2(制动踏板)
    act3(转向机构)
    prop1(车身颜色)
    prop2(车门数量)
    prop3(油箱油量)
    prop4(行驶速度)
    prop5(行驶里程)
  end
  func1---act1
  func2---act2
  func3---act3
  data1---prop1
  data2---prop2
  data3---prop3
  data4---prop4
  data5---prop5
```

### 1.4.7 封装

类通过成员函数和数据成员将行为和属性封装在对象中。

```mermaid
graph LR
  subgraph 类
    func(成员函数)
    data(数据成员)
  end
  subgraph 对象
    act(行为)
    prop(属性)
  end
  func---act
  data---prop
```

对象之间可以交流，但它们并不需要知道其它对象的实现细节，这些细节被隐藏在对象内部。

```mermaid
graph LR
  obj(对象)
  subgraph 对象
    interface(接口)
    implement(细节)
    interface-->implement
  end
  obj--访问-->interface
```

这种信息隐藏对于良好的软件工程而言至关重要。

### 1.4.8 继承

继承体现了一种一般与特殊的关系。借助继承，可以快速、方便地创建新类。新类接收了现有类的特征，并通过扩展来增加自己的特征。敞篷车显然也是一种汽车，它的特殊性在于车顶可以伸缩。

```mermaid
classDiagram
  现有类 <|-- 新类 : 继承
  现有类 : 现有的数据成员
  现有类 : 现有的成员函数()
  新类 : 新增的数据成员
  新类 : 新增的成员函数()
```

### 1.4.9 面向对象的分析和设计

对于一个庞大而复杂的软件系统，在着手编写代码之前，最好先制定一套最佳的解决方案。其中包括：

- 一个详细的分析过程，以明确项目的需求，即定义系统应该做什么
- 一个完整的设计过程，以明确实现的步骤，即定义系统应该如何做

如果上述分析和设计的过程是从面向对象的角度出发的，就称为面向对象的分析和设计（Object-Oriented Analysis and Design, OOAD）。基于面向对象的分析和设计的编程方法，称为面向对象编程（Object-Oriented Programming, OOP）。支持面向对象编程的程序设计语言，称为面向对象的编程语言，比如C++。