## 2.4 另一个C++程序

下面的程序读取用户通过键盘输入的两个整数，相加求和，并输出结果。

...\Ch02\Sum\Sum.cpp：

```c++
// Sum.cpp
//
// 计算两个整数的和
//
#include <iostream> // 使程序可以从键盘输入数据并在屏幕上输出数据

// 主函数是程序执行的起点
int main() {
    // 声明并初始化变量
    int number1{ 0 }; // 用于加法计算的第一个整数，初始化为0
    int number2{ 0 }; // 用于加法计算的第二个整数，初始化为0
    int sum{ 0 }; // 第一个整数和第二个整数相加的和，初始化为0

    std::cout << "输入第一个整数："; // 提示用户输入数据
    std::cin >> number1; // 读取第一个整数到number1中

    std::cout << "输入第二个整数："; // 提示用户输入数据
    std::cin >> number2; // 读取第二个整数到number2中

    sum = number1 + number2; // 将第一个整数和第二个整数相加求和，结果保存到sum中

    std::cout << "两个整数相加和：" << sum << "\n"; // 显示计算结果
} // 主函数结束
```

编译运行：

```
输入第一个整数：123
输入第二个整数：456
两个整数相加和：579
```

### 2.4.1 变量的声明与初始化

程序代码的第10到12行声明并初始化了三个类型为int即整型，名字分别为number1、number2和sum的变量。所有的变量声明必须包含一个数据类型和一个名称。在变量名的后面用一对花括号括起它们的初值。这种语法称为花括号初始化。初始化虽然不是必须的，但这样做有助于防止很多难以预料的问题。有关变量初始化的旧式语法类似下面这个样子：

```c++
int number1 = 0; // 用于加法计算的第一个整数，初始化为0
int number2 = 0; // 用于加法计算的第二个整数，初始化为0
int sum = 0; // 第一个整数和第二个整数相加的和，初始化为0
```

这种写法现在依然可用，但不建议。

### 2.4.2 一次声明多个变量

相同类型的多个变量可以在一条声明语句中被声明和初始化，其间以逗号分隔，如下所示：

```c++
int number1{ 0 }, number2{ 0 }, sum{ 0 }; // 用于加法计算的两个整数及它们的和，初始化为0
```

这种写法的可读性并不好，而且也不方便为每个变量单独加注释说明。

### 2.4.3 基本类型

关键字int表示整数类型，int型变量可以存放类似13、29或者-42这样的数值。

关键字double表示双精度浮点数类型，double型变量用于存放类似3.4、0.0或者-11.19这样的数值。

关键字char表示字符类型，char型变量只能容纳一个小写英文字母、一个大写英文字母、一个介于0到9之间的阿拉伯数字，或者一个特殊符号，如“+”、“#”、“$”等。

C++语言的完整基本数据类型列表及其取值范围，参见：https://zh.cppreference.com/w/cpp/language/types。

### 2.4.4 标识符和驼峰命名法

变量名，如number1等，可以是任何有效的标识符。标识符是由26个英文字母大小写、0到9十个阿拉伯数字和下划线组成的字符序列，第一个字符不能是数字，整个标识符不能与关键字冲突。C++语言是严格区分大小写的，Ok、ok、oK和OK四个完全不同的标识符。

C++语言对标识符的长度没有限制。

自己定义的标识符最好不要以一个下划线跟一个大写字母开头，也不要以两个下划线开头。这种形式的标识符是C++编译器内部使用的。

根据惯例，作为变量名的标识符，通常以小写字母开头，如果其中包含多个单词，从第二个单词开始，每个单词的首字母大写。例如：size、minimumSize、minimumSizeHint等。这种命名惯例称为驼峰命名法。

### 2.4.5 变量声明的位置

可以在程序的任何地方声明变量，只要在使用之前即可。例如，将程序代码中第10、11和12行的声明语句：

```c++
...
int number1{ 0 }; // 用于加法计算的第一个整数，初始化为0
int number2{ 0 }; // 用于加法计算的第二个整数，初始化为0
int sum{ 0 }; // 第一个整数和第二个整数相加的和，初始化为0
...
```

分别放到第15、18和20行之前，没有任何问题：

```c++
...
int number1{ 0 }; // 用于加法计算的第一个整数，初始化为0
std::cin >> number1; // 读取第一个整数到number1中
...
int number2{ 0 }; // 用于加法计算的第二个整数，初始化为0
std::cin >> number2; // 读取第二个整数到number2中
...
int sum{ 0 }; // 第一个整数和第二个整数相加的和，初始化为0
sum = number1 + number2; // 将第一个整数和第二个整数相加求和，结果保存到sum中
...
```

其中，变量sum的声明、初始化和赋值甚至可以合并书写成这样：

```c++
int sum{ number1 + number2 }; // 第一个整数和第二个整数相加的和，初始化为number1+number2
```

### 2.4.6 从用户处获取第一个值

为了从用户处获取值，在读取用户输入前先打印一条提示信息，是很常见的做法：

```c++
std::cout << "输入第一个整数："; // 提示用户输入数据
```

打印完这条提示信息，光标位于冒号后面：

```
输入第一个整数：_
```

同时程序执行下面的输入语句，等待用户输入：

```c++
std::cin >> number1; // 读取第一个整数到number1中
```

来自std名字空间的cin对象表示标准输入流，即键盘。输入语句中的“>>”称为流提取操作符，表示从其左边的流对象cin中，提取值到其右边的变量中。注意箭头的方向就是数据的流动方向。当用户通过键盘输入一个整数并按下回车键时：

```
输入第一个整数：123<回车>
```

输入语句执行完毕，变量number1得到用户输入的值123。

### 2.4.7 从用户处获取第二个值

类似的操作过程再度执行：

```c++
std::cout << "输入第二个整数："; // 提示用户输入数据
std::cin >> number2; // 读取第二个整数到number2中
```

用户根据提示输入第二个整数并按下回车键：

```
输入第一个整数：456<回车>
```

输入语句执行完毕，变量number2得到用户输入的值456。

### 2.4.8 计算用户输入值的和

程序代码的第20行是一条赋值语句，将变量number1和number2中的值相加，结果经由赋值操作符“=”保存到变量sum中。这里的“+”和“=”都是二元操作符，即都有两个操作数。加号操作符“+”的左右操作数分别为number1和number2，表示左操作数加上右操作数，表达式的值为相加的结果。赋值操作符“=”的左右操作数分别为sum和表达式number1+number2的值，表示将右操作数赋值给左操作数，表达式的值为左操作数本身。二元操作符左右两侧的空格可以使操作符更加醒目，提高程序代码的可读性。

### 2.4.9 显示计算结果

程序代码的第22行用于显示计算结果。在同一条输出语句中可以输出不同类型的值。比如在这条语句中：

```c++
std::cout << "两个整数相加和：" << sum << "\n"; // 显示计算结果
```

既有字符串，也有整数。流插入操作符“<<”知道如何输出不同类型的数据。在单条语句中的使用多个流插入操作符，称为对流插入操作进行连接。

也可以在输出语句中执行计算，从而省略了变量sum，就象下面这样：

```c++
std::cout << "两个整数相加和：" << number1 + number2 << "\n"; // 显示计算结果
```

C++语言的标志性特征就是可以创建自己的数据类型，称为类，甚至可以教会C++如何使用流提取操作符“>>”和流插入操作符“<<”，输入输出该类的对象，这就叫操作符重载。