## 2.5 算术运算

C++语言为算术运算提供的算术操作符如下表所示：

算术运算|算术操作符|数学形式|表达式形式
:-:|:-:|:-:|:-:
加法|`+`|$a+b$|`a + b`
减法|`-`|$a-b$|`a - b`
乘法|`*`|$a\times b$、$a\cdot b$或$ab$|`a * b`
除法|`/`|$a\div b$、$\frac a b$或$a/b$|`a / b`
求余|`%`|$a\mod b$|`a % b`

注意这些算术运算的数学形式和表达式形式并不总是一致的，特别是一些特殊符号，如“*”和“%”的使用。

C++的算术操作符都是二元操作符，即作用于其左右两个操作数。

### 2.5.1 整数除法和求余

整数间的除法和求余在数学上是这样定义的：

被除数$\div$除数$=$商$\cdots$余数，其中：

除数$\times$商$+$余数$=$被除数，例如：

$17\div5=3\cdots2$

$17\div(-5)=-3\cdots2$

$-17\div5=-3\cdots-2$

$-17\div(-5)=3\cdots-2$

注意余数的正负总是和被除数的正负保持一致。

在C++中两个整数相除得到的商依然是一个整数，计算规则与数学定义严格一致：

```c++
std::cout <<  17 /  5 << ", " <<  17 %  5 << std::endl; //  3,  2
std::cout <<  17 / -5 << ", " <<  17 % -5 << std::endl; // -3,  2
std::cout << -17 /  5 << ", " << -17 %  5 << std::endl; // -3, -2
std::cout << -17 / -5 << ", " << -17 % -5 << std::endl; //  3, -2
```

### 2.5.2 用圆括号对子表达式分组

圆括号在C++表达式中的用法和在数学中的用法一样，即通过对子表达式分组人为指定运算的顺序：

```c++
std::cout << 3 *  2 + 1  << std::endl; // 7
std::cout << 3 * (2 + 1) << std::endl; // 9
std::cout << 3 -  2 + 1  << std::endl; // 2
std::cout << 3 - (2 + 1) << std::endl; // 0
```

### 2.5.3 操作符的优先级和关联性

当一个表达式中含有多个操作符时，它们的计算顺序由优先级决定，相同优先级的操作符，其计算顺序由关联性决定：

- 算术操作符的优先级和它们在数学中的定义相同，即先乘除后加减
  ```
  3 * 2 + 1
  |___|   |
    6     |
    |_____|
       7
  ```
  
- 算术操作符的关联性都是从左至右
  ```
  3 - 2 + 1
  |___|   |
    1     |
    |_____|
       2
  ```
  
- 通过圆括号可以人为指定运算顺序
  ```
  3 * (2 + 1)
  |    |___|
  |      3
  |______|
     9
  
  3 - (2 + 1)
  |    |___|
  |      3
  |______|
      0
  ```