## 2.6 决策与关系运算

### 2.6.1 关系操作符

C++语言为关系运算提供的关系操作符如下表所示：

关系运算|关系操作符|数学形式|表达式形式
:-:|:-:|:-:|:-:
小于|`<`|$a<b$|`a < b`
小于等于|`<=`|$a\leqslant b$|`a <= b`
大于|`>`|$a>b$|`a > b`
大于等于|`>=`|$a\geqslant b$|`a >= b`
等于|`==`|$a=b$|`a == b`
不等于|`!=`|$a\ne b$|`a != b`

- 前四个关系操作符具有相同的优先级，关联性从左至右
- 后两个关系操作符具有相同的优先级，关联性从左至右
- 后两个关系操作符的优先级低于前四个关系操作符
- 包含两个字符的关系操作符，如“<=”、“>=”、“!=”等，字符顺序不能颠倒
- 严格区分表示相等的“==”操作符和表示赋值的“=”操作符

### 2.6.2 决策

下面的程序通过六个if语句比较用户输入的两个整数。如果其中某个if语句的条件为真，其后花括号中的语句块就会执行，否则就不执行。

...\Ch02\Decision\Decision.cpp：

```c++
// Decision.cpp
//
// 借助if语句和关系操作符比较两个整数
//
#include <iostream> // 使程序可以从键盘输入数据并在屏幕上输出数据

using std::cout; // 声明std名字空间中的名字cout
using std::cin; // 声明std名字空间中的名字cin

// 主函数是程序执行的起点
int main() {
    int number1{ 0 }; // 第一个参与比较的整数，初始化为0
    int number2{ 0 }; // 第二个参与比较的整数，初始化为0

    cout << "输入两个整数："; // 提示用户输入数据
    cin >> number1 >> number2; // 读取用户输入的数据

    if (number1 < number2) {
        cout << number1 << "小于" << number2 << "\n";
    }

    if (number1 <= number2) {
        cout << number1 << "小于等于" << number2 << "\n";
    }

    if (number1 > number2) {
        cout << number1 << "大于" << number2 << "\n";
    }

    if (number1 >= number2) {
        cout << number1 << "大于等于" << number2 << "\n";
    }

    if (number1 == number2) {
        cout << number1 << "等于" << number2 << "\n";
    }

    if (number1 != number2) {
        cout << number1 << "不等于" << number2 << "\n";
    }
} // 主函数结束
```

编译运行：

```
输入两个整数：123 456
123小于456
123小于等于456
123不等于456

输入两个整数：456 123
456大于123
456大于等于123
456不等于123

输入两个整数：123 123
123小于等于123
123大于等于123
123等于123
```

#### 2.6.2.1 using声明

程序代码的第7行和第8行，通过using关键字将std名字空间中的名字cout和cin，声明到当前作用域。其后的代码可以象引用任何在当前作用域中声明的标识符一样，直接引用cout和cin，而无需再通过“std::”作用域限定，显式表明其来自std名字空间。

#### 2.6.2.2 using指令

与使用using声明的效果类似的另一种写法，是使用using指令：

```c++
using namespace std;
```

它的作用是使std名字空间中的所有名字，在当前作用域中都可见。这样做的结果是，在其后的代码中可以直接引用cout和cin，而无需使用“std::”作用域限定。事实上，using指令是比using声明更常用的，简化代码书写的方法。

#### 2.6.2.3 声明变量并读取用户输入

程序代码的第12行和第13行声明了两个变量number1和number2，并通过花括号语法为其指定了初值0。第16行则通过两个连续的“>>”操作符，从代表标准输入即键盘的cin对象中，依次提取两个整数到number1和number2变量中。

#### 2.6.2.4 比较数值

程序代码的第18、22、26、30、34和38行结合使用了if语句和关系操作符，比较变量number1和unmber2中的值。一旦条件为真，相应的if块即被执行，输出比较结果。

#### 2.6.2.5 花括号和语句块

与每个if语句相对应的执行体可以是单条语句，也可以是由多条语句组成的复合语句。除了使用正确的缩进表示层次关系外，还通过一对花括号将其括起来，这样的结构称为语句块或块。当语句块中仅包含一条语句时，作为块边界的一对花括号可以省略不写。例如：

```c++
if (number1 < number2)
    cout << number1 << "小于" << number2 << "\n";
```

但很多时候，为了避免错误，建议养成将所有if语句的执行体都用花括号括起来的习惯，无论其中是单条语句还是多条语句。

#### 2.6.2.6 在条件之后误加分号

在if语句的条件右圆括号后面紧跟一个分号：

```c++
if (number1 < number2); {
    cout << number1 << "小于" << number2 << "\n";
}
```

这并不是一个语法错误，但通常会导致逻辑错误。分号导致if语句的执行体为空，即什么也不做，但本该作为该if语句执行体的语句块却与该if语句分离。无论if后面的条件是否为真，该块总会被执行。这常常导致程序执行结果错误。大多数编译器会对这种情况发出警告。

#### 2.6.2.7 分割冗长的语句

出于可读性的考虑，通常会将一条过于冗长的语句分割成多行书写。这时就需要遵循合理的折行和缩进规则：

- 在一个列表项的逗号分隔符后折行
  ```c++
  StringConfig stringConfigTable[] = {
      {"groupName",        "group001",        &groupName       },
      {"storagePaths",     "../data",         &storagePaths    },
      {"trackerAddresses", "127.0.0.1:21000", &trackerAddresses},
      {"idAddresses",      "127.0.0.1:22000", &idAddresses     },
      {"mysqlAddresses",   "127.0.0.1",       &mysqlAddresses  },
      {"redisAddresses",   "127.0.0.1:6379",  &redisAddresses  }};
  ```
  
- 在一个表达式的某个操作符后折行
  ```c++
  hash ^= (i & 1) ? ~((hash << 11) ^ buf[i] ^ (hash >> 5)) :
                     ((hash <<  7) ^ buf[i] ^ (hash >> 3));
  ```

- 除第一行外的所有后续行都采用统一的缩进
  ```c++
  logDebug("groupName: %s, storagePaths: %s, trackerAddresses: %s, "
      "idAddresses: %s, mysqlAddresses: %s, redisAddresses: %s, "
      "bindPort: %d, intervalSeconds: %d, mysqlTimeout: %d, "
      "maxConnections: %d, connTimeout: %d, rwTimeout: %d, "
      "keyTimeout: %d",
      groupName, storagePaths, trackerAddresses,
      idAddresses, mysqlAddresses, redisAddresses,
      bindPort, intervalSeconds, mysqlTimeout,
      maxConnections, connTimeout, rwTimeout,
      keyTimeout);
  ```

#### 2.6.2.8 操作符优先级、关联性和分组

操作符的优先级和关联性决定了其在表达式中被处理的顺序。

绝大多数操作符的关联性都是从左至右，但赋值操作符的关联性却是从右至左：

```
a = b = c
|   |___|
|_____|
```

先将c的值赋给b，再将b的赋值结果赋给a。整个表达式等价于下面的写法：

```
a = (b = c)
```

任何时候，但凡不确定复杂表达式中多个操作符的处理顺序，要么将其分解为多个由简单表达式构成的语句序列，要么通过圆括号将其划分为若干子表达式，显式指定处理顺序。