## 2.7 对象自然案例

在学会创建自己的类之前，首先应熟练使用现成的类，创建出对象，实现程序的功能，这就叫对象自然。之前的程序代码中已经使用了cout、cin这些对象，它们分别表示标准输出即屏幕和标准输入即键盘，用于输出和输入。这些对象是使用头文件\<iostream\>中的类创建的。类似地，头文件<string>中的string类可用于对字符串的处理，如下所示：

...\Ch02\StringTest\StringTest.cpp：

```c++
// StringTest.cpp
//
// 标准库string类测试
//
#include <iostream>
#include <string>
using namespace std;

int main() {
    string s1{ "你好" };
    string s2{ "世界" };
    string s3; // 空字符串

    // 显示字符串及其长度
    cout << "字符串\"" << s1 << "\"的长度为" << s1.length() << "字节\n"
         << "字符串\"" << s2 << "\"的长度为" << s2.length() << "字节\n"
         << "字符串\"" << s3 << "\"的长度为" << s3.length() << "字节\n";

    // 字符串的相等性比较
    cout << "字符串\"" << s1 << "\"和字符串\"" << s2 << (s1 == s2 ?
        "\"相等\n" : "\"不等\n");
    cout << "字符串\"" << s2 << "\"和字符串\"" << s3 << (s2 != s3 ?
        "\"不等\n" : "\"相等\n");

    // 字符串判空
    cout << "字符串\"" << s1 << (s1.empty() ?
        "\"是个空字符串\n" : "\"不是空字符串\n");
    cout << "字符串\"" << s2 << (s2.empty() ?
        "\"是个空字符串\n" : "\"不是空字符串\n");
    cout << "字符串\"" << s3 << (s3.empty() ?
        "\"是个空字符串\n" : "\"不是空字符串\n");

    // 字符串拼接与赋值
    s3 = s1 + "，" + s2 + "！";
    cout << "将字符串\"" << s1 << "\"、字符串\"" << s2 <<
        "\"及标点符号拼接成字符串\"" << s3 << "\"\n";

    // string类的其它成员函数
    cout << boolalpha;
    cout << "字符串\"" << s1 << "\"始于\"你\"：" <<
        s1.starts_with("你") << "\n";
    cout << "字符串\"" << s2 << "\"止于\"界\"：" <<
        s2.ends_with("界") << "\n";
    cout << "字符串\"" << s3 << "\"始于\"你好\"：" <<
        s3.starts_with("你好") << "\n";
    cout << "字符串\"" << s3 << "\"止于\"世界\"：" <<
        s3.ends_with("世界") << "\n";
}
```

编译运行：

```
字符串"你好"的长度为4字节
字符串"世界"的长度为4字节
字符串""的长度为0字节
字符串"你好"和字符串"世界"不等
字符串"世界"和字符串""不等
字符串"你好"不是空字符串
字符串"世界"不是空字符串
字符串""是个空字符串
将字符串"你好"、字符串"世界"及标点符号拼接成字符串"你好，世界！"
字符串"你好"始于"你"：true
字符串"世界"止于"界"：true
字符串"你好，世界！"始于"你好"：true
字符串"你好，世界！"止于"世界"：false
```

### 2.7.1 使用string类

类自己是不能执行的。main函数需要通过string类的对象，调用其成员函数来驱动它，同时并不需要知道string类的实现细节。这里的main函数称为驱动程序，它通过调用类中定义的成员函数，驱动该类的实例化对象。

```mermaid
graph TB
  cls(string类)
  obj(string类的对象)
  main(main函数)
  method(string类的成员函数)
  cls--实例化-->obj
  main--驱动-->obj--调用-->method
```

### 2.7.2 实例化对象

一般而言，除非创建了某个类的对象，否则不能调用该类的成员函数。为此，程序代码的第10、11和12行创建了三个string类的对象：

- s1：用字面值字符串“你好”初始化
- s2：用字面值字符串“世界”初始化
- s3：默认初始化为空字符串

之前在声明int型变量时，编译器知道int是什么，它是C++语言内置的基本数据类型。但在这段代码中，如果不事先告诉编译器，它是不知道string为何物的。为此，在程序代码的第6行，包含了名为\<string\>的头文件，其中包含了有关string类的定义。象C++这样的面向对象的编程语言，其最显著的优势之一就是提供了内容丰富且功能强大的类库。只要包含相应的头文件，就可以直接重用象string这样的，预定义在标准库中的类。作为类的名字string，就象对象的名字cout和cin一样，隶属于std名字空间。

### 2.7.3 string类的length成员函数

string类的length成员函数，返回一个特定字符串的长度。所谓字符串的长度，并非构成该字符串的字符的个数，而是该字串在内存中所占用的字节数。字符串的字符数和字节数并不总是相等的，这与具体采用的字符编码有关。

程序代码的第15、16和17行，通过在对象名，如s1、s2和s3后面加一个“.”操作符，再跟成员函数名，如length，以及被一对圆括号括起来的实参表的形式，完成了对string类length成员函数的调用。同时将该函数的返回值，即字符串的长度，通过“<<”操作符插入到代表标准输出，即屏幕的输出流对象cout中。整套动作的执行过程类似下面这个样子：

1. 程序的控制流从main函数的第15行跳转到string类length成员函数中；
2. 因为length是通过s1对象调用的，所以它知道所要处理的是哪个对象的数据；
3. length函数返回调用该函数的对象，即s1所表示字符串的长度，即字节数给main函数；
4. 就象司机并不需要知道汽车如何实现加速，只管踩下油门踏板一样，main函数也不需要知道length函数具体做了什么，只管调用它并接收它的返回值即可；
5. 最终length函数返回的s1字符串的长度，经由“<<”操作符被插入到cout对象中，显示在屏幕上；
6. 相似的过程同样也发生在s2和s3对象上。

很关键的一点，通过哪个对象调用该对象所属类的成员函数：

```c++
类 对象{ ... };
对象.成员函数(...);
```

该函数就在哪个对象的上下文中被执行：

```mermaid
graph LR
  caller(调用者)
  subgraph 类
    method(成员函数)
  end
  subgraph 对象
    context(上下文)
  end
  caller--调用-->method--执行-->context
```

### 2.7.4 字符串的相等性比较

和数字一样，string类型的字符串对象之间也可以进行关系比较。所有的关系操作符，包括“<”、“<=”、“>”、“>=”、“==”和“!=”，也都适用于string类的对象。程序代码的第20、22行，分别在字符串对象s1和s2之间，以及s2和s3之间，做了相等和不等性的比较。注意C++的字符串比较是严格区分大小写的。真正参与比较的，其实是构成字符串的每个字符的编码值。

### 2.7.5 string类的empty成员函数

程序代码的第26、28和30行，分别在字符串对象s1、s2和s3上调用了string类empty成员函数。该函数在字符串为空时返回true，字符串非空时返回false。

### 2.7.6 字符串拼接与赋值

程序代码的第34行通过“+”操作符，将字符串对象s1和s2，以及两个标点符号“，”和“！”，拼接成一个完整的字符串，赋值给字符串对象s3。第36行输出s3，验证拼接和赋值的正确性。

### 2.7.7 string类的starts_with和ends_with成员函数

程序代码的第41、43、45和47行通过string类的starts_with和ends_with成员函数，判断调用字符串（写在“.”前面的字符串）是否以参数字符串（写在圆括号中的字符串）开始或结束。这两个成员函数返回布尔型判断结果，true或false。

默认情况下，将布尔型数据插入到输出流中，显示出来的会是“1”和“0”，“1”代表真，“0”代表假。程序代码的第39行将来自头文件\<iostream\>的流控制符boolalpha插入cout中，意在告诉它用“true”和“false”表示真和假，不要再用“1”和“0”了。