## 3.2 控制结构

20世纪60年代，人们清楚地认识到，导致软件开发团队深陷困境的根源在于控制转移的滥用。人们将责任归咎于goto语句。当时大多数编程语言（包括C和C++）都提供了goto语句，它可以不受约束地将控制流转移到程序中的几乎任何位置。为了获得无限的自由，而不得不付出的代价和承担的风险，无疑是巨大的。

人们不禁要反思，在不使用任何goto语句的前提下，是否还能编写出高质量的程序？结构化编程的概念应运而生，它几乎成了“无goto编程”的同义词。理论和实践无可辩驳地证明，结构化编程不仅是可行的，而且往往意味着更短的开发周期、更经常的按时交付、更经济的成本投入。而这一系列的成功皆源自，结构化程序的逻辑更加清晰，调试和维护更加容易。最重要的是，它从一开始就规避了，可能诱使人们犯错的一切机会。

研究表明，所有程序都可以只使用三种基本控制结构编写，它们是：

- 顺序结构
- 选择结构
- 循环结构

### 3.2.1 顺序结构

顺序结构通常是一种默认结构，即一条语句一条语句，按照从上到下的顺序依次执行。如下图所示：

```mermaid
graph TB
  entrance((入口))
  s1(语句1)
  s2(语句2)
  s3(语句3)
  exit((出口))
  entrance-->s1-->s2-->s3-->exit
```

在任何可以放置一条语句的地方，都可以放置呈顺序结构的多条语句。

### 3.2.2 选择结构

#### 3.2.2.1 单选择结构

```mermaid
graph TB
  entrance((入口))
  cond{条件}
  yes(语句)
  exit((出口))
  entrance-->cond--满足-->yes-->exit
  cond--不满足-->exit
```

#### 3.2.2.2 双选择结构

```mermaid
graph TB
  entrance((入口))
  cond{条件}
  yes(语句1)
  no(语句2)
  exit((出口))
  entrance-->cond--满足-->yes-->exit
  cond--不满足-->no-->exit
```

#### 3.2.2.3 多选择结构

```mermaid
graph TB
  entrance((入口))
  switch{出现}
  case1(语句1)
  case2(语句2)
  case3(语句3)
  exit((出口))
  entrance-->switch--第一种情况-->case1-->exit
  switch--第二种情况-->case2-->exit
  switch--其它情况-->case3-->exit
```

### 3.2.3 循环结构

#### 3.2.3.1 前条件循环结构

```mermaid
graph TB
  entrance((入口))
  cond{条件}
  yes(语句)
  exit((出口))
  entrance-->cond--满足-->yes-->cond--不满足-->exit
```

#### 3.2.3.2 后条件循环结构

```mermaid
graph TB
  entrance((入口))
  yes(语句)
  cond{条件}
  exit((出口))
  entrance-->yes-->cond--满足-->yes
  cond--不满足-->exit
```

#### 3.2.3.3 三语句循环结构

```mermaid
graph TB
  entrance((入口))
  init(语句1)
  cond{条件}
  yes(语句2)
  next(语句3)
  exit((出口))
  entrance-->init-->cond--满足-->yes-->next-->cond--不满足-->exit
```

#### 3.2.3.4 范围循环结构

```mermaid
graph TB
  entrance((入口))
  range{依次获取范<br>围内的元素}
  yes(语句)
  exit((出口))
  entrance-->range--取到-->yes-->range--取不到-->exit
```

### 3.2.4 关键字

下面是C++关键字的完整列表：

关键字|含义
:-:|:-
alignas|设置内存对齐的字节数
alignof|获取内存对齐的字节数
and|逻辑与，等价于“&&”
and_eq|位与并赋值，等价于“&=”
asm|嵌入汇编代码
auto|自动变量（C++11后已弃用）<br/>根据变量的值推导其类型
bitand|位与，等价于“&”
bitor|位或，等价于“\|”
bool|布尔类型
break|跳出循环或开关分支结构
case|开关分支语句的一个分支
catch|捕获异常
char|字符类型
char8_t|单字节字符类型
char16_t|双字节字符类型
char32_t|四字节字符类型
class|类类型<br/>结构体类型<br/>有作用域的枚举类型<br/>模板的类型参数
co_await|挂起协程直到恢复
co_return|结束协程并返回值
co_yield|挂起协程并返回值
compl|按位取反，等价于“~”
concept|概念，用于约束模板参数的类型
const|常量类型限定
const_cast|去除指针或引用的常量类型限定
consteval|函数的返回值可在常量表达式中使用
constexpr|变量的值或函数的返回值可在常量表达式中使用
constinit|变量必须在编译期被静态初始化
continue|终止本次循环，开始下一次循环
decltype|获取表达式的数据类型
default|开关分支语句的默认分支<br/>为类的特殊成员函数和比较运算符函数显式指定预置定义
delete|销毁先前由new创建的对象，并释放其内存区域<br/>弃置函数
do|do-while循环语句的入口
double|双精度浮点数类型
dynamic_cast|将基类类型的指针或引用转换为其具有多态性的子类类型的指针或引用
else|条件分支语句的否定分支
enum|枚举类型
explicit|指定类型转换构造函数、类型转换操作符函数或推导指引是否必须显式使用
export|从当前模块中导出
extern|外部链接特性<br/>语言链接特性<br/>显式模板实例化
false|逻辑假字面值
final|禁止类被继承<br/>禁止基类中的成员函数（包括虚函数）在子类中被重写
float|单精度浮点数类型
for|for循环语句的入口
friend|友元类及友元函数
goto|无条件控制转移
if|条件分支语句的肯定分支
import|导入模块
inline|内联函数
int|整数类型
long|长整数类型<br/>修饰int、long和double
module|模块
mutable|指定类的成员变量可在该类带有常限定的成员函数中被修改<br/>去除按值捕获外部变量的Lambda表达式的小括号操作符函数的常限定
namespace|名字空间
new|动态分配内存并在其中创建对象
noexcept|声明一个函数是否抛出异常<br/>在编译期检查给定表达式是否抛出异常
not|逻辑非，等价于“!”
not_eq|不等于，等价于“!=”
nullptr|空指针字面值
operator|操作符函数
or|逻辑或，等价于“\|\|”
or_eq|位或并赋值，等价于“\|=”
override|覆盖（重写）基类中的虚函数
private|私有成员<br/>私有继承<br/>私有模块片段
protected|保护成员<br/>保护继承
public|公有成员<br/>公有继承
register|寄存器变量（C++17后已弃用）
reinterpret_cast|在不同类型的指针或引用之间，或在指针和整数之间转换
requires|约束模板类型参数的可取类型
return|从函数中返回
short|短整数类型<br/>修饰int
signed|有符号整数类型<br/>修饰char、short、int、long和long long
sizeof|获取类型的字节数<br/>获取形参包中的元素数
static|静态全局变量和静态全局函数<br/>静态局部变量<br/>静态成员变量和静态成员函数
static_assert|编译时断言检查
static_cast|不做运行时检查的隐式类型转换和自定义类型转换
struct|类类型<br/>结构体类型<br/>有作用域的枚举类型
switch|开关分支语句的入口
template|函数模板或类模板<br/>模板型模板参数
this|this指针<br/>显式对象形参
thread_local|线程局域存储
throw|抛出异常<br/>异常说明
true|逻辑真字面值
try|异常处理结构入口
typedef|定义类型别名
typeid|获取类型信息
typename|模板的类型参数<br/>模板内部的待决有限定名类型
union|联合体类型
unsigned|无符号整数类型<br/>修饰char、short、int、long和long long
using|名字空间指令<br/>名字空间声明<br/>引入枚举类型<br/>定义类型别名<br/>定义模板别名
void|空类型，即值为空集的类型<br/>无形参函数的形参表<br/>无返回值函数的返回值类型
volatile|易变类型限定
virtual|虚函数<br/>虚基类
wchar_t|宽字符类型
while|while循环语句的入口<br/>do-while循环语句的出口
xor|异或，等价于“^”
xor_eq|异或并赋值，等价于“^=”

### 3.2.5 控制语句小结

与控制结构相对应的是各种编程语言中的控制语句，比如C++语言的控制语句：

控制结构|控制语句
:-:|:-:
顺序结构|默认
单选择结构|if条件语句
双选择结构|if-else条件分支语句
多选择结构|switch-case开关分支语句
前条件循环结构|while循环语句
后条件循环结构|do-while循环语句
三语句循环结构|for循环语句
范围循环结构|基于范围的for循环语句

每种控制语句都只有一个入口和一个出口，因此可以借助控制语句堆叠技术，即将一个控制语句的出口与另一个控制语句的入口相连，构建出可读性好的程序。例如：

```mermaid
graph TB
  subgraph 单选择结构
    entrance1((入口))
    cond1{条件}
    yes1(语句)
    exit1((出口))
    entrance1-->cond1--满足-->yes1-->exit1
    cond1--不满足-->exit1
  end
  subgraph 前条件循环结构
    entrance2((入口))
    cond2{条件}
    yes2(语句)
    exit2((出口))
    entrance2-->cond2--满足-->yes2-->cond2--不满足-->exit2
  end
  exit1-->entrance2
```

另外，也可以将一个控制语句嵌套在另一个控制语句中。例如：

```mermaid
graph LR
  subgraph 单选择结构
    entrance1((入口))
    cond1{条件}
    subgraph 前条件循环结构
      entrance2((入口))
      cond2{条件}
      yes2(语句)
      exit2((出口))
      entrance2-->cond2--满足-->yes2-->cond2--不满足-->exit2
    end
    exit1((出口))
    entrance1-->cond1--满足-->entrance2
    exit2-->exit1
    cond1--不满足-->exit1
  end
```

由此可见，C++语言仅通过顺序、选择和循环三类控制语句，堆叠和嵌套两种组合方式，就足以实现任意复杂的算法逻辑，这就叫大道至简。