## 3.4 if-else条件分支语句

借助if-else条件分支语句可以在条件满足或不满足时执行不同的语句。例如：

```c++
if (studentGrade >= 60) {
	cout << "及格" << endl;
}
else {
    cout << "不及格" << endl;
}
```

如果某学生的考试成绩大于等于60分，if后面关系表达式的值为真，条件满足，那么其后花括号中的语句就会被执行，打印“及格”。这种情况下，else后面花括号中的语句是不执行的。相反，如果某学生的考试成绩小于60分，if后面关系表达式的值为假，条件不满足，那么其后花括号中的语句就不执行，else后面花括号中的语句被执行，打印“不及格”。执行过程如下图所示：

```mermaid
graph TB
  entrance(入口)
  cond{成绩不低于60}
  yes(打印“及格”)
  no(打印“不及格”)
  exit(出口)
  entrance-->cond--是-->yes-->exit
  cond--否-->no-->exit
```

if-else条件分支语句是一种单入单出的控制语句。

### 3.4.1 嵌套的if-else语句

可以在if-else语句的if分支和（或）else分支中再嵌套if-else语句，以产生更多的条件分支。例如：

```c++
if (studentGrade >= 90) {
    cout << "优" << endl;
}
else {
    if (studentGrade >= 80) {
        cout << "良" << endl;
    }
    else {
        if (studentGrade >= 70) {
            cout << "中" << endl;
        }
        else {
            if (studentGrade >= 60) {
                cout << "差" << endl;
            }
            else {
                cout << "劣" << endl;
            }
        }
    }
}
```

程序根据学生所处的分数段，打印不同的评定级别：

- 90分和90分以上，包括100分：优
- 80分和80分以上，不到90分：良
- 70分和70分以上，不到80分：中
- 60分和60分以上，不到70分：差
- 60分以下：劣

借助“else if”，既可以减少花括号的数量，又可以避免嵌套过深。这段代码的等价写法如下所示：

```c++
if (studentGrade >= 90) {
    cout << "优" << endl;
}
else if (studentGrade >= 80) {
    cout << "良" << endl;
}
else if (studentGrade >= 70) {
    cout << "中" << endl;
}
else if (studentGrade >= 60) {
    cout << "差" << endl;
}
else {
    cout << "劣" << endl;
}
```

如果if、else if、else后面的花括号中只包含一条语句，则可以省略花括号。例如：

```c++
if (studentGrade >= 90)
    cout << "优" << endl;
else if (studentGrade >= 80)
    cout << "良" << endl;
else if (studentGrade >= 70)
    cout << "中" << endl;
else if (studentGrade >= 60)
    cout << "差" << endl;
else
    cout << "劣" << endl;
```

但这种写法极易导致所谓“悬空else”的语法错误。例如：

```c++
if (studentGrade >= 90)
    cout << "优" << endl;
    ++goodStudents;
else if (studentGrade >= 80)
    cout << "良" << endl;
else if (studentGrade >= 70)
    cout << "中" << endl;
else if (studentGrade >= 60)
    cout << "差" << endl;
else
    cout << "劣" << endl;
```

增加了第3行，该行虽然使用了缩进，但它不但不属于之前的if分支，而且打破了整个if-else if-else的整体结构，使得第4行的else if因为没有与之匹配的if而导致编译失败。这就是所谓的“悬空else”错误。因此，强烈建议在所有的if、else if、else后面都使用花括号，即使其中只包含一条语句。

### 3.4.2 代码块

被一对花括号括起来的一到多条语句构成一个代码块，简称块。一般而言，能放得下一条语句的地方，就能放得下一个语句块。例如：

```c++
if (studentGrade >= 60) {
	cout << "及格" << endl;
    passes = passes + 1;
}
else {
    cout << "不及格" << endl;
    failures = failures + 1;
}
```

再次强调，上述代码中if分支的花括号绝对不可以省略，否则将导致“悬空else”语法错误。else分支的花括号如果不写，虽然不会导致编译错误，但是第7行的代码会跑到if-else结构之外，无论学生的考试成绩是否大于等于60都会执行，这显然是一个逻辑错误。由此可见，作为代码块边界的花括号非常重要。

### 3.4.3 空语句

什么都不做的语句就叫空语句，也叫空操作。一般而言，任何能放单条语句的地方，都能放空语句。例如：

```c++
if (studentGrade >= 60) {
    ;
}
else {
    failures = failures + 1;
}
```

或者

```c++
if (studentGrade >= 60) {
}
else {
    failures = failures + 1;
}
```

在参加考试的学生总数和不及格人数已知的前提下，及格人数完全可以计算而得，没必要单独统计。因此这里的if分支使用了空语句。

### 3.4.4 条件操作符

条件操作符是C++语言中，唯一的三元，即带有三个操作数的操作符。它可以表示与if-else语句相同的逻辑，但具有更短、更清晰的优点。条件操作符连同一个条件和两个操作数，共同组成了形如“条件 ? 操作数1 : 操作数2”的条件表达式。条件表达式的值取决于条件是否得到满足，满足取操作数1的值，不满足取操作数2的值，如下所示：

```mermaid
graph LR
  expr((表达式取))
  cond{条件}
  yes(操作数1)
  no(操作数2)
  value((的值))
  expr-->cond--满足-->yes-->value
  cond--不满足-->no-->value
```

例如下面这条输出语句：

```c++
cout << (studentGrade >= 60 ? "及格" : "不及格") << endl;
```

插入到标准输出流的是一个条件表达式的值，该值在某学生的考试成绩大于等于60分时，为字符串“及格”，在某学生的考试成绩小于60分时，为字符串“不及格”。注意，条件操作符的优先级较低，最好将整个条件表达式用圆括号括起来。