## 3.6 计数器控制的循环

问题描述：一个班级里的10个学生参加一次考试。要求录入每个学生的考试成绩，并打印整个班级的平均成绩。

为了解决这个问题，可以使用在计数器控制下的循环，每次循环录入一个学生的考试成绩，一共循环10次。计数器的作用就在于控制一组语句的执行次数。

### 3.6.1 实现计数器控制的循环

在下面的程序中，main函数用一个在计数器控制下的循环来输入每一个学生的考试成绩，同时计算班级总分。在处理完10个学生的考试成绩后，计算班级平均分并打印出总分和平均分。

...\Ch03\CounterLoop\CounterLoop.cpp：

```c++
// CounterLoop.cpp
//
// 借助计数器控制的循环求解平均值问题
//
#include <iostream>
using namespace std;

int main() {
    // 初始化阶段
    int total{ 0 }; // 班级总分
    int gradeCounter{ 0 }; // 成绩计数器

    // 处理阶段，在计数器控制下的循环
    while (gradeCounter < 10) { // 循环10次
        cout << "输入第" << gradeCounter + 1 << "位同学的考试成绩："; // 提示用户输入
        int grade;
        cin >> grade; // 读取用户输入的考试成绩
        total = total + grade; // 将考试成绩累加到班级总分中
        gradeCounter = gradeCounter + 1; // 成绩计数器加一
    }

    // 最后阶段
    int average{ total / gradeCounter }; // 计算班级平均分

    // 输出班级总分和平均分
    cout << "\n班级总分：" << total << "\n";
    cout << "班级平均分：" << average << "\n";
}
```

编译运行：

```
输入第1位同学的考试成绩：90
输入第2位同学的考试成绩：80
输入第3位同学的考试成绩：100
输入第4位同学的考试成绩：77
输入第5位同学的考试成绩：85
输入第6位同学的考试成绩：92
输入第7位同学的考试成绩：88
输入第8位同学的考试成绩：72
输入第9位同学的考试成绩：60
输入第10位同学的考试成绩：79

班级总分：823
班级平均分：82
```

#### 3.6.1.1 main函数中的局部变量

程序代码的第10、11、16和23行分别声明int类型的变量total、gradeCounter、grade和average。变量grade用于存储用户输入的一个学生的考试成绩。所有在代码块中声明的变量都是局部变量，其作用域从声明的位置开始，到代码块的右花括号为止。局部变量必须先声明后使用。在while循环语句的循环体代码块中声明的局部变量grade，只能在这个代码块中使用。

#### 3.6.1.2 初始化total和gradeCounter变量

程序代码的第10和第11行声明了变量total和gradeCounter，并初始化为0，分别用于存储班级总分和成绩计数值。注意，这些变量必须先声明并初始化，然后才能在后面的代码中使用。

#### 3.6.1.3 从用户处读取10个考试成绩

只要gradeCounter的值小于10，从第14到20行的while循环语句，就会一直不断重复执行循环体中的代码。在打印提示信息后，将用户通过键盘输入的考试成绩读取到grade变量中，随即将其累加到表示班级总分的total变量里。在完成这些操作后，令成绩计数器gradeCounter的值增加1，表示程序已经处理了gradeCounter个考试成绩。gradeCounter中的值不断递增，当处理完第10个考试成绩后，它的值也将达到10。在接下来对循环是否继续的条件判断中，因gradeCounter<10为假，而退出循环。

#### 3.6.1.4 计算并输出班级平均分

循环退出后，程序代码的第23行，利用前面得到的班级总分total和成绩计数值gradeCounter，二者相除得到班级平均分，保存到average变量中。在程序代码的第26和27行输出整个班级的总分和平均分。

### 3.6.2 整数除法和截断

在有关班级平均分的计算中：

```c++
int average{ total / gradeCounter }; // 计算班级平均分
```

total和gradeCounter都是整数类型，因此它们相除的商值也是整数。在 C++语言中，两个整数相除的运算称为整数除法，其计算结果中的任何小数部分都会被舍弃。即使是象$7\div 4$这样的常规算术，计算的结果也只能是$1$，而不是$1.75$或四舍五入后的$2$。