## 3.8 嵌套控制语句

C++语言的控制语句有堆叠和嵌套两种组合方式。而所谓嵌套，就是将一个控制语句嵌套在另一个控制语句中。

### 3.8.1 问题描述

10个学生在完成一个阶段课程的学习后被要求参加阶段考试。现在需要知道一共有多少个学生通过了该门考试，又有多少学生没有通过。如果通过考试的学生超过8人，则给任课教师发奖金。

- 在一个被计数器控制的循环中录入每个学生的考试情况，1表示通过考试，2表示没有通过
- 在循环体中根据学生的考试情况，分别统计通过考试和没通过考试的学生人数
- 在循环退出时，输出统计结果，并决定是否给任课教师发奖金

流程如下图所示：

```mermaid
graph TB
  a((开始))
  b{处理的学生数<br/>是否到达10人}
  c(输入考试情况)
  d{是否通过考试}
  e(通过人数加一)
  f(未通过人数加一)
  g(处理的学生数加一)
  h(输出统计结果)
  i{通过考试的学<br/>生是否超过8人}
  j(给任课教师发奖金)
  k((结束))
  a-->b--没达到-->c-->d--通过-->e-->g
  d--没通过-->f-->g-->b--达到-->h-->i--超过-->j-->k
  i--没超过-->k
```

### 3.8.2 程序实现

...\Ch03\Nesting\Nesting.cpp：

```c++
// Nesting.cpp
//
// 基于嵌套控制语句的考试情况统计
//
#include <iostream>
using namespace std;

int main() {
    // 声明变量并初始化
    int passes{ 0 }; // 通过考试的人数
    int failures{ 0 }; // 未通过考试的人数
    int studentCounter{ 0 }; // 学生计数器

    // 在计数器控制的循环中处理10个学生的考试情况
    while (studentCounter < 10) {
        // 提示用户输入并获取其输入的数据
        cout << "输入第" << studentCounter + 1
             << "位同学的考试情况（1=通过，2=未通过）：";
        int result;
        cin >> result;

        // 嵌套在while循环语句中的if-else条件分支语句
        if (result == 1) {
            passes = passes + 1; // 通过考试的人数加一
        }
        else {
            failures = failures + 1; // 未通过考试的人数加一
        }

        studentCounter = studentCounter + 1; // 学生计数器加一
    }

    // 输出统计结果
    cout << "\n共有" << passes << "位同学通过考试，"
         << failures << "位同学没有通过考试。\n";

    // 决定是否给任课教师发奖金
    if (passes > 8) {
        cout << "\n建议给任课教师发奖金！\n";
    }
}
```

编译运行：

```
输入第1位同学的考试情况（1=通过，2=未通过）：1
输入第2位同学的考试情况（1=通过，2=未通过）：1
输入第3位同学的考试情况（1=通过，2=未通过）：1
输入第4位同学的考试情况（1=通过，2=未通过）：1
输入第5位同学的考试情况（1=通过，2=未通过）：2
输入第6位同学的考试情况（1=通过，2=未通过）：1
输入第7位同学的考试情况（1=通过，2=未通过）：1
输入第8位同学的考试情况（1=通过，2=未通过）：1
输入第9位同学的考试情况（1=通过，2=未通过）：1
输入第10位同学的考试情况（1=通过，2=未通过）：1

共有9位同学通过考试，1位同学没有通过考试。

建议给任课教师发奖金！
```

程序代码的第10到12行以及第19行，声明了用于统计考试情况的四个局部变量。第15到31行的while循环语句，在学生计数器studentCounter的控制下，重复执行了10次循环体中的代码块。每次执行均输入并处理了一个学生的考试情况。针对每个学生，用一个if-else条件分支语句，根据其是否通过了考试，分别在第24和第27行，对通过考试的人数passes和未通过考试的人数failures做递增。这个if-else条件分支语句是嵌套在while循环语句中的。在第15行重新测试继续循环的条件之前，先在第30行对学生计数器studentCounter中的值做递增。

在输入10个值后，循环终止。第34和35行输出了针对考试情况的统计结果。第38到40行则借助if条件语句，判断通过考试的学生是否超过8人。确定超过8人，则输出“建议给任课教师发奖金！”消息。

### 3.8.3 用花括号初始化防止收缩转换

程序代码的第12行是这样声明并初始化变量的：

```c++
int studentCounter{ 0 }; // 学生计数器
```

等价的写法，在旧式语法中是下面这个这样子：

```c++
int studentCounter = 1; // 学生计数器
```

其实二者还是有区别的。对于基本类型的变量，使用花括号初始化，能够防止可能导致数据丢失的收缩转换。例如按照旧式写法：

```c++
int x = 12.7;
```

它会将double型字面值常量12.7的小数部分截断，取其整数部分12用于对int型变量x的初始化。这就是一种导致数据丢失的收缩转换。对于这种情况，编译器通常会给出一个警告，但仍然可以通过编译。而如果采用新式的写法：

```c++
int x{ 12.7 };
```

将引发编译错误，这有助于规避一个潜在的，不易察觉的逻辑错误。即使是下面的代码：

```c++
int x{ 12.0 };
```

同样会引发编译错误。编译器报错的依据是初始化器的类型double，而非其值12.0。

在之前的程序代码中曾经写过：

```c++
int average{ total / gradeCounter }; // 计算班级平均分
```

这里看似发生了收缩转换，其实不然。因为total和gradeCounter都是int类型，整数除法的商值也是int类型，所以整个初始化器的类型为int，初始化int类型的average并不会发生收缩转换，编译器也不会报告错误。