## 3.10 递增递减操作符

递增操作符“++”和递减操作符“--”都是一元操作符，作用是令其唯一的操作数增加一或减少一，但其前缀用法和后缀用法在语义上存在微妙的差别。如下表所示：

操作符名称|示例表达式|解释
:-:|:-:|:-
前缀递增操作符（前加加）|`++a`|先使变量a的值增加一，再在a所在的表达式中使用a的增后值
后缀递增操作符（后加加）|`a++`|先在变量a所在的表达式中使用a的当前值，再使a的值增加一
前缀递减操作符（前减减）|`--a`|先使变量a的值减少一，再在a所在的表达式中使用a的减后值
后缀递减操作符（后减减）|`a--`|先在变量a所在的表达式中使用a的当前值，再使a的值减少一

递增递减操作符的优先级通常是最高的。

与二元操作符的书写习惯不同，一元操作符和操作数之间通常不留空格。

### 3.10.1 对比前缀递增操作符和后缀递增操作符

下面的代码演示了前缀递增操作符和后缀递增操作符的区别。

...\Ch03\Increment\Increment.cpp：

```c++
// Increment.cpp
//
// 前缀递增操作符和后缀递增操作符
//
#include <iostream>
using namespace std;

int main() {
    // 前缀递增操作符
    int prefix = { 5 };
    cout << "变量在递增之前的值：" <<   prefix << endl; // 5
    cout << "前缀递增表达式的值：" << ++prefix << endl; // 6
    cout << "变量在递增之后的值：" <<   prefix << endl; // 6                     

    cout << endl;

    // 后缀递增操作符
    int postfix{ 5 };
    cout << "变量在递增之前的值：" << postfix   << "\n"; // 5
    cout << "后缀递增表达式的值：" << postfix++ << "\n"; // 5
    cout << "变量在递增之后的值：" << postfix   << "\n"; // 6                     
}
```

编译运行：

```
变量在递增之前的值：5
前缀递增表达式的值：6
变量在递增之后的值：6

变量在递增之前的值：5
后缀递增表达式的值：5
变量在递增之后的值：6
```

从这段程序代码及其运行结果不难看出，无论是前缀递增操作符，还是后缀递增操作符，作为操作数的变量，其值在递增后肯定比在递增前多一。二者的区别在于递增表达式本身的值，是取操作数变量的增后值还是取它的增前值。前缀递增取增后值，后缀递增取增前值。

### 3.10.2 用算术复合赋值操作符和递增递减操作符简化语句
之前程序代码中的如下语句：

```c++
passes = passes + 1;
failures = failures + 1;
studentCounter = studentCounter + 1;
```

可用算术复合赋值操作符简化为：

```c++
passes += 1;
failures += 1;
studentCounter += 1;
```

甚至可用前缀递增操作符进一步简化为：

```c++
++passes;
++failures;
++studentCounter;
```

当然也可以使用后缀递增操作符：

```c++
passes++;
failures++;
studentCounter++;
```

如果变量的递增递减操作是一条独立的语句，那么使用它的前缀或后缀版本，在效果上并没有什么区别。只有当变量的递增递减操作被置于一个更大的表达式中时，它前缀和后缀版本才会产生不同的结果。

任何试图在除可被赋值的表达式以外的表达式上，使用递增递减操作符，都会导致编译错误。例如：

```c++
++(x + 1);
```

因为`(x + 1)`不可被赋值，所以`++(x + 1)`将引发编译错误。

### 3.10.3 操作符的优先级和关联性

下表总结了到目前为止介绍的所有操作符的优先级和关联性。从上到下操作符的优先级依次递减：

操作符|关联性
:-:|:-:
++（后缀）、--（后缀）、static_cast\<类型>()|从左到右
++（前缀）、--（前缀）、+（正号）、-（负号）|从右到左
*、/、%|从左到右
+、-|从左到右
<<、>>|从左到右
<、<=、>、>=|从左到右
==、!=|从左到右
? :|从右到左
=、+=、-=、*=、/=、%=|从右到左

其中，只有前缀递增递减操作符、正负号操作符、条件操作符，以及赋值和复合赋值操作符的关联性是从右到左，其它操作符的关联性都是从左到右。