## 3.13 用format函数格式化文本

C++20通过声明在\<format\>头文件中的format函数，提供强大的字符串格式化功能，以更接近于Python、C#、Visual Basic甚或Rust的风格，大幅简化了旧式C++语言中的文本格式化语法。

### 3.13.1 C++20的字符串格式化尚未完全实现

截止目前，只有Visual C++实现了完整的C++20的文本格式化功能。其它编译器需要借助第三方的fmt库使用这部分功能。该库已在GitHub（https://github.com/fmtlib/fmt）上开源。

### 3.13.2 格式字符串占位符

format函数的第一个参数是格式字符串，包含一系列“{}”形式的占位符。该函数用后面的参数替换对应的占位符，并返回格式化好的字符串。

...\Ch03\UseFmt\UseFmt.cpp：

```c++
// UseFmt.cpp
//
// C++20的字符串格式化
//
#include <iostream>
#include <format>
using namespace std;

int main() {
    string name{ "张飞" };
    int age{ 22 };
    float grade{ 87.5 };

    cout << format("{}同学今年{}岁，考了{}分。\n", name, age, grade);
}
```

编译运行：

```
张飞同学今年22岁，考了87.5分。
```

### 3.13.3 占位符从左至右替换

```
format("{}同学今年{}岁，考了{}分。\n", name, age, grade) -> 张飞同学今年22岁，考了87.5分。
         ^       ^        ^          |     |     |
         |_______|________|__________|     |     |
                 |________|________________|     |
                          |______________________|
```

格式字符串中的占位符按照从左至右的顺序，被后面的参数依次替换。