## 4.3 for循环语句

for循环语句用一行代码囊括了计数器控制的循环的所有要素。

...\Ch04\CounterFor\CounterFor.cpp：

```c++
// CounterFor.cpp
//
// 计数器控制下的for循环语句
//
#include <iostream>
using namespace std;

int main() {
    // 在一行代码中囊括了计数器控制的循环的所有要素：
    // 1. 声明并初始化用于控制循环次数的计数器
    // 2. 根据计数器的值决定循环是否还要继续
    // 3. 对计数器做累加，记录循环体的执行次数
    for (int counter{ 0 }; counter < 10; ++counter) {
        cout << counter << " ";
    }

    cout << endl;
}
```

编译运行：

```
0 1 2 3 4 5 6 7 8 9
```

程序代码的第13到15行是一个典型的，用for循环语句实现的，在计数器控制下的循环过程。循环伊始，先声明了用于控制循环次数的计数器变量counter，并将其值初始化为0；接着通过对条件counter是否小于10的判断，决定循环是否还要继续；如果条件满足则执行花括号中的循环体，并对计数器的值做累加；然后再次判断继续循环的条件，满足则执行循环体，不满足则退出循环。

### 4.3.1 剖析for循环语句头

```c++
for (int counter{ 0 }; counter < 10; ++counter)
 |          |     |  |       |     |     |_____ 计数器累加
 |          |     |  |       |     |___________ 必须写分号
 |          |     |  |       |_________________ 循环继续条件
 |          |     |  |_________________________ 必须写分号
 |          |     |____________________________ 计数器初值
 |          |__________________________________ 计数器变量
 |_____________________________________________ for循环关键字
```

for循环语句头承包了计数器控制的循环的所有要素。

### 4.3.2 for循环语句的常规格式

```c++
for (初始化语句; 循环继续条件; 递增语句) {
    循环体语句;
}
```

```mermaid
graph TB
  entrance((入口))
  init(初始化语句)
  cond(循环继续条件)
  increment(递增语句)
  body(循环体语句)
  exit((出口))
  entrance-->init-->cond--满足-->body-->increment-->cond--不满足-->exit
```

for循环语句的几点注意事项：

- 初始化语句仅在循环开始时执行一次，以后每次循环不再执行
- 递增语句在每次执行完循环体语句之后执行，随即判断循环继续条件
- 对循环继续条件的判断发生在意图执行循环体语句之前，根据判断结果决定是执行循环体还是退出循环
- 如果循环继续条件从一开始就不满足，那么循环体语句一次都不会执行
- 退出循环即执行for循环语句后的第一条语句

### 4.3.3 for循环语句控制变量的作用域

for循环语句的控制变量，如果在初始化语句中被声明，那么它的作用域仅限于该for循环语句。一个变量的作用域决定了它的寿命和可被使用的区域。将变量限制在尽可能小的作用域中是一个好的做法。

### 4.3.4 for循环语句头中的子语句是可选的

for循环语句头中的三个子语句都是可选的：

- 如果控制循环的变量已在for循环语句之前被声明和初始化，则可以省略初始化语句
- 如果循环永不终止，或可以在循环体中决定是否终止，则可以省略循环继续条件
- 如果控制循环的变量已在循环体中实现递增，则可以省略递增语句
- 无论省略哪个子语句，作为子语句分隔符的分号都不能省略

例如：

```c++
int counter{ 0 };
for (;;) {
    if (counter >= 10)
        break;
    cout << counter << " ";
    ++counter;
}
```

for循环语句头中的递增语句，通常由一个独立的递增表达式构成。这种情况下，使用前缀递增操作符和使用后缀递增操作符，在效果上并没有什么不同，只是使用前缀递增操作符的运行性能会更具优势。

### 4.3.5 在for循环语句的循环体中使用控制变量

在for循环语句的循环体中经常需要显示控制变量的值，或引用该值参与某种运算，但这并不是必要的。在for循环语句的循环体中也可以改变控制变量的值，但这样做极易导致不易察觉的错误。如果一定要在循环体内部修改控制变量的值，最好使用while循环而非for循环。

### 4.3.6 for循环语句的执行顺序

```
初始化语句-->循环继续条件-->循环体语句-->递增语句
               ^                       |
               |_______________________|
```

初始化语句仅在首次测试循环继续条件前执行一次，而递增语句则在每次执行完循环体语句后都要执行。