## 4.6 复利计算

将本金$p$存入银行，年利率为$r$，$n$年后的账户余额：$a_n=a_{n-1}+a_{n-1}r=a_{n-1}(1+r)$，$a_0=p$

...\Ch04\CompoundInterest\CompoundInterest.cpp：

```c++
// CompoundInterest.cpp
//
// 基于for循环的复利计算
//
#include <iostream>
#include <iomanip>
using namespace std;

int main() {
    // 输入本金、利率和存期
    cout << "输入本金：";
    double principal;
    cin >> principal;
    cout << "输入利率：";
    double rate;
    cin >> rate;
    cout << "输入存期：";
    int years;
    cin >> years;

    cout << endl;

    // 打印表头
    cout << "年份" << setw(20) << "账户余额" << endl;
    // 设置浮点数输出格式
    cout << fixed << setprecision(2);

    // 计算十年中每一年的账户余额
    double amount{ principal }; // 初始余额即本金
    for (int year{ 1 }; year <= years; ++year) {
        // 当年余额 = 往年余额 x (1 + 利率)
        amount *= (1 + rate);
        // 打印年份和账户余额
        cout << setw(4) << year << setw(20) << amount << endl;
    }
}
```

编译运行：

```
输入本金：1000
输入利率：0.05
输入存期：10

年份            账户余额
  1            1050.00
  2            1102.50
  3            1157.62
  4            1215.51
  5            1276.28
  6            1340.10
  7            1407.10
  8            1477.46
  9            1551.33
 10            1628.89
```

程序代码的第11到19行提示用户输入并读取用户输入的本金、利率和存期。在打印表头并将浮点数的输出格式设置为两位定点小数后，在第29行声明了表示账户余额的double型变量amount，其初值即为本金，$a_0=p$。程序代码的第30到35行是一个用for循环语句实现的计数器控制的循环，控制该循环次数的计数器为表示年份的变量year。该变量从1累加到用户输入的年份。在循环体中计算并打印第year年的账户余额。计算公式为：$a_n=a_{n-1}(1+r)$。

### 4.6.1 用域宽和对齐来格式化

为了显示精确到分的定点小数形式的账户余额，这里使用了流控制符fixed和setprecision，同时为了将年份和账户余额按列对齐，使用了流控制符setw。setw是一个带有参数的流控制符，其参数表示下一个输出值所占用的域宽。域宽用字符数表示。例如：

```c++
int a{ 12 };
cout << "[" << setw(4) << a << "]";
```

显示效果为：

```
[  12]
```

变量a的值占用4字符域宽。默认采用右对齐，不足域宽左补空格。对齐方式也可以通过流控制符人为设置。例如：

```c++
int a{ 12 };
cout << "[" << left  << setw(4) << a << "]"; // 左对齐
cout << "[" << right << setw(4) << a << "]"; // 右对齐
```

显示效果为：

```
[12  ]
[  12]
```

如果所输出数据的字符数超过指定域宽，则将域宽向右扩展，以容纳下整个数据。例如：

```c++
int a{ 123456 };
cout << "[" << setw(4) << a << "]";
```

显示效果为：

```
[123456]
```

注意，setw是唯一没有粘性的流控制符，只影响紧随其后的输出数据。例如：

```c++
int a{ 12 }, b{ 34 };
cout << "[" << setw(4) << a << "][" << b << "]";
```

显示效果为：

```
[  12][34]
```

如果希望b也象a一样占用4字符域宽，必须在向输出流对象插入b之前，再次插入setw(4)流控制符。例如：

```c++
int a{ 12 }, b{ 34 };
cout << "[" << setw(4) << a << "][" << setw(4) << b << "]";
```

显示效果为：

```
[  12][  34]
```

### 4.6.2 用标准库函数pow计算利息

进一步推导复利计算公式：

$$
\begin{align}
a_n
&=a_{n-1}(1+r) \\
&=a_{n-2}(1+r)(1+r) \\
&=a_{n-3}(1+r)(1+r)(1+r) \\
&=\cdots \\
&=a_0\underbrace{(1+r)(1+r)\cdots(1+r)}_n \\
&=a_0(1+r)^n\quad(a_0=p) \\
&=p(1+r)^n
\end{align}
$$

计算并打印账户余额的for循环语句也可以写成这样：

```c++
for (int year{ 1 }; year <= years; ++year) {
    // 余额 = 本金 x (1 + 利率) ^ 年份
    double amount = principal * pow(1 + rate, year);
    // 打印年份和账户余额
    cout << setw(4) << year << setw(20) << amount << endl;
}
```

C++语言没有内置的幂运算操作符。标准库函数`double pow(double x, double y)`用于计算$x^y$。这里使用该函数计算$(1+rate)^{year}$即公式中$(1+r)^n$，再将其与公式中表示为$p$的本金$principal$相乘，即得到该年的账户余额。

注意，这里调用pow函数的第一个参数1+rate，在每次执行循环体的过程中始终保持不变。出于性能优化的考虑，如今大多数编译器都会在编译后的代码中，将该计算提到进入循环之前。

### 4.6.3 浮点数精度与内存需求

单精度浮点数类型float，在大多数系统上占用4字节内存，其中约有7个有效数字。双精度浮点数类型double，在大多数系统上占用8字节内存，其中约有15个有效数字，是float类型有效数字的两倍，故名双精度浮点数。多数情况下都会使用double类型的浮点数。C++语言中的浮点数字面值，如3.14159，默认也是double类型的。

事实上C++标准并没有规定各种浮点数类型所占用内存的大小，只是规定了double类型至少具备和float类型一样的精度。另外还有一种long double类型的浮点数，它至少具备和double类型一样的精度。

C++语言基本数据类型的完整列表及其取值范围，参见：https://zh.cppreference.com/w/cpp/language/types。

### 4.6.4 浮点数都是近似值

现实世界中的浮点数可能包含无穷多个小数位。例如：

- $\frac{1}{3}=0.3333333333\cdots$
- $\sqrt{2}=1.4142135623\cdots$
- $\pi=3.1415926535\cdots$
- $e=2.7182818284\cdots$

但计算机只能使用固定大小的内存空间容纳这些值。这就导致计算机中的浮点数只能是一个近似值。象float类型或者double类型的数据，存在着所谓表示误差。如果想当然地以为浮点数表示的是准确值（比如在做相等或不相等判断时），就会导致错误的结果：

```c++
double a, b;
...
if (a == b) {
    a与b相等时执行的代码
}
```

由于误差的存在，a有可能永远不会等于b。事实上只要它们之间的差足够小，就足以认定二者相等：

```c++
double a, b;
...
if (fabs(a - b) < 1e-6) {
    a与b相等时执行的代码
}
```

在实际应用中，以近似值表示的浮点数是完全可以接受的。就象人的体温，温度计显示$36.5^\circ C$，但它实际上可能是$36.494732103^\circ C$。在大多数场合，其实并不需要精确到小数点后这么多位。因此36.5这个近似的浮点数，对于人的体温而言，是可以接受的。尽管如此，在编写程序代码时，人们还是更倾向于使用double类型的浮点数，毕竟它表示的浮点数更精确。

有关二进制浮点数的算术标准是IEEE 754，参见：https://zh.wikipedia.org/wiki/IEEE_754。

### 4.6.5 关于显示舍入值的警告

例如下面的代码：

```c++
double a{ 14.234 }, b{ 18.673 };
cout << fixed << setprecision(2) << a << "+" << b << "=" << a + b << endl;
```

你以为的输出：

```
14.23+18.67=32.90
```

而实际的输出：

```
14.23+18.67=32.91
```

在计算机的内部，double型变量a和b的值分别为14.234和18.673，表达式a+b的值为32.907，四舍五入保留两位小数后为32.91。这种看似有悖于数学常识的算式，在计算机的世界里其实很正常。

### 4.6.6 即使是普通金额值也会出现浮点误差

例如下面的金额：

```c++
double a{ 123.02 };
```

如果以10位小数显示：

```c++
cout << fixed << setprecision(2) << d << endl;
```

输出的结果很可能是：

```
123.0199999999
```

这也是因为double型的变量a，在计算机内部保存的其实是13.02的近似值。这在金融领域可能会引发不小的风波，但它确实是包括C++在内的许多编程语言共有的问题。当然，也可以自己定义一个能够精确处理货币金额的类，取代基类数据类型double。