## 4.7 do-while循环语句

while循环语句总是从对循环继续条件的判断开始，条件满足则执行循环体中的语句，然后再次判断循环继续条件是否满足，满足则执行循环体中的语句，不满足则退出循环。如果循环继续条件从一开始就不满足，那么循环体中的语句一次也不会执行。

do-while循环语句则是先执行循环体中的语句，然后再判断循环继续条件是否满足，满足则再次执行循环体中的语句，不满足则退出循环。因此，do-while循环语句的循环体至少会被执行一次。

回顾之前编写的在哨兵控制下的循环：

```c++
// SentinelLoop.cpp
//
// 借助哨兵值控制的循环求解平均值问题
//
#include <iostream>
#include <iomanip>
using namespace std;

int main() {
    // 初始化阶段
    int total{ 0 }; // 班级总分
    int gradeCounter{ 0 }; // 成绩计数器

    // 处理阶段，在哨兵值控制下的循环
    int grade{ 0 };
    while (grade != -1) { // 循环直到哨兵值出现
        cout << "输入第" << gradeCounter + 1
             << "位同学的考试成绩，输入-1退出："; // 提示用户输入
        cin >> grade; // 读取用户输入的考试成绩
        if (grade != -1) {
            total = total + grade; // 将考试成绩累加到班级总分中
            gradeCounter = gradeCounter + 1; // 成绩计数器加一
        }
    }

    // 最后阶段
    if (gradeCounter) {
        double average{ static_cast<double>(total) / gradeCounter }; // 计算班级平均分

        // 输出班级总分和平均分
        cout << "\n班级总分：" << total << "\n";
        cout << "班级平均分：" << setprecision(2) << fixed << average << "\n";
    }
    else {
        cout << "\n没有输入考试成绩\n";
    }
}
```

程序代码的第15行将grade变量初始化为0是有必要的。因为如果不初始化，其值将是一个不确定的数，有可能恰好是-1。如果是这样，从第17到23行的循环体中语句一次都不会执行，完全没有给用户输入数据的机会，程序就直接退出了。

该程序的do-while版本。

...\Ch04\DoWhile\DoWhile.cpp：

```c++
// DoWhile.cpp
//
// do-while循环语句
//
#include <iostream>
#include <iomanip>
using namespace std;

int main() {
    // 初始化阶段
    int total{ 0 }; // 班级总分
    int gradeCounter{ 0 }; // 成绩计数器

    // 处理阶段，在哨兵值控制下的循环
    int grade;
    do {
        cout << "输入第" << gradeCounter + 1
            << "位同学的考试成绩，输入-1退出："; // 提示用户输入
        cin >> grade; // 读取用户输入的考试成绩
        if (grade != -1) {
            total = total + grade; // 将考试成绩累加到班级总分中
            gradeCounter = gradeCounter + 1; // 成绩计数器加一
        }
    } while (grade != -1); // 循环直到哨兵值出现

    // 最后阶段
    if (gradeCounter) {
        double average{ static_cast<double>(total) / gradeCounter }; // 计算班级平均分

        // 输出班级总分和平均分
        cout << "\n班级总分：" << total << "\n";
        cout << "班级平均分：" << setprecision(2) << fixed << average << "\n";
    }
    else {
        cout << "\n没有输入考试成绩\n";
    }
}
```

程序代码的第15行即使没有对grade变量做初始化也不会有问题。因为do-while循环保证了，先执行循环体中的代码，然后判断循环继续条件，用户总有输入数据的机会。

对比while循环语句和do-while循环语句：

```mermaid
graph LR
  subgraph do-while循环语句
    entrance2((入口))
    yes2(循环体语句)
    cond2{循环继续条件}
    exit2((出口))
    entrance2-->yes2-->cond2--满足-->yes2
    cond2--不满足-->exit2
  end
  subgraph while循环语句
    entrance1((入口))
    cond1{循环继续条件}
    yes1(循环体语句)
    exit1((出口))
    entrance1-->cond1--满足-->yes1-->cond1--不满足-->exit1
  end
```