## 4.8 switch-case开关分支语句

switch-case开关分支语句根据一个变量或表达式的值，在多个不同操作中做出选择。每个操作均与一个整型常量或常量表达式关联。任何字符或整型常量及其组合都可以作为上述选择的依据。switch-case开关分支语句的一般形式如下：

```c++
switch (表达式) {
    case 整型常量1:
        操作1;
        break;
    case 整型常量2:
        操作2;
        break;
    ...
    default:
        其它操作;
        break;
}
```

若表达式的值与整型常量1相等，则执行操作1，若其与整型常量2相等，则执行操作2，依此类推，若与所有整型常量都不相等，这执行其它操作。

### 4.8.1 基于switch-case开关分支语句的级别评定

程序根据学生所处的分数段，将其评定为不同级别，统计各级别的学生人数：

- 90分和90分以上，包括100分：优
- 80分和80分以上，不到90分：良
- 70分和70分以上，不到80分：中
- 60分和60分以上，不到70分：差
- 60分以下：劣

...\Ch04\SwitchCase\SwitchCase.cpp：

```c++
// SwitchCase.cpp
//
// 利用switch-case开关分支语句统计各级别学生人数
//
#include <iostream>
using namespace std;

int main() {
    int gradeCounter{ 0 }; // 成绩计数器
    int aCounter{ 0 }; // 优计数器
    int bCounter{ 0 }; // 良计数器
    int cCounter{ 0 }; // 中计数器
    int dCounter{ 0 }; // 差计数器
    int eCounter{ 0 }; // 劣计数器

    cout << "输入每位同学的考试成绩，按<Ctrl+Z>结束："; // 提示用户输入

    int grade;
    while (cin >> grade) {
        switch (grade / 10) {
            case 10: // 100分
                ++aCounter;
                break; // 退出switch-case分支语句

            case 9:  // 90分和90分以上
                ++aCounter;
                break; // 退出switch-case分支语句

            case 8: // 80分和80分以上，不到90分
                ++bCounter;
                break; // 退出switch-case分支语句

            case 7: // 70分和70分以上，不到80分
                ++cCounter;
                break; // 退出switch-case分支语句

            case 6: // 60分和60分以上，不到70分
                ++dCounter;
                break; // 退出switch-case分支语句

            default: // 60分以下
                ++eCounter;
                break; // 退出switch-case分支语句
        }

        ++gradeCounter;
    }

    cout << endl;

    if (gradeCounter) {
        // 输出各级别学生人数
        cout << "优：" << aCounter << endl
             << "良：" << bCounter << endl
             << "中：" << cCounter << endl
             << "差：" << dCounter << endl
             << "劣：" << eCounter << endl;
    }
    else {
        cout << "\n没有输入考试成绩\n";
    }
}
```

编译运行：

```
输入每位同学的考试成绩，按<Ctrl+Z>结束：65 95 88 59 75 70 100 50 60 0 99 ^Z

优：3
良：1
中：2
差：2
劣：3
```

程序在switch-case语句中，根据用户输入的学生成绩对10做整数除法后得到的值，匹配不同的case分支：

- grade除以10得10，说明grade为100，优，匹配“case 10”分支，aCounter递增
- grade除以10得9，说明grade大于等于90且小于100，优，匹配“case 9”分支，aCounter递增
- grade除以10得8，说明grade大于等于80且小于90，良，匹配“case 8”分支，bCounter递增
- grade除以10得7，说明grade大于等于70且小于80，中，匹配“case 7”分支，cCounter递增
- grade除以10得6，说明grade大运等于60且小于70，差，匹配“case 6”分支，dCounter递增
- grade除以10得到其它值，说明grade小于60，劣，匹配“default”分支，eCounter递增

#### 4.8.1.1 从用户处读取成绩

程序代码的第19行从用户处读取学生的考试成绩。while循环语句的循环继续条件是由提取操作符“>>”构成的表达式的值。如果从代表标准输入即键盘的cin对象中，成功提取到一个整数，保存到整型变量grade中，则该表达式的值为真，执行循环体中的代码，否则该表达式的值为假，退出while循环。使该表达式的值为假的方法，是从键盘输入一个文件结束符（EOF）。在不同操作系统上，与文件结束符对应的按键或按键组合不尽相同。在Windows系统中，按下Ctrl键的同时按下Z键，会产生一个文件结束符，而在UNIX、Linux或MacOS系统中，产生文件结束符则是同时按下Ctrl键和D键。

#### 4.8.1.2 处理用户输入的成绩

程序代码的第20到44行利用一个switch-case开关分支语句，根据用户输入的成绩落在不同的区间，执行不同的操作。在switch-case开关分支语句的一对花括号中，包含一到多个case分支和一个可选的default分支。default分支和各case分支的先后顺序没有硬性规定，但通常将default分支放在所有case分支之后。

当控制流执行到switch-case开关分支语句时，先对switch关键字后圆括号中的表达式进行求值，再将该值与花括号中每个case关键字后面的值进行比较。注意参与比较的两个值必须是bool、char、char8_t、char16_t、char32_t、wchar_t、short、int、long、long long等类型的有符号或无符号整数。一旦发现与哪个case关键字后面的值匹配，就执行该case分支中的代码，都不匹配，则执行default分支中的代码（如果有default分支）。每个case分支和default分支的最后一条语句都是break语句，用于退出switch-case结构。

程序中的“case 10”和“case 9”两个分支所执行的操作是完全相同的。执行相同操作的case分支可以合并。例如：

```c++
...
switch (grade / 10) {
    case 10: // 100分
    case 9:  // 90分和90分以上
        ++aCounter;
        break; // 退出switch-case分支语句
	...
}
...
```

switch-case开关分支语句没有提供针对范围的匹配机制，每个需要检测的可能值都必须被放在一个单独的case分支中。每个case分支和default分支都可以包含多条语句，但无需使用花括号括起，除非在分支内部声明了变量。

#### 4.8.1.3 break语句的必要性和fallthrough关键字

无论是case分支还是default分支，除了花括号中的最后一个分支，其中的break语句都是必要的。遗漏break语句，在执行完匹配分支后，下面的分支也都会执行，直到遇到一个break语句，或到达switch-case结构的终点。这被称为直通（falling through）。例如：

```c++
...
switch (grade / 10) {
    ...
    case 8: // 80分和80分以上，不到90分
        ++bCounter;

    case 7: // 70分和70分以上，不到80分
        ++cCounter;
        break; // 退出switch-case分支语句
    ...
}
...
```

对于成绩在80分和80分以上，不到90分的情况，不但“case 8”分支中的代码会执行，“case 7”分支中的代码也会执行，这将导致cCounter计数器的值明显比正确值偏大。直通因其可能引发的逻辑错误，而被视为一种潜在缺陷，导致多数编译器发出警告。但也不排除直通有时可能恰恰是出于代码编写者有意为之的设计本意。例如：

```c++
...
switch (grade / 10) {
    case 10: // 100分
    case 9:  // 90分和90分以上
        ++over90;

    case 8: // 80分和80分以上，不到90分
        ++over80;

    case 7: // 70分和70分以上，不到80分
        ++over70;

    case 6: // 60分和60分以上，不到70分
        ++over60;
        break; // 退出switch-case分支语句

    default: // 60分以下
        ++below60;
        break; // 退出switch-case分支语句
}
...
```

这段代码意在统计成绩在90分及以上、80分及以上、70分及以上、60分及以上，乃至60分以下的人数。分数大于等于90者，必然也大于80、70和60。分数大于等于80者，必然也大于70和60。分数大于等于70者，必然也大于60。直通在这里就是一种完全合理的设计。为了将这种有意为之的合理性传达给编译器，避免其给出不必要的警告，可以在通常放置break语句的地方，显式注明“[[fallthrough]]”。例如：

```c++
...
switch (grade / 10) {
    case 10: // 100分
    case 9:  // 90分和90分以上
        ++over90;
        [[fallthrough]];

    case 8: // 80分和80分以上，不到90分
        ++over80;
        [[fallthrough]];

    case 7: // 70分和70分以上，不到80分
        ++over70;
        [[fallthrough]];

    case 6: // 60分和60分以上，不到70分
        ++over60;
        break; // 退出switch-case分支语句

    default: // 60分以下
        ++below60;
        break; // 退出switch-case分支语句
}
...
```

#### 4.8.1.4 default分支

如果switch关键字后圆括号中表达式的值，不能与任何一个case关键字后面的值匹配，就会执行default分支中的代码。程序代码的第41到43行在一个default分支中统计成绩不及格的人数。如果没有找到匹配，且没有default分支，就会从switch-case语句的下一条语句继续执行。在一个switch-case语句中，穷尽所有可能，乃至都不匹配的可能，是一个好的做法。

#### 4.8.1.5 输出统计结果

如果用户至少输入了一个成绩，gradeCounter的值因递增而非零，即真，那么程序代码的第53到57行即被执行，输出各级别学生人数。如果没有输入任何成绩，程序代码的第60行会打印相应的提示信息。

### 4.8.2 switch-case语句的控制流程

```mermaid
graph TB
  entrance((入口))
  switch{"switch (表达式)"}
  case1(语句1)
  case2(语句2)
  case3(语句3)
  exit((出口))
  entrance-->switch--case 值1-->case1--break-->exit
  switch--case 值2-->case2--break-->exit
  switch--default-->case3--break-->exit
```

switch-case开关分支语句的最后一个分支，无论是case分支还是default分支，其中的break语句都是可有可无的，因为到了这个时候，肯定会从switch-case语句的下一条语句继续执行。无论如何，在每个switch-case开关分支语句中都尽量提供一个default分支，并放在所有分支的最后，集中精力处理意外和特殊情况。

### 4.8.3 关于case的注意事项

switch-case开关分支语句的每个case关键字后面，都必须是一个整型常量表达式。比较常见的是：

- 整型常量字面值，如：“case 9”
- 字符型常量字面值，如：“case ‘A’”
- 被定义为整型或字符型常量的宏，如：“case CMD_TRACKER_JOIN”
- 枚举，如：“case STORAGE_STATUS_ONLINE”