## 4.9 带初始化器的选择语句

如前所述，for循环语句的控制变量，如果在初始化语句中被声明，那么它的作用域仅限于该for循环语句。事实上，自C++17以后，可以在if条件语句、if-else条件分支语句和switch-case开关分支语句等选择语句中，使用类似的语法，即带初始化器的选择语句。与for循环语句的情况一样，这些在初始化器中声明的变量，其作用域仅限于该选择语句本身。

...\Ch04\Initializer\Initializer.cpp：

```c++
// Initializer.cpp : 此文件包含 "main" 函数。程序执行将在此处开始并结束。
//
// 带初始化器的if-else条件分支语句
//
#include <iostream>
using namespace std;

int main() {
    if (int value{ 7 }; value == 7) {
        cout << "值为" << value << "\n";
    }
    else {
        cout << "值非7，值为" << value << "\n";
    }

    if (int value{ 13 }; value == 9) {
        cout << "值为" << value << "\n";
    }
    else {
        cout << "值非9，值为" << value << "\n";
    }
}
```

编译运行：

```
值为7
值非9，值为13
```

### 4.9.1 带初始化器的选择语句语法

在if条件语句、if-else条件分支语句或switch-case开关分支语句中的初始化器，必须位于if或switch关键字后面的圆括号中。初始化器必须以分号结束。初始化器中可以同时声明多个类型相同的变量，其间以逗号分隔。在初始化器中声明的变量，也可以只声明不初始化，但并不推荐这么做。

### 4.9.2 在初始化器中声明的变量的作用域

在if条件语句、if-else条件分支语句和switch-case开关分支语句的初始化器中声明的任何变量，都只能在该语句的其余部分使用。

...\Ch04\InitializerScope\InitializerScope.cpp：

```c++
// InitializerScope.cpp
//
// 在初始化器中声明的变量的作用域
//
#include <iostream>
using namespace std;

int main() {
    if (int value{ 7 }; value == 7) {
        cout << "值为" << value << "\n";
    }
    else {
        cout << "值非7，值为" << value << "\n";
    }

    //cout << "值为" << value << "\n";
    int value{ 9 };

    if (int value{ 13 }; value == 9) {
        cout << "值为" << value << "\n";
    }
    else {
        cout << "值非9，值为" << value << "\n";
    }
    
    cout << "值为" << value << "\n";
}
```

打开第16行的注释将导致一个编译错误。第9行声明的变量value仅限于在第一个if-else条件分支语句中使用。第16行位于该语句之外，不能再继续使用第9行声明的value变量。

编译运行：

```
值为7
值非9，值为13
值为9
```

在从第19到24行的if-else条件分支语句中使用的value变量，源自该语句在第19行的初始化器中的声明，与第17行声明的value变量无关，后者在第26行被引用。