## 4.10 break语句和continue语句

除了选择和循环，C++语言还提供了break和continue语句来改变控制流。

### 4.10.1 break语句

在switch-case开关分支语句或while/do-while/for循环语句中使用break语句，将直接退出包含该语句的开关分支语句或循环语句，转而执行开关分支语句或循环语句的下一条语句。以while循环为例：

```mermaid
graph TB
  entrance((入口))
  cond1{循环继续条件}
  yes1(语句)
  cond2{循环退出条件}
  yes2(语句)
  exit((出口))
  entrance-->cond1--满足-->yes1-->cond2--不满足-->yes2-->cond1--不满足-->exit
  cond2--满足<br/>break-->exit
```

下面是哨兵值控制循环的另一个版本。

...\Ch04\Break\Break.cpp：

```c++
// Break.cpp
//
// 利用break语句退出循环
//
#include <iostream>
#include <iomanip>
using namespace std;

int main() {
    // 初始化阶段
    int total{ 0 }; // 班级总分
    int gradeCounter{ 0 }; // 成绩计数器

    // 处理阶段，在哨兵值控制下的循环
    while (true) {
        cout << "输入第" << gradeCounter + 1
            << "位同学的考试成绩，输入-1退出："; // 提示用户输入
        int grade;
        cin >> grade; // 读取用户输入的考试成绩

        if (grade == -1) // 哨兵值出现
            break; // 退出循环

        total = total + grade; // 将考试成绩累加到班级总分中
        gradeCounter = gradeCounter + 1; // 成绩计数器加一
    }

    // 最后阶段
    if (gradeCounter) {
        double average{ static_cast<double>(total) / gradeCounter }; // 计算班级平均分

        // 输出班级总分和平均分
        cout << "\n班级总分：" << total << "\n";
        cout << "班级平均分：" << setprecision(2) << fixed << average << "\n";
    }
    else {
        cout << "\n没有输入考试成绩\n";
    }
}
```

编译运行：

```c++
输入第1位同学的考试成绩：90
输入第2位同学的考试成绩：80
输入第3位同学的考试成绩：100
输入第4位同学的考试成绩：77
输入第5位同学的考试成绩：85
输入第6位同学的考试成绩：92
输入第7位同学的考试成绩：88
输入第8位同学的考试成绩：72
输入第9位同学的考试成绩：60
输入第10位同学的考试成绩：79
输入第11位同学的考试成绩：-1

班级总分：823
班级平均分：82
```

嵌套在while循环语句中的if条件语句检测到用户输入了-1，即哨兵值出现，这时执行break语句退出循环，转而执行该while循环语句的下一条语句，即第29行的if条件语句。

### 4.10.2 continue语句

在while/do-while/for循环语句的循环体中使用continue语句，将会跳过循环体的剩余语句，直接进入下一轮循环：

- 对于while/do-while循环，在执行完continue语句后，立即跳转到对循环继续条件的判断
- 对于for循环，在执行完continue语句后，立即跳转到语句头部第三语句的执行，继而判断循环继续条件

以while循环为例：

```mermaid
graph TB
  entrance((入口))
  cond1{循环继续条件}
  yes1(语句)
  cond2{直进下轮条件}
  yes2(语句)
  exit((出口))
  entrance-->cond1--满足-->yes1-->cond2--不满足-->yes2-->cond1--不满足-->exit
  cond2--满足<br/>continue-->cond1
```

考虑上面的案例，假设合理的考试成绩介于0到100之间，如果用户的输入不在此范围内，应打印错误提示并允许用户重新输入。

...\Ch04\Continue\Continue.cpp：

```c++
// Continue.cpp : 此文件包含 "main" 函数。程序执行将在此处开始并结束。
//
// 利用continue语句直接进入下一轮循环
//
#include <iostream>
#include <iomanip>
using namespace std;

int main() {
    // 初始化阶段
    int total{ 0 }; // 班级总分
    int gradeCounter{ 0 }; // 成绩计数器

    // 处理阶段，在哨兵值控制下的循环
    while (true) {
        cout << "输入第" << gradeCounter + 1
            << "位同学的考试成绩，输入-1退出："; // 提示用户输入
        int grade;
        cin >> grade; // 读取用户输入的考试成绩

        if (grade == -1) // 哨兵值出现
            break; // 退出循环

        if (grade < 0 || 100 < grade) { // 输入成绩无效
            cout << "输入成绩无效，重新输入\n";
            continue; // 跳过本轮循环，直接进入下一轮循环
        }

        total = total + grade; // 将考试成绩累加到班级总分中
        gradeCounter = gradeCounter + 1; // 成绩计数器加一
    }

    // 最后阶段
    if (gradeCounter) {
        double average{ static_cast<double>(total) / gradeCounter }; // 计算班级平均分

        // 输出班级总分和平均分
        cout << "\n班级总分：" << total << "\n";
        cout << "班级平均分：" << setprecision(2) << fixed << average << "\n";
    }
    else {
        cout << "\n没有输入考试成绩\n";
    }
}
```

编译运行：

```
输入第1位同学的考试成绩：90
输入第2位同学的考试成绩：80
输入第3位同学的考试成绩：120
输入成绩无效，重新输入
输入第3位同学的考试成绩：100
输入第4位同学的考试成绩：77
输入第5位同学的考试成绩：85
输入第6位同学的考试成绩：92
输入第7位同学的考试成绩：-10
输入成绩无效，重新输入
输入第7位同学的考试成绩：88
输入第8位同学的考试成绩：72
输入第9位同学的考试成绩：60
输入第10位同学的考试成绩：79
输入第11位同学的考试成绩：-1

班级总分：823
班级平均分：82
```

程序代码的第24到27行是一个if条件语句，对用户输入的考试成绩做有效性检测，一旦发现该输入小于0或大于100，即认定为无效，在打印错误提示后，执行continue语句，跳过本轮循环的后续操作，直接进入下一轮循环，控制流跳转到第15行，在判断循环继续条件是否满足后，继续执行循环体中的代码。

注意程序代码的第21到22行和第24到27行，两个if条件语句的前后顺序不能颠倒，否则当用户输入-1时，循环将无法退出。

有一种观点认为，break和continue语句在某种程度上破坏了结构化编程的基本原则。如果不使用它们也能达到同样的效果，还是尽量避免使用它们为好。

有时候，对高质量软件工程的追求，和编写最佳性能的程序代码之间，会存在难以权衡的矛盾。为了实现其中的一个目标往往不得不牺牲另一个。关键要看这种牺牲是否值得，即对性价比的评估。除非对性能有着特别的要求，否则代码的简洁性和易维护性还是需要被优先考虑的。在此基础上，再力图使其体积更小，速度更快，但只是在特别必要的时候。