## 4.11 逻辑操作符

在if条件语句、if-else条件分支语句、while/do-while/for循环语句中，需要通过条件决定如何继续一个程序的控制流。这里的条件可以是简单的条件，如“studentGrade >= 60”、“counter < 10”或者“grade != -1”等，也可以是由多个简单条件组合而成的复杂条件。C++语言的逻辑操作符就是用于组合简单条件的操作符，包括：

- 逻辑与操作符（&&）
- 逻辑或操作符（||）
- 逻辑非操作符（!）

### 4.11.1 逻辑与操作符（&&）

假设在程序的某个位置，需要确保只有在两个条件都满足时，才能执行某个操作。这种情况下，就可以使用逻辑与操作符（&&）。例如：

```c++
if (gender == FEMALE && age >= 65) {
    ++seniorFemales;
}
```

条件“gender == FEMALE”用于判断是否是女性，条件“age >= 65”用于判断是否是老人。只有当两个条件都满足时，表示老年女性的计数器变量seniorFemales的值才会递增。任何一个条件不满足，或者两个条件都不满足，程序都将跳过递增操作。

以下是逻辑与操作符（&&）的真值表：

左操作数|右操作数|左操作数 && 右操作数
:-:|:-:|:-:
true|true|true
true|false|false
false|true|false
false|false|false

### 4.11.2 逻辑或操作符（||）

假设在程序的某个位置，需要确保两个条件中至少有一个满足时，就能执行某个操作。这种情况下，就可以使用逻辑或操作符（||）。例如：

```c++
if (semesterAverage >= 90 || finalExam >= 90) {
	cout << "优\n";
}
```

条件“semesterAverage >= 90”用于判断学生的平时成绩是否足够优秀，条件“finalExam >= 90”用于判断学生的期末考试成绩是否足够优秀。两个条件中只要有一个满足，或者两个条件都满足，该学生即可被评定为优。只用当两个条件都不满足时，该生才不能被评优。

以下是逻辑或操作符（||）的真值表：

| 左操作数 | 右操作数 | 左操作数 \|\| 右操作数 |
| :------: | :------: | :--------------------: |
|   true   |   true   |          true          |
|   true   |  false   |          true          |
|  false   |   true   |          true          |
|  false   |  false   |         false          |

### 4.11.3 短路求值

包含逻辑与（&&）或逻辑或（||）的表达式，只要能提前知道最终结果，就不会再继续求值。例如：

```c++
gender == FEMALE && age >= 65
```

如果gender不等于FEMALE，其中第一个表达式“gender == FEMALE”的值即为false，那么整个表达式的值肯定也为false，第二个表达式“age >= 65”将被忽略，不再进行计算。这就叫短路与。又如：

```c++
semesterAverage >= 90 || finalExam >= 90
```

如果semesterAverage大于等于90，其中第一个表达式“semesterAverage >= 90”的值即为true，那么整个表达式的值肯定也为true，第二个表达式“finalExam >= 90”将被忽略，不再进行计算。这就叫短路或。

在逻辑与操作符（&&）连接的两个条件A和B中，如果对B条件的判断必须以A条件满足为前提，那么B条件就是A条件的附属条件。附属条件必须放在逻辑与操作符（&&）的后面，以防止发生错误。例如：

```c++
a != 0 && b / a == 2
```

“b / a == 2”就是“a != 0”的附属条件，即只有在a的值非零的前提下，才能判断b/a是否等于2，否则将导致除零错误。将附属条件“b / a == 2”放在逻辑与操作符（&&）后面的意义在于，如果a的值为零，前面的表达式“a != 0”的值即为false，整个表达式的值亦为false，短路与特性决定了后面的表达式“b / a == 2”不再被计算，避免了除零错误的发生。

### 4.11.4 逻辑非操作符（!）

与逻辑与操作符（&&）和逻辑或操作符（||）这些二元操作符不同，逻辑或操作符（!）是个典型的一元操作符，即它只有一个操作数。它的作用在于对其操作数做逻辑反转，即真变假，假变真。

以下是逻辑非操作符（!）的真值表：

| 操作数 | !操作数 |
| :-: | :-: |
|  true  |   false   |
|  false  |  true  |

如果某个操作只能在某个条件为假的前提下执行，那么就可以将该条件放在逻辑非操作符（!）之后。例如：

```c++
if (!(grade == -1)) {
    total = total + grade;
}
```

只有不是-1，即使表达式“grade == -1”的值为false的grade才是有效的考试成绩，才会被累加到班级总分中。注意“grade == -1”两端的圆括号不可或缺，因为逻辑非操作符（!）比等于操作符（==）的优先级高。

逻辑非操作符（!）可能导致程序代码的可读性变差，如果能用恰当的关系操作符等价替换之，仍不失为更好的选择。例如：

```c++
if (grade != -1) {
    total = total + grade;
}
```

### 4.11.5 打印真值表

下面的程序会打印出与、或、非三种基本逻辑运算的真值表。

...\Ch04\TruthTable\TruthTable.cpp：

```c++
// TruthTable.cpp
//
// 三种基本逻辑运算的真值表
//
#include <iostream>
#include <format>
using namespace std;

int main() {
    bool t{ true }, f{ false };

    // 逻辑与真值表
    cout << format(
        "逻辑与：\n"
        "{} && {} = {}\n"
        "{} && {} = {}\n"
        "{} && {} = {}\n"
        "{} && {} = {}\n",
        t, t, t && t,
        t, f, t && f,
        f, t, f && t,
        f, f, f && f);

    cout << endl;

    // 逻辑或真值表
    cout << format(
        "逻辑或：\n"
        "{} || {} = {}\n"
        "{} || {} = {}\n"
        "{} || {} = {}\n"
        "{} || {} = {}\n",
        t, t, t || t,
        t, f, t || f,
        f, t, f || t,
        f, f, f || f);

    cout << endl;

    // 逻辑非真值表
    cout << format(
        "逻辑非：\n"
        "!{} = {}\n"
        "!{} = {}\n",
        t, !t,
        f, !f);
}
```

编译运行：

```
逻辑与：
true && true = true
true && false = false
false && true = false
false && false = false

逻辑或：
true || true = true
true || false = true
false || true = true
false || false = false

逻辑非：
!true = false
!false = true
```

程序代码的第13到22行、第27到36行和第41到46行，分别在屏幕上打印出逻辑与、逻辑或和逻辑非，三种基本逻辑运算的真值表。默认情况下，将bool类型值插入标准输出流cout，会在屏幕上显示1和0分别代表真和假。format函数会将其格式化为true和false。

### 4.11.6 操作符的优先级和关联性

下表总结了到目前为止介绍的所有操作符的优先级和关联性。从上到下操作符的优先级依次递减：

|                    操作符                    |  关联性  |
| :------------------------------------------: | :------: |
| ++（后缀）、--（后缀）、static_cast\<类型>() | 从左到右 |
| ++（前缀）、--（前缀）、+（正号）、-（负号）、! | 从右到左 |
|                   *、/、%                    | 从左到右 |
|                     +、-                     | 从左到右 |
|                    <<、>>                    | 从左到右 |
|                 <、<=、>、>=                 | 从左到右 |
|                    ==、!=                    | 从左到右 |
|&&|从左到右|
|\|\||从左到右|
|                     ? :                      | 从右到左 |
|            =、+=、-=、*=、/=、%=             | 从右到左 |
|,|从左到右|

其中，只有前缀递增递减操作符、正负号操作符、逻辑非操作符、条件操作符，以及赋值和复合赋值操作符的关联性是从右到左，其它操作符的关联性都是从左到右。