## 4.12 混淆等于操作符（==）和赋值操作符（=）

即使是经验丰富的C++程序员也无法保证，永远不在使用等于操作符（==）和赋值操作符（=）的场合发生混淆。这种错误的危害在于，它通常不会引发编译错误，但却因逻辑错误导致非预期的结果。很多现代编译器会对本该使用等于操作符（\==），却误用了赋值操作符（=）的情况给出警告。

任何具有值的表达式，都可以作为选择或循环语句的判断条件。只要该表达式的值非零，即被视为逻辑真，为零，则被视为逻辑假。而赋值操作本身也是一种表达式——赋值表达式，该表达式也有值，就是赋值操作符左边的值。因此下面的代码在语法上是正确的：

```c++
if (grade = -1)
    break;
```

结果就是，无论之前grade变量的值是什么，break语句总会执行。因为赋值表达式“grade = -1”的值是-1，非零即真，if条件总能满足。更糟糕的是，grade变量的值被修改了，而它本来应该来自用户的输入。其实代码编写者的本意是这样的：

```c++
if (grade == -1)
    break;
```

即只在grade的值为-1时执行break语句。这种逻辑错误不会被编译器发现，却导致程序运行结果异常。

### 4.12.1 左值和右值

赋值操作符（=）左侧的值称为左值（lvalue），右侧的值称为右值（rvalue）。赋值操作符（=）的左值不允许是字面值，而等于操作符（==）却没有这个限制。因此，不妨把上面的if条件语句写成这样：

```c++
if (-1 == grade)
    break;
```

这样的写法有时称为Yoda条件，其功能并未改变，依然是检查grade的值与-1的相等性，但却降低了出现失误的风险，因为下面的代码：

```c++
if (-1 = grade)
    break;
```

将字面值-1放在了赋值操作符（=）的左边，这是无论如何无法通过编译的。

### 4.12.2 该用“=”却用了“==”

另一种令人沮丧的情形可能发生在意图对某个变量进行赋值操作：

```c++
total = total + grade;
```

却无意中写成了：

```c++
total == total + grade;
```

这同样不是一个语法错误。编译可以正常通过，但运行时发现变量total并没有得到被“赋予”的新值，因为这行代码实际上只是对total的值，和该值加上grade的值所得到的结果，做了相等性比较。比较的结果可能是true也可能是false，然后被丢弃。变量total的值丝毫未变。这同样是一个不易察觉但足以导致程序运行结果异常的逻辑错误。

### 4.12.3 启用警告

针对本该使用等于操作符（==）却误用赋值操作符（=）的情况：

- Clang C++编译器会默认给出一条警告
- GNU C++编译器在使用“-Wall”编译选项的前提下，会给出警告
- Visual C++编译器则需要做如下配置，才会给出警告
  - 右键单击项目名称，选择上下文菜单中的“属性”
  - 展开左侧窗格中的“Code Analysis”，选择“常规”
  - 将右侧窗格中的“生成时启用Code Analysis”设置为“是”
  - 点击“确定”按钮