## 4.13 对象自然案例

通过数据压缩可以减少数据的大小，节省内存、硬盘和网络带宽。具体分为：

- 无损压缩：压缩过程不丢失数据，压缩后的数据可以精确还原到压缩前的状态
  - 适用于压缩文本、可执行程序等需要精确还原的数据
- 有损压缩：压缩过程会丢失数据，压缩后的数据只能近似还原到压缩前的状态
  - 适用于压缩图像、音频、视频等只需近似还原的数据

### 4.13.1 ZIP文件

ZIP是一种无损压缩格式，其算法的核心思想是利用信息的重复性，将文本文件中连续出现的内容相同的字符串，或图像文件中连续出现的颜色相同的像素，替换为对单一副本的引用。

ZIP还可以将一个目录连同该目录下的所有子目录和文件，一起压缩成一个归档文件，俗称压缩包。

### 4.13.2 miniz-cpp库

miniz-cpp库是一个单文件库，整个库由一个名为“zip_file.hpp”的头文件组成，包含该文件即可使用该库。

...\Ch04\UseMiniz\UseMiniz.cpp：

```c++
// UseMiniz.cpp
//
// 借助miniz-cpp库读写压缩文件
//
#include <iostream>
#include <string>
using namespace std;

#include "zip_file.hpp"

int main() {
    miniz_cpp::zip_file zout; // 创建压缩包

    // 待压缩内容
    string compressedContent{
        "臣亮言先帝创业未半而中道崩殂今天下三分益州疲弊此诚危急存亡之秋也然"
        "侍卫之臣不懈于内忠志之士忘身于外者盖追先帝之殊遇欲报之于陛下也诚宜"
        "开张圣听以光先帝遗德恢弘志士之气不宜妄自菲薄引喻失义以塞忠谏之路也"
        "宫中府中俱为一体陟罚臧否不宜异同若有作奸犯科及为忠善者宜付有司论其"
        "刑赏以昭陛下平明之理不宜偏私使内外异法也侍中侍郎郭攸之费祎董允等此"
        "皆良实志虑忠纯是以先帝简拔以遗陛下愚以为宫中之事事无大小悉以咨之然"
        "后施行必能裨补阙漏有所广益将军向宠性行淑均晓畅军事试用于昔日先帝称"
        "之曰能是以众议举宠为督愚以为营中之事悉以咨之必能使行阵和睦优劣得所"
        "亲贤臣远小人此先汉所以兴隆也亲小人远贤臣此后汉所以倾颓也先帝在时每"
        "与臣论此事未尝不叹息痛恨于桓灵也侍中尚书长史参军此悉贞良死节之臣愿"
        "陛下亲之信之则汉室之隆可计日而待也臣本布衣躬耕于南阳苟全性命于乱世"
        "不求闻达于诸侯先帝不以臣卑鄙猥自枉屈三顾臣于草庐之中咨臣以当世之事"
        "由是感激遂许先帝以驱驰后值倾覆受任于败军之际奉命于危难之间尔来二十"
        "有一年矣先帝知臣谨慎故临崩寄臣以大事也受命以来夙夜忧叹恐托付不效以"
        "伤先帝之明故五月渡泸深入不毛今南方已定兵甲已足当奖率三军北定中原庶"
        "竭驽钝攘除奸凶兴复汉室还于旧都此臣所以报先帝而忠陛下之职分也至于斟"
        "酌损益进尽忠言则攸之祎允之任也愿陛下托臣以讨贼兴复之效不效则治臣之"
        "罪以告先帝之灵若无兴德之言则责攸之祎允等之慢以彰其咎陛下亦宜自谋以"
        "咨诹善道察纳雅言深追先帝遗诏臣不胜受恩感激今当远离临表涕零不知所言"
    };
    cout << "待压缩内容：" << compressedContent << endl;
    cout << "待压缩内容长度：" << compressedContent.length() << endl;

    // 将待压缩内容写入压缩包中的指定文件
    zout.writestr("出师表.txt", compressedContent); // 在压缩包中创建文件并写入内容

    cout << "压缩文件路径：";
    string path;
    getline(cin, path); // 输入一行文本
    zout.save(path); // 压缩包->压缩文件

    miniz_cpp::zip_file zin{ path }; // 压缩文件->压缩包

    // 显示压缩包文件和目录
    cout << "压缩包文件：" << zin.get_filename() << endl
         << "压缩包目录：" << endl;
    zin.printdir();

    // 获取压缩包中指定文件的信息
    miniz_cpp::zip_info zinfo{ zin.getinfo("出师表.txt") };
    cout << "文件名：" << zinfo.filename << endl
         << "原始大小：" << zinfo.file_size << endl
         << "压缩大小：" << zinfo.compress_size << endl;

    // 抽取压缩包中指定文件的内容
    string extractedContent{ zin.read(zinfo) };
    cout << "被抽取内容：" << extractedContent << endl;
    cout << "被抽取内容长度：" << extractedContent.length() << endl;
}
```

编译运行：

```
待压缩内容：臣亮言先帝创业未半而中道崩殂今天下三分益州疲弊此诚危急存亡之秋也然侍卫之臣不懈于内忠志之士忘身于外者盖追先帝之殊遇欲报之于陛下也诚宜开张圣听以光先帝遗德恢弘志士之气不宜妄自菲薄引喻失义以塞忠谏之路也宫中府中俱为一体陟罚臧否不宜异同若有作奸犯科及为忠善者宜付有司论其刑赏以昭陛下平明之理不宜偏私使内外异法也侍中侍郎郭攸之费祎董允等此皆良实志虑忠纯是以先帝简拔以遗陛下愚以为宫中之事事无大小悉以咨之然后施行必能裨补阙漏有所广益将军向宠性行淑均晓畅军事试用于昔日先帝称之曰能是以众议举宠为督愚以为营中之事悉以咨之必能使行阵和睦优劣得所亲贤臣远小人此先汉所以兴隆也亲小人远贤臣此后汉所以倾颓也先帝在时每与臣论此事未尝不叹息痛恨于桓灵也侍中尚书长史参军此悉贞良死节之臣愿陛下亲之信之则汉室之隆可计日而待也臣本布衣躬耕于南阳苟全性命于乱世不求闻达于诸侯先帝不以臣卑鄙猥自枉屈三顾臣于草庐之中咨臣以当世之事由是感激遂许先帝以驱驰后值倾覆受任于败军之际奉命于危难之间尔来二十有一年矣先帝知臣谨慎故临崩寄臣以大事也受命以来夙夜忧叹恐托付不效以伤先帝之明故五月渡泸深入不毛今南方已定兵甲已足当奖率三军北定中原庶竭驽钝攘除奸凶兴复汉室还于旧都此臣所以报先帝而忠陛下之职分也至于斟酌损益进尽忠言则攸之祎允之任也愿陛下托臣以讨贼兴复之效不效则治臣之罪以告先帝之灵若无兴德之言则责攸之祎允等之慢以彰其咎陛下亦宜自谋以咨诹善道察纳雅言深追先帝遗诏臣不胜受恩感激今当远离临表涕零不知所言
待压缩内容长度：1254
压缩文件路径：test.zip
压缩包文件：test.zip
压缩包目录：
  Length      Date    Time    Name
---------  ---------- -----   ----
     1254  01/17/2024 16:20   出师表.txt
---------                     -------
     1254                     1 file
文件名：出师表.txt
原始大小：1254
压缩大小：937
被抽取内容：臣亮言先帝创业未半而中道崩殂今天下三分益州疲弊此诚危急存亡之秋也然侍卫之臣不懈于内忠志之士忘身于外者盖追先帝之殊遇欲报之于陛下也诚宜开张圣听以光先帝遗德恢弘志士之气不宜妄自菲薄引喻失义以塞忠谏之路也宫中府中俱为一体陟罚臧否不宜异同若有作奸犯科及为忠善者宜付有司论其刑赏以昭陛下平明之理不宜偏私使内外异法也侍中侍郎郭攸之费祎董允等此皆良实志虑忠纯是以先帝简拔以遗陛下愚以为宫中之事事无大小悉以咨之然后施行必能裨补阙漏有所广益将军向宠性行淑均晓畅军事试用于昔日先帝称之曰能是以众议举宠为督愚以为营中之事悉以咨之必能使行阵和睦优劣得所亲贤臣远小人此先汉所以兴隆也亲小人远贤臣此后汉所以倾颓也先帝在时每与臣论此事未尝不叹息痛恨于桓灵也侍中尚书长史参军此悉贞良死节之臣愿陛下亲之信之则汉室之隆可计日而待也臣本布衣躬耕于南阳苟全性命于乱世不求闻达于诸侯先帝不以臣卑鄙猥自枉屈三顾臣于草庐之中咨臣以当世之事由是感激遂许先帝以驱驰后值倾覆受任于败军之际奉命于危难之间尔来二十有一年矣先帝知臣谨慎故临崩寄臣以大事也受命以来夙夜忧叹恐托付不效以伤先帝之明故五月渡泸深入不毛今南方已定兵甲已足当奖率三军北定中原庶竭驽钝攘除奸凶兴复汉室还于旧都此臣所以报先帝而忠陛下之职分也至于斟酌损益进尽忠言则攸之祎允之任也愿陛下托臣以讨贼兴复之效不效则治臣之罪以告先帝之灵若无兴德之言则责攸之祎允等之慢以彰其咎陛下亦宜自谋以咨诹善道察纳雅言深追先帝遗诏臣不胜受恩感激今当远离临表涕零不知所言
被抽取内容长度：1254
```

#### 4.13.2.1 创建压缩包对象

程序代码的第12行用miniz-cpp库中，位于miniz_cpp名字空间的zip_file类，创建了一个代表压缩包的对象zout。

#### 4.13.2.2 准备待压缩内容

程序代码的第15到37行声明并初始化了一个string类型的变量compressedContent，用于容纳待压缩的数据内容，随即将这段内容及其长度，即字节数打印到屏幕上。

#### 4.13.2.3 将待压缩内容写入压缩包中的指定文件

程序代码的第40行通过zout对象调用了zip_file类的writestr成员函数，将待压缩内容写入压缩包中的指定文件。

#### 4.13.2.4 读取用户输入的压缩文件路径

程序代码的第42到44行调用了\<string\>头文件中声明的getline函数，从用户处读取一行文本，即压缩文件的存储路径。

#### 4.13.2.5 将压缩包保存到压缩文件

程序代码的第45行通过zout对象调用了获取压缩包中指定文件的信息save函数，将压缩包保存到指定路径下的压缩文件中。

#### 4.13.2.6 压缩文件的内容

压缩文件是一个二进制文件，在文本编辑器中显示成下面这个样子：

```
PK    鰟1X圾痾?  ?
   出师表.txt5TiS跦鵆?&裐穌杁[氰鲗GA芈榔` 豑?;?\?G麟~飖玭巌濌鐨z?嗸@?铳X訯?鼙孁Pn碶CW诸蔩LJ濧ITT揟Wq癊赼藔鵏B齬?挾紴\YIc
~{M~憌鶚I/?祮歼檶??▆-H?謠`>?)~S人]C蚗>€Y?p4}g衒L奓P殳[摸€S芏箴緆緢HX擾覹倹?7瓇靨??瀄An鐍t哙"+丯脾瑱<婅?鲪=鋧缆<?褥枉-撎炳渗F憺j笹C樘?Z%鹆霽@a$驋K燯G允檲PT]j+|Pmx┯[?屽锩?U靁濔??罴S[/?榖岨?鮫跄唑?H[溅仴芁⑽p*癥sL齉uo廗:{芸x?OV絩怮H鬉綢毢T]j鄡4昨:眭?P淕?嵏褰政-檿	(煻H羯冴F鏩嘻釉蠆S鏡隴捴甭鶠?９z凩ヲ?録r簓嗟翯?\%
擆堶踌?p?鷻?鉓踧赘帛??霛{?口Fz婇`K?P?瑟i悉eO?Q讙颞疁]?碶'踭g绐KZ??鱊摡.P]F乣塪(卑{?疸?i睸蟵+?3KA哟"mw阺溉|刚醦蘋7苳?繎D08脅 物ó肺?
箱?
$m吀裻?hg廯摛瘥柫g??逴蛝荱?2操2{Jgp杲k觾眖,抗
寷?{r勔澘昬銟a?m将籟?
諠;??M伴丐?僰êBA鵛囵??纰?^a!i'吠 升麈0S??Q凝Z伈转蘑焸裌伬u?戋KV?摎g犞菥損灳V??娙FЁ\c?&凐?>v??t(范a雭?f阊榝O鹽ｖ_眞??k鳍嗽8藦??脘??PK      鰟1X圾痾?  ?
                 出师表.txtPK      8   ?
```

#### 4.13.2.7 从压缩文件加载到压缩包

程序代码的第47行创建了一个名为zin的zip_file对象，并用压缩文件的存储路径对其进行初始化。这样做的结果是将指定路径下的压缩文件加载到代表压缩包的zin对象中。

#### 4.13.2.8 显示压缩包文件和目录

程序代码的第50到52行显示了与压缩包相对应的，压缩文件的名字，和压缩包内部的目录列表。

#### 4.13.2.9 获取压缩包中指定文件的信息

程序代码的第55到58行通过zin对象调用了zip_file类的getinfo函数，获取压缩包中指定文件的信息，随即将该文件的文件名、原始大小、压缩大小等信息打印到屏幕上。

#### 4.13.2.10 抽取压缩包中指定文件的内容

程序代码第61到63行通过zin对象调用了zip_file类的read函数，用于抽取压缩包中指定文件的内容。最后将被抽取到的数据内容及其长度，即字节数打印到屏幕上。