## 4.14 用域宽和精度格式化文本

C++20通过\<format\>头文件，提供了功能强大的format函数，用于对文本做格式化。该函数完全可以取代旧式语法中流控制符的使用。下面是用format函数替代流控制符的复利计算程序。

...\Ch04\CompoundInterestFormat\CompoundInterestFormat.cpp：

```c++
// CompoundInterestFormat.cpp
//
// 带文本格式化的复利计算
//
#include <iostream>
#include <format>
using namespace std;

int main() {
    // 输入本金、利率和存期
    cout << "输入本金：";
    double principal;
    cin >> principal;
    cout << "输入利率：";
    double rate;
    cin >> rate;
    cout << "输入存期：";
    int years;
    cin >> years;

    // 打印表头
    cout << format("\n{}{:>20}\n", "年份", "账户余额");

    // 计算十年中每一年的账户余额
    for (int year{ 1 }; year <= years; ++year) {
        // 余额 = 本金 x (1 + 利率) ^ 年份
        double amount = principal * pow(1 + rate, year);
        // 打印年份和账户余额
        cout << format("{:4d}{:20.2f}\n", year, amount);
    }
}
```

编译运行：

```
输入本金：1000
输入利率：0.05
输入存期：10

年份            账户余额
  1            1050.00
  2            1102.50
  3            1157.62
  4            1215.51
  5            1276.28
  6            1340.10
  7            1407.10
  8            1477.46
  9            1551.33
 10            1628.89
```

### 4.14.1 格式化表头

程序代码的第22行输出经过格式化的表头字符串。这里使用的格式字符串为“\n{}{:>20}\n”。其中包含两个使用花括号对表示的占位符。第一个占位符匹配格式字符串后面的第一个参数“年份”。该占位符不含任何格式说明符。第二个占位符匹配格式字符串后面的第二个参数“账户余额”。该占位符含有格式说明符。占位符花括号中的冒号用于引入一个格式说明符。这里的格式说明符指示被格式化数据占20字符域宽，大于号表示右对齐。字符串格式化的默认对齐方式为左对齐，因此这里显式指明使用右对齐。

### 4.14.2 格式化年份和账户余额

程序代码的第29行输出经过格式化的年份和账户余额字符串。这里使用的格式字符串为“{:4d}{:20.2f}\n”。其中包含两个使用花括号对表示的占位符。第一个占位符匹配格式字符串后面的第一个参数year。该占位符含有格式说明符。格式说明符中的“4”表示域宽，“d”表示格式化为十进制整数。第二个占位符匹配格式字符串后面的第二个参数amount。该占位符含有格式说明符。格式说明符中的“20”表示域宽，“.2”表示保留两位小数，“f”表示格式化为浮点数。这两个占位符的格式说明符都没有显式指明对齐方式，故采用针对整数和浮点数格式化的默认对齐方式——右对齐。

注意浮点数的域宽既包括其整数部分，也包括其小数部分，连同小数点，以及可能存在的前导（右对齐）或后跟（左对齐）空格。如下图所示：

```
 _______域宽_______
/                  \
             1050.00
\___________/\__/|\/
   前导空格    整数|小数
               小数点
```