## 5.3 数学函数

C++中的函数分为两种，一种是在类中定义的成员函数，只能通过类或类类型的对象调用，另一种是不属于任何类的全局函数，不需要通过类或类类型的对象，就可以直接调用。

标准库通过\<cmath\>头文件提供了许多全局函数，它们都位于std名字空间，完成常见的数学计算。例如：

```c++
double x, y;
y = sqrt(x);
```

计算双精度型变量x的平方根，即$\sqrt x$，保存到同样是双精度型的变量y中。sqrt是一个函数，接受一个double类型的参数，并返回一个double类型的值。调用该函数前不需要创建任何对象。调用这种函数，只需要指定函数的名称，后跟被圆括号括起的实参表。实参表中可能有一到多个参数，也可能没有参数。

下表总结了一些常用的数学函数，其中的x和y都是double类型的实参变量：

函数|功能|示例
:-:|:-:|:-
ceil(x)|对x向上取整|ceil(9.2)的返回值是10.0<br/>ceil(-9.2)的返回值是-9.0
floor(x)|对x向下取整|floor(9.2)的返回值是9.0<br/>floor(-9.2)的返回值是-10.0
fabs(x)|求x的绝对值<br/>$|x|$|fabs(5.1)的返回值是5.1<br/>fabs(0.0)的返回值是0.0<br/>fabs(-5.1)的返回值是5.1
fmod(x, y)|求x除以y的余数|fmod(2.6, 1.2)的返回值是0.2
pow(x, y)|求x的y次幂<br/>$x^y$|pow(2.0, 3.0)的返回值是8.0<br/>pow(9.0, 0.5)的返回值是3.0
exp(x)|求自然常数的x次幂<br/>$e^x$|exp(1.0)的返回值是2.718282<br/>exp(2.0)的返回值是7.389056
log(x)|求x的自然对数<br/>$\ln x=\log_ex$|log(2.718282)的返回值是1.0<br/>log(7.389056)的返回值是2.0
log10(x)|求x的常用对数<br/>$\log_{10}x$|log10(10.0)的返回值是1.0<br/>log10(100.0)的返回值是2.0
sqrt(x)|求x的平方根<br/>$\sqrt x$|sqrt(9.0)的返回值是3.0<br/>sqrt(2.0)的返回值是1.414214
sin(x)|求x的正弦，x的单位是弧度<br/>$\sin x$|sin(0.0)的返回值是0.0<br/>sin(1.570796)的返回值是1.0
cos(x)|求x的余弦，x的单位是弧度<br/>$\cos x$|cos(0.0)的返回值是1.0<br/>cos(1.570796)的返回值是0.0
tan(x)|求x的正切，x的单位是弧度<br/>$\tan x$|tan(0.0)的返回值是0.0<br/>tan(0.785398)的返回值是1.0

### 5.3.1 C++11新增的数学函数

C++11在\<cmath\>头文件中增加了几十个数学函数。其中有些是全新的，有些是在原有的，double类型参数的基础上，增加了对float和long double类型参数的支持。例如新增的hypot函数，可用于计算直角三角形的斜边。

在\<cmath\>头文件中声明的所有函数，参见：https://zh.cppreference.com/w/cpp/numeric/math。

C++标准中所有浮点类型的数学函数，参见：https://timsong-cpp.github.io/cppwp/n4861/c.math。

### 5.3.2 C++20新增的数学常数和\<numbers\>头文件

在C++20之前，C++语言和标准库并没有提供对圆周率$\pi$、自然常数$e$等数学常数的支持。某些编译器以预定义宏的形式提供了部分支持，如用M_PI表示圆周率、用M_E表示自然常数等。但这种支持并非标准的一部分，兼容性很差。C++20新增了一个名为\<numbers\>的头文件，以常量的形式，对科学和工程计算中常用的数学常数进行了标准化。其中包括：

常量|数学形式
:-|:-:
numbers::e|$e$（自然常数）
numbers::log2e|$\log_2e$
numbers::log10e|$\log_{10}e$
numbers::ln2|$\log_e2$
numbers::ln10|$\log_e10$
numbers::pi|$\pi$（圆周率）
numbers::inv_pi|$\frac{1}{\pi}$
numbers::inv_sqrtpi|$\frac{1}{\sqrt\pi}$
numbers::sqrt2|$\sqrt2$
numbers::sqrt3|$\sqrt3$
numbers::inv_sqrt3|$\frac{1}{\sqrt3}$
numbers::egamma|$\gamma$（欧拉——马歇罗尼常数）
numbers::phi|$\varphi=\frac{1+\sqrt5}{2}$（斐波那契黄金分割比）

### 5.3.3 C++17新增的数学特殊函数

C++17在\<cmath\>头文件中增加了几十个数学特殊函数，以满足科学和工程计算的要求。例如：

- 拉盖尔多项式和连带拉盖尔多项式
- 勒让德多项式和连带勒让德多项式
- 埃尔米特多项式
- 指数积分
- 第一二三类完全和不完全椭圆积分
- $\beta$函数
- 黎曼$\zeta$函数
- 规则变形圆柱贝塞尔函数和不规则变形圆柱贝塞尔函数
- 第一类圆柱贝塞尔函数和第一类球面贝塞尔函数
- 圆柱诺依曼函数和球面诺依曼函数
- 球面连带勒让德函数