## 5.4 函数定义与声明

创建一个名为maximum的自定义函数，返回其三个int类型参数中最大的一个。

...\Ch05\Maximum\Maximum.cpp：

```c++
// Maximum.cpp
//
// 定义并声明求三个整数中最大值的函数
//
#include <iostream>
#include <iomanip>
using namespace std;

int maximum(int x, int y, int z); // 函数声明 

int main() {
    cout << "输入三个整数：";
    int a, b, c;
    cin >> a >> b >> c;

    cout << "其中最大值是：" << maximum(a, b, c) << endl; // 函数调用
}

// 函数定义  
int maximum(int x, int y, int z) {
    int maximumValue{ x }; // 假设x是最大值

    if (y > maximumValue) { // 如果y更大
        maximumValue = y; // 那么y是最大值
    }

    if (z > maximumValue) { // 如果z更大
        maximumValue = z; // 那么z是最大值
    }

    return maximumValue; // 返回最大值
}
```

编译运行：

```
输入三个整数：27 93 19
其中最大值是：93
```

main函数从用户处读取三个整数，以之为参数调用maximum函数，并将该函数返回的，传给它的三个参数中的最大值，显示在屏幕上。这里包括了函数语法的三个基本要素：函数定义、函数声明和函数调用。

### 5.4.1 函数定义

通常在函数定义的第一行，亦称函数头中要指明该函数返回值的数据类型、函数名和包含参数列表的一对圆括号。参数列表由一系列参数项组成。每个参数项均包括一个数据类型和一个参数名。两个或更多个参数项之间以逗号分隔。参数列表中也可以没有任何参数，但包围参数列表的圆括号不可省略。函数定义中的参数也叫形式参数，简称形参。函数定义中的参数表也叫形参表。函数头后面是被一对花括号括起来的函数体。函数体由一系列语句组成，表示函数的执行过程。函数定义的一般形式如下所示：

```
               _________形参表__________
              /                        \
返回类型 函数名(类型1 形参1, 类型2 形参2, ...) <- 函数头
{                                         \
    ...                                    > 函数体
}                                         /
```

程序代码的第20到32行定义了一个函数。该函数返回一个int类型的返回值，函数名为maximum，带有三个int类型的形参，分别为x、y和z。函数体中先假设x的值是最大值，然后判断y的值是否更大一些，若是，则将y的值作为最大值，再判断z的值是否更大一些，若是，则将z的值作为最大值。最后返回所得到的最大值。

### 5.4.2 函数调用

程序代码的第16行调用了自定义函数maximum，并将其返回值插入到标准输出流cout中。调用函数时需要指明所调函数的名称和由一对圆括号括起的参数列表。参数列表由一系列参数组成。两个或更多个参数之间以逗号分隔。参数列表中也可以没有任何参数，但包围参数列表的圆括号不可省略。参数列表中参数可以是常量、变量、由常量和变量组成的表达式，甚至来自其它函数的返回值等。函数调用中的参数也叫实际参数，简称实参。函数调用中的参数表也叫实参表。实参依其在参数表中的位置，传递其值给对应位置的形参。实参与对应形参的数据类型原则上应该一致。函数调用的一般形式如下所示：

```
                               ____实参表____
                              /             \
返回类型 接收返回值的变量 = 函数名(实参1, 实参2, ...);
```

当发生函数调用时，首先完成从实参到形参的数值传递，然后程序的控制流从调用语句跳转到函数体的第一条语句，执行函数体中的代码，直到遇到return语句或包围函数体的右花括号，将函数的返回值（如果有的话）赋予接收返回值的变量或某个匿名的临时变量，继续执行函数调用后面的语句。

### 5.4.3 函数声明

程序代码的第9行是一条函数声明。函数声明由特定函数的原型组成。函数原型可以理解为，函数定义中的函数头加分号。函数原型告诉编译器该函数的返回值类型、函数名、参数的个数和每个参数的类型。函数原型中的参数名并不重要，也不要求必须与函数定义一致，甚至可以省略不写。例如：

```c++
int maximum(int u, int v, int w);
```

或者

```c++
int maximum(int, int, int);
```

当编译器“看到”函数声明时，会记录下所声明函数的原型，并在编译函数定义和函数调用是使用该原型：

- 检查函数定义的函数头，是否与该函数的原型一致
- 检查函数调用的实参表，其中的实参个数和每个实参的类型，是否与原型形参表中的形参个数和每个形参的类型一致
- 检查函数调用的上下文，是否正确地使用了函数原型中的返回类型
- 无论是参数还是返回值，一旦类型不一致，编译器会根据原型中的目标类型，做有关类型转换的尝试
- 执行以上操作的过程中，发生任何导致编译器无法继续的情况，都会引发编译错误

如果函数定义在函数调用之前，即当编译器“看到”函数调用时，已经知道该函数的原型了，则函数声明可以省略。例如：

```c++
...
// 函数定义  
int maximum(int x, int y, int z) {
    int maximumValue{ x }; // 假设x是最大值

    if (y > maximumValue) { // 如果y更大
        maximumValue = y; // 那么y是最大值
    }

    if (z > maximumValue) { // 如果z更大
        maximumValue = z; // 那么z是最大值
    }

    return maximumValue; // 返回最大值
}
...
int main() {
    cout << "输入三个整数：";
    int a, b, c;
    cin >> a >> b >> c;

    cout << "其中最大值是：" << maximum(a, b, c) << endl; // 函数调用
}
...
```