## 5.6 函数原型与实参类型转换

除非让编译器在“看到”对某个函数的调用之前先“看到”对该函数的定义，否则必须在调用函数之前先对其进行声明，即列出该函数的原型。如果函数调用之前，既没有定义该函数，也没有声明该函数，将会引发编译错误。要求所有的函数一定在调用之前定义是不现实的。这种将函数定义顺序固定化的做法，对改善程序代码的组织结构，并没有多少积极意义，而且函数的定义顺序很容易随着程序代码的演进而发生变化。通过提供函数原型，在调用函数前先对其进行声明，才是最妥当的做法。事实上，在调用标准库函数sqrt的代码中包含头文件\<cmath\>，就是为了从该头文件中引入对sqrt函数的声明。

### 5.6.1 函数签名和函数原型

函数原型包括函数的返回值类型、函数名、形参表等信息，而函数签名则只包含函数名和形参表。函数的返回值类型并不是函数签名的一部分。注意区分函数的签名、原型、声明、定义和调用等概念：

```c++
     __________函数签名_________
    /                          \
int maximum(int a, int b, int c);  -- 函数原型（声明）
 _____________函数头____________
/                              \
int maximum(int a, int b, int c) { \
	函数体                           > 函数定义
}                                  /
int d = maximum(a, b, c);          -- 函数调用
```

函数的作用域是指程序中“知道”该函数存在，并可调用该函数的代码范围。同一作用域内的每个函数必须拥有唯一的签名。函数的返回值类型、函数名、参数的个数及类型等，在函数原型（声明）、函数头和函数调用之间必须严格一致，否则将导致编译错误。

### 5.6.2 实参类型转换

如果函数调用中的实参类型，与函数原型中的形参类型不一致，编译器会尝试将其转换为形参所要求的类型，前提是这种转换不属于收缩转换等风险性操作。如果实参的数据类型不能被隐式转换为形参的预期类型，将引发编译错误或警告。

### 5.6.3 类型升级和隐式转换

当函数调用中的实参与函数原型中的形参，因类型不一致而不得不做类型转换时，遵循基于类型升级的隐式转换规则。该规则也同样适用于多元表达式中的类型协调。基本数据类型从高级到低级的排列顺序如下表所示：

基本数据类型|等价写法
:-:|:-:
long double|
double|
float|
unsigned long long|unsigned long long int
long long|long long int
unsigned long|unsigned long int
long|long int
unsigned int|unsigned
int|
unsigned short|unsigned short int
short|short int
unsigned char|
char|
bool|

所有的类型转换都是按照目标类型，构造源类型数值的临时拷贝，被转换的数值保持不变。

#### 5.6.3.1 可能引发错误或警告的类型转换

将较高级的数据类型转换为较低级的数据类型，例如将double型的实参传递给int型的形参，可能因为收缩转换而导致数据损失。编译器一般会为这种情况报告错误或给出警告。

#### 5.6.3.2 显式的收缩转换

收缩转换必须显式进行。建议使用准则支持库（GSL）的narrow_cast操作符，明确地向编译器表达收缩转换的设计本意。例如：

```c++
int d = maximum(gsl::narrow_cast<int>(2.7), gsl::narrow_cast<int>(9.3), gsl::narrow_cast<int>(1.9));
```

圆括号中的值被显式地收缩转换为尖括号中的类型。