## 5.9 掷骰子游戏

游戏规则：

> 两个骰子，每个骰子有六个面，每个面上的点数分别为一、二、三、四、五、六。玩家掷出两个骰子，计算面向上的点数之和。如果第一次掷出七或十一点，则玩家赢。如果第一次掷出二、三或十二点，则玩家输。如果第一次掷出其它点，则继续掷骰子，直到掷出和第一次相同的点，玩家赢。如果在这之前掷出了七点，则玩家输。

游戏过程如下图所示：

```mermaid
graph TB
  begin((开始))
  roll1{掷骰子}
  won1(玩家赢)
  lost1(玩家输)
  roll2{掷骰子}
  won2(玩家赢)
  lost2(玩家输)
  over((结束))
  begin-->roll1--七或十一点-->won1-->over
  roll1--二、三或十二点-->lost1-->over
  roll1--其它点-->roll2--和第一次相同的点-->won2-->over
  roll2--七点-->lost2-->over
  roll2--其它点-->roll2
```

...\Ch05\Craps\Craps.cpp：

```c++
// Craps.cpp
//
// 掷骰子游戏
//
#include <iostream>
#include <format>
#include <random> 
using namespace std;

int roll(); // 掷出两个骰子，计算面向上的点数之和

int main() {
    // 用带作用域的枚举表示游戏状态
    enum class Status { won, lost, keep }; // 玩家赢、玩家输、继续玩

    int first{ roll() }; // 第一次掷骰子
    Status status{ Status::keep }; // 游戏状态

    // 根据第一次掷出的点数决定游戏状态
    switch (first) {
        case 7: // 第一次掷出七点，玩家赢
        case 11: // 第一次掷出十一点，玩家赢          
            status = Status::won;
            break;

        case 2: // 第一次掷出二点，玩家输
        case 3: // 第一次掷出三点，玩家输
        case 12: // 第一次掷出十二点，玩家输
            status = Status::lost;
            break;

        default: // 第一次掷出其它点，继续玩
            cout << format("第一次掷出：{}\n", first);
            break; // 最后一个分支的break语句可以不写，但建议写上  
    }

    // 循环，直到游戏结束
    while (Status::keep == status) { // 继续玩
        // 再次掷骰子，根据掷出的点数决定游戏状态
        if (int const points{ roll() }; points == first) { // 掷出和第一次相同的点，玩家赢
            status = Status::won;
        }
        else if (points == 7) { // 掷出七点，玩家输
            status = Status::lost;
        }
    }

    // 显示游戏结果
    if (Status::won == status) {
        cout << "玩家赢" << endl;
    }
    else {
        cout << "玩家输" << endl;
    }
}

// 掷出两个骰子，计算面向上的点数之和
int roll() {
    // 设置随机数生成器
    static random_device rd; // 随机种子生成器
    static default_random_engine engine{ rd() }; // 默认随机引擎 
    static uniform_int_distribution die{ 1, 6 }; // 平均整数分布

    int const face1{ die(engine) }; // 掷第一个骰子
    int const face2{ die(engine) }; // 掷第二个骰子
    int const points{ face1 + face2 }; // 计算点数之和

    // 显示掷骰子结果
    cout << format("玩家掷出：{}+{}={}\n", face1, face2, points);

    return points;
}
```

编译运行：

```
玩家掷出：6+1=7
玩家赢

玩家掷出：1+1=2
玩家输

玩家掷出：5+1=6
第一次掷出：6
玩家掷出：2+1=3
玩家掷出：4+6=10
玩家掷出：6+6=12
玩家掷出：2+3=5
玩家掷出：4+6=10
玩家掷出：3+2=5
玩家掷出：2+1=3
玩家掷出：3+2=5
玩家掷出：1+5=6
玩家赢

玩家掷出：6+3=9
第一次掷出：9
玩家掷出：6+4=10
玩家掷出：1+3=4
玩家掷出：6+2=8
玩家掷出：5+2=7
玩家输
```

### 5.9.1 实现游戏

这段程序用两个函数，main和roll，以及switch-case开关分支语句、while循环语句和if-else条件分支语句，模拟了一个掷骰子的游戏。程序代码的第10行即对roll函数做了声明，该函数的原型表明其不带有任何参数（一对空的圆括号）且返回一个整数（两个骰子面向上的点数之和）。

### 5.9.2 带作用域的枚举

程序用一个名为status的枚举型变量跟踪游戏的状态——玩家赢、玩家输或者继续玩。该枚举类型在程序代码的第14行被定义为Status。枚举类型的定义形式如下所示：

```c++
enum class 枚举类型 { 枚举元素, 枚举元素, ... };
```

枚举中的枚举元素均为符号化的整型常量，因此也被称为枚举常量：

- 枚举常量的基础类型为int
- 枚举常量默认从0开始，依次递增1
- 同一个枚举类型中的多个枚举常量，名字必须唯一，但取值可以相同
- 枚举常量只能被赋予其所属类型的变量

按照惯例，枚举类型的名字，首字母应该大写。如果由多个单词组成，则后续每个单词的首字母也应该大写。枚举常量和一般变量采用相同的命名规则。

引用带作用域的枚举常量，需要使用作用域解析操作符“::”，如：Status::keep，而不能直接写成keep。

### 5.9.3 第一次掷骰子

程序代码的第16到35行根据玩家第一掷骰子的结果，判定其或赢或输或继续玩。在一个switch-case开关分支语句中：

- 针对七或十一点的情况，将status变量的值设为Status::won，表示玩家赢
- 针对二、三或十二点的情况，将status变量的值设为Status::lost，表示玩家输
- 针对其它点数的情况，保持status的初值Status::keep不变，即继续玩

同时，保存第一次掷出点数的变量first，会在后续代码中继续使用。

### 5.9.4 继续掷骰子

程序代码的第38到46行用一个while循环语句，模拟了玩家不断掷骰子的情形。只要status的值是Status::keep，循环体就是一直被执行。执行过程中不停掷出骰子，并根据掷出结果调整status的值：

- 掷出和第一次相同的点，将status变量的值设为Status::won，表示玩家赢，退出循环
- 掷出七点，将status变量的值设为Status::lost，表示玩家输，退出循环
- 掷出其它点，保持status的初值Status::keep不变，即继续玩，进入下一轮循环

很多时候，什么都不做也是一种做，即保持原状，比如这里对status的初值Status::keep的保持。

### 5.9.5 显示游戏结果

程序代码的第49到54行根据status变量的值是Status::won还是Status::lost，判定玩家或赢或输，并将结果显示在屏幕上。注意程序运行到这里，status变量的值已经不可能再是Status::keep，因为如果那样的话，前面的while循环语句就不会退出，因此这里无需考虑这种情况。

### 5.9.6 roll函数

程序代码的第58到72行定义了一个名为roll的函数，模拟玩家掷出两个骰子，计算并返回面向上的点数之和。

这里的random_device、default_random_engine和uniform_int_distribution对象，都被定义成静态局部变量。一般局部变量会在函数每次被调用时创建，并随着函数的返回而消亡。静态局部变量则在函数第一次被调用时创建，但不因函数的返回而消亡，并在该函数的后续调用中重用，直到程序终止。

产生随机数的一般模式如下图所示：

```mermaid
graph LR
  device(随机种子生成器对象)
  engine(随机引擎对象)
  dist(随机分布对象)
  seed((随机种子))
  rand{随机数}
  device-->seed-->engine-->dist-->rand
```

- 先用random_device类，实例化一个随机种子生成器对象
- 再用类似default_random_engine的类，实例化一个随机引擎对象，其初始化参数为上一步得到的随机种子生成器对象所生成的随机种子
- 用类似uniform_int_distribution的类，实例化一个随机分布对象，其初始化参数为分布范围的边界
- 以随机引擎对象为参数，调用随机分布对象，得到一个随机数

### 5.9.7 带作用域的枚举的其它注意事项

用枚举类型名和作用域解析操作符“::”，作为在“enum class”中定义的枚举常量的前缀，可以在不同枚举类型中定义同名的枚举常量，而不致引发歧义。例如：

```c++
enum class PenColor { black, white, red, green, blue };
enum class BrushColor { black, white, red, green, blue };
...
PenColor pc = PenColor::red;
BrushColor bc = BrushColor::red;
```

月份也是一种常见的带作用域的枚举类型。例如：

```c++
enum class Month { jan = 1, feb, mar, apr, may, jun, jul, aug, sep, oct, nov, dec };
```

这里的第一个枚举常量的值被显式设置为1，而非默认的0。其余各值从1开始依次递增，正好表示一年中的十二个月。任何枚举常量都可以在定义枚举类型时被指定一个整数值，后续枚举常量的值在该值基础上依次递增，直到被再次显式设置。

### 5.9.8 旧式语法中的枚举

旧式语法中的枚举定义没有class关键字。例如：

```c++
enum Status { won, lost, keep };
```

枚举常量可以直接引用，无需被冠以由枚举类型名和作用域解析操作符组成的前缀。例如：

```c++
Status status{ keep };
status = won;
status = lost;
```

这样的枚举称为无作用域的枚举。两个或更多的无作用域枚举，如果出现了同名的枚举常量，将因名字冲突而导致编译失败。建议始终使用带作用域的枚举。

### 5.9.9 指定枚举常量的类型

枚举常量的基础类型为int，但也可以通过如下形式，指定为其它整数类型。例如：

```c++
enum class Status : short { won, lost, keep };
```

### 5.9.10 对枚举的using声明

在不发生名字冲突的前提下，可以通过对枚举类型使用using声明：

```c++
using enum Status;
```

它允许在程序代码的剩余部分，直接使用枚举常量，而省略由枚举类型和作用域解析操作符组成的前缀：

```c++
Status status{ keep };
status = won;
status = lost;
```

using声明也可以只对单个枚举常量使用：

```c++
using Status::keep;
```

达到部分简化的效果：

```c++
Status status{ keep };
status = Status::won;
status = Status::lost;
```

这种using声明通常只出现在使用它们的代码块中。