## 5.11 内联函数

从软件工程的角度，以一组函数的形式分拆程序的功能，确实体现了分而治之的思想，但因频繁的函数调用而增加的运行时间开销，通常也是导致程序运行性能下降的原因之一。C++通过所谓内联函数的优化机制，在不破坏模块化设计原则的前提下，减少了函数调用的开销，提高了程序的运行性能。

在定义函数时，于其返回类型的前面加上一个inline关键字，即将该函数指定为内联函数。只要条件允许，编译器在所有调用内联函数的地方，都生成一份函数体二进制代码的拷贝，从而避免了对该函数的实际调用。当然由此带来的副作用，是导致程序的体积增大，牺牲空间换取时间。

与非内联函数在头文件中声明，在源文件中定义不同，可重用内联函数的定义通常位于头文件中，这样它们就可以在每个包含该头文件并调用该函数的源文件中被内联。一旦更改了某个内联函数的定义，所有调用该函数的代码都必须被重新编译，以确保更新后的函数体二进制代码被重新内联。

使用inline关键字，只是表达了程序设计者的一种期望，编译器有权忽略该关键字。比如将该关键字用在被稀少调用的复杂函数或者被递归调用的函数上，编译器并不会真的内联这样的函数。内联被稀少调用的复杂函数并不划算，而被递归调用的函数事实上无法内联。

处于性能优化的考虑，某些函数即便没有被显式添加inline关键字，编译器也会自动将其处理为内联。C++核心准则要求，只对小且时间性能要求高的函数进行内联。

有关内联函数的更多细节，参见：https://isocpp.org/wiki/faq/inline-functions。

下面的程序通过内联函数cube计算立方体的体积。

...\Ch05\Cube\Cube.cpp：

```c++
// Cube.cpp
//
// 计算立方体体积的内联函数
//
#include <iostream>
using namespace std;

// 内联函数：计算立方体的体积
// 在调用之前定义，故无需声明
// 函数定义的第一行即函数原型
inline double cube(double side) {
    return side * side * side; // 计算体积
}

int main() {
    double side; // 边长
    cout << "输入立方体的边长：";
    cin >> side; // 读取用户输入的边长

    // 计算并输出立方体的体积
    cout << "边长为" << side << "的立方体的体积：" << cube(side) << endl;
}
```

编译运行：

```
输入立方体的边长：10
边长为10的立方体的体积：1000
```