## 5.13 缺省参数

程序从多个地方调用同一个函数，并为某个特定的参数传递相同的值，这时一种很常见的情况。在这种情况下，可以为该参数指定一个缺省值。如果在调用该函数时，没有为带有缺省值的形参传递对应的实参，编译器会自动改写函数调用，将该参数的缺省值作为实参传递给对应的形参。为函数参数指定的缺省值，亦称函数的缺省参数。

### 5.13.1 带有缺省参数的函数

下面的程序演示了如何声明、定义和调用，带有缺省参数的函数。

...\Ch05\BoxVolume\BoxVolume.cpp：

```c++
// BoxVolume.cpp
//
// 函数的缺省参数
//
#include <iostream>
using namespace std;

// 声明带有缺省参数的函数
int boxVolume(int length = 1, int width = 1, int height = 1);

int main() {
    // 不提供任何参数，全部参数都取缺省值
    cout << "缺省长方体的体积：" << boxVolume() << endl;

    // 指定长度，宽和高取缺省值
    cout << "长度为10，宽和高取缺省值1的长方体体积：" << boxVolume(10) << endl;

    // 指定长和宽，高度取缺省值
    cout << "长为10宽为5，高度取缺省值1的长方体体积：" << boxVolume(10, 5) << endl;

    // 指定所有参数
    cout << "长宽高分别为10、5、2的长方体体积：" << boxVolume(10, 5, 2) << endl;
}

// 定义带有缺省参数的函数
int boxVolume(int length, int width, int height) {
    return length * width * height;
}
```

编译运行：

```
缺省长方体的体积：1
长度为10，宽和高取缺省值1的长方体体积：10
长为10宽为5，高度取缺省值1的长方体体积：50
长宽高分别为10、5、2的长方体体积：100
```

程序代码的第26到28行定义了一个名为boxVolume的函数，用于计算长方体的体积。该函数带有三个整型参数，length、width和height，分别表示长方体的长度、宽度和高度。长方体的体积为三者的乘积。在程序代码第9行的函数声明中，以“=1”的形式，为该函数的每个参数都指定了一个缺省值1。

程序代码的第13行以完全无参的形式调用boxVolume函数。这时该函数的三个参数都取缺省值1。所计算长方体的体积为：$length\times width\times height=1\times 1\times 1=1$。

程序代码的第16行在调用boxVolume函数时，只提供了一个实参10传递给length，因此width和height取缺省值1。所计算长方体的体积为：$length\times width\times height=10\times 1\times 1=10$。

程序代码的第19行在调用boxVolume函数时，为length和width提供了实参10和5，没有为height提供实参，因此height取缺省值1。所计算长方体的体积为：$length\times width\times height=10\times 5\times 1=50$。

程序代码的第22行在调用boxVolume函数时，为其所有的形参都提供了对应的实参，因此没有参数再取缺省值。所计算长方体的体积为：$length\times width\times height=10\times 5\times 2=100$。

注意，函数的实参与形参，是按位置对应结合的，结合的顺序为从左至右，因此总是在参数表中位置靠右的参数取得缺省值。

### 5.13.2 缺省参数的注意事项

函数的缺省参数必须集中于函数参数表的尾部。调用拥有多个缺省参数的函数，如果某个被省略的实参不是该函数的最后一个实参，那么该实参后面的所有实参也都必须省略，并取缺省值。

函数的缺省参数必须在函数名首次被编译器“看到”时指定，通常出现在函数声明中。如果省略了函数声明，那么函数定义也被看作是声明，其函数头中同样应包含缺省参数。

函数参数的缺省值可以是任意表达式，包括常量、全局变量或其它函数调用的返回值。

内联函数的参数也可以带有缺省值。

使用缺省参数可以简化函数调用语句，但也有人认为，显式提供所有参数，会令程序代码更加清晰易读。