## 5.14 一元作用域解析操作符

可以通过一元作用域解析操作符“::”，显式指明所引用的标识符是一个全局变量或全局函数，而非局部域或成员域的变量或函数。这在局部域或成员域对全局域构成同名隐藏时，非常有用。注意，不能用这种方法引用外层非全局域的同名标识符。

...\Ch05\UnaryScopeRes\UnaryScopeRes.cpp：

```c++
// UnaryScopeRes.cpp
//
// 一元作用域解析操作符
//
#include <iostream>
using namespace std;

const int number{ 7 }; // 名为number的全局变量

int main() {
    const double number{ 10.5 }; // 名为number的局部变量

    // 显示局部和全局变量的值
    cout << "名为number的局部变量：" << number << endl;
    cout << "名为number的全局变量：" << ::number << endl;
}
```

编译运行：

```
名为number的局部变量：10.5
名为number的全局变量：7
```

程序代码的第6和11行，分别定义了一个整型，且名为number的全局变量，和一个双精度浮点型，但同样名为number的局部变量。局部变量number隐藏了全局变量number。第14行，不使用一元作用域解析操作符，所引用的是局部变量number，而第15行，通过一元作用域解析操作符，显式指明所引用的是全局变量number。

如果全局变量的名字与局部变量的名字不同，则可以不通过一元作用域解析操作符，直接引用全局变量。而且通常会鼓励这么做，因为将同名的变量用于不同的目的，并不是一种好的设计。但即便如此，仍建议通过一元作用域解析操作符引用全局变量，一方面它使程序代码更好理解，清晰地表明所引用的是一个全局变量，另一方面它使程序代码更好维护，规避了因局部变量意外隐藏全局变量，而导致逻辑错误的风险。