## 5.17 递归函数

对于某些问题，让函数调用自己会非常有用。递归函数是一个能直接或间接（通过另一个函数）调用自己的函数。

### 5.17.1 递归的概念

以阶乘为例讨论哪些问题适合以递归方式解决：

$$
n!=\begin{cases}
1\quad&(n\leqslant1) \\
n(n-1)!\quad&(n>1)
\end{cases}
$$

- 编写一个函数求任意整数number的阶乘
  ```c++
  long factorial(int number) {
  }
  ```
- 虽然该问题的一般解未知，但该问题的基本解已知
  ```c++
  long factorial(int number) {
      if (number <= 1) {
          return 1;
      }
  }
  ```
- 基本解以外的情况，分为若干子问题，每个子问题要么解已知，要么与母问题类似，并收敛于基本解
  ```c++
  long factorial(int number) {
      if (number <= 1) {
          return 1;
      }
      else {
          return number * (number - 1)!;
      }
  }
  ```
- 与母问题类似的子问题，可以调用解决母问题的函数——递归调用
  ```c++
  long factorial(int number) {
      if (number <= 1) {
          return 1;
      }
      else {
          return number * factorial(number - 1);
      }
  }
  ```

如果遗漏了针对基本解的处理，或者问题无法收敛于基本解，递归调用会无限持续下去，这通常会导致栈溢出。如果一个未被设计为递归的函数意外调用了自己，无论是直接调用还是通过另一个函数间接调用，也会引发无限递归。

在递归调用的执行过程中，上一个执行过程尚未结束，即借由对函数自身的调用，进入了下一个执行过程，以此类推，不断地在调用中继续调用，直到函数识别出基本解的特征，如“number <= 1”，这时整个调用过程才开始沿着与调用相反的顺序，逐级返回，直至返回最初的调用者为止。

```mermaid
graph LR
  invoker(最初调用者)
  f5("factorial(5)")
  f4("factorial(4)")
  f3("factorial(3)")
  f2("factorial(2)")
  f1("factorial(1)")
  invoker--5-->f5--4-->f4--3-->f3--2-->f2--1-->f1
  f1--1-->f2--2-->f3--6-->f4--24-->f5--120-->invoker
```

### 5.17.2 递归法求阶乘

...\Ch05\RecursiveFactorial\RecursiveFactorial.cpp：

```c++
// RecursiveFactorial.cpp
//
// 递归法求阶乘
//
#include <iostream>
#include <iomanip>
using namespace std;

long factorial(int number); // 函数原型

int main() {
    // 计算从0到10的阶乘
    for (int number{ 0 }; number <= 10; ++number) {
        cout << setw(2) << number << "! = " << factorial(number) << endl;
    }
}

// 递归法求阶乘的函数
long factorial(int number) {
    if (number <= 1) { // 测试基本解的条件
        return 1; // 基本解：n!=1, (n<=1)
    }
    else { // 递归调用
        return number * factorial(number - 1);
    }
}
```

编译运行：

```
 0! = 1
 1! = 1
 2! = 2
 3! = 6
 4! = 24
 5! = 120
 6! = 720
 7! = 5040
 8! = 40320
 9! = 362880
10! = 3628800
```

程序代码的第19到26行定义了一个递归法求阶乘的函数。第20行首先判断递归终止条件，即基本解的条件。如果参数number的值小于等于1，即不再继续递归，而是直接返回计算结果1。而对于其它情况，即参数number的值大于1，则执行第24行的递归调用，求取number-1的阶乘，与number相乘后，得到number的阶乘，返回最终结果。而在number-1的阶乘求取过程中，重复以上步骤，直至递归终止。

### 5.17.3 阶乘值的快速增长

factorial函数接受一个int类型的参数并返回一个long类型的结果。一个long类型整数通常占用至少4字节的内存空间，拥有32个二进制位，能表示从-2147483648到2147483647范围内的值。但遗憾的是，factorial函数产生大整数的速度非常快，计算不了几个数的阶乘，就会达到long类型整数的最大值。为此，可以考虑使用long long类型的整数，或能够表示任意大小整数的类（如BigNumber）类型的对象，作为阶乘计算的结果。