## 5.18 斐波那契数列

斐波那契数列的第一项和第二项分别为0和1，后续各项均为其前两项的和，如：

$$
0,1,1,2,3,5,8,13,21,\cdots
$$

当斐波那契数列的项数趋近于无穷大时，相邻两项的后项与前项之比趋近于一个常数：

$$
lim_{n\to\infty}\frac{fibonacci(n)}{fibonacci(n-1)}=1.618\dots
$$

该常数成为黄金分割比，在\<numbers\>头文件中被定义为常量“::phi”。如果一个矩形的长度和宽度之比为黄金分割比，人类倾向于认为该矩形最具美感。

### 5.18.1 斐波那契数列的递归定义

斐波那契数列的递归定义如下所示：

$$
fibonacci(n)=\begin{cases}
0\quad&(n=1) \\
1\quad&(n=2) \\
fibonacci(n-1)+fibonacci(n-2)\quad&(n>2)
\end{cases}
$$

- 编写一个函数求斐波那契数列第number项的值
  ```c++
  long factorial(int number) {
  }
  ```
- 两个基本解，即当number的值为1和2时，函数的返回值是已知的
  ```c++
  long fibonacci(long number) {
      if (number == 1 || number == 2) {
          return number - 1;
      }
  }
  ```
- 其它情况，划分为两个子问题，即求斐波那契数列第number-1和number-2项的值，二者相加
  ```c++
  long fibonacci(long number) {
      if (number == 1 || number == 2) {
          return number - 1;
      }
      else {                                
          return 第number-1项的值 + 第number-2项的值;
      }
  }
  ```
- 子问题与母问题高度相似，递归调用解决母问题的函数
  ```c++
  long fibonacci(long number) {
      if (number == 1 || number == 2) {
          return number - 1;
      }
      else {                                
          return fibonacci(number - 1) + fibonacci(number - 2);
      }
  }
  ```
- 子问题最终会收敛于基本解

...\Ch05\RecursiveFibonacci\RecursiveFibonacci.cpp：

```c++
// RecursiveFibonacci.cpp
//
// 递归法求斐波那契数列
//
#include <iostream>
using namespace std;

long fibonacci(long number); // 函数原型

int main() {
    // 显示斐波那契数列的前10项
    for (int number{ 1 }; number <= 10; ++number) {
        cout << fibonacci(number) << ' ';
    }

    cout << "... ";

    // 显示更多项
    cout << fibonacci(20) << " ... " << fibonacci(30) << " ... " << fibonacci(35) << endl;
}

// 递归法求斐波那契数列的函数
long fibonacci(long number) {
    if (number == 1 || number == 2) { // 测试基本解的条件
        return number - 1;
    }
    else { // 递归调用
        return fibonacci(number - 1) + fibonacci(number - 2);
    }
}
```

编译运行：

```
0 1 1 2 3 5 8 13 21 34 ... 4181 ... 514229 ... 5702887
```

程序代码的第23到30行定义了一个递归法求斐波那契数列的函数。第24行首先判断递归终止条件，即基本解的条件。如果参数number的值为1或2，即不再继续递归，而是直接返回0或1。而对于其它情况，即参数number的值大于2，则执行第28行的递归调用，求斐波那契数列第number-1和number-2项的值，二者相加，得到第number项的值，并返回之。而在第number-1和number-2项的求值过程中，重复以上步骤，直至递归终止。

### 5.18.2 fibonacci函数调用展开

```mermaid
graph BT
  f5("fibonacci(5)")
  f43("return fibonacci(4) + fibonacci(3)")
  f32("return fibonacci(3) + fibonacci(2)")
  f21a("return fibonacci(2) + fibonacci(1)")
  f21b("return fibonacci(2) + fibonacci(1)")
  r1a(return 1)
  r1b(return 1)
  r1c(return 1)
  r0a(return 0)
  r0b(return 0)
  f43--3-->f5
  f32--2-->f43
  f21a--1-->f43
  f21b--1-->f32
  r1a--1-->f32
  r1b--1-->f21a
  r0a--0-->f21a
  r1c--1-->f21b
  r0b--0-->f21b
```

### 5.18.3 操作数的求值顺序

在类似fibonacci(4)+fibonacci(3)的表达式中，究竟是先求fibonacci(4)的值，再求fibonacci(3)的值，还是先求fibonacci(3)的值，再求fibonacci(4)的值，C++语言并没有明确的规定。在不同的编译器上可有会有不同的求值顺序。在这个程序中，不同的求值顺序，最终的结果都是一样的。但在另一些程序中，表达式中操作数的求值顺序，可能会影响到最终的计算结果。例如：

```c++
int i{ 0 }, j{ 0 };
j = ++i - ++i;
```

如果先对左操作数求值再对右操作数求值，变量j的值将是$1-2=-1$，而如果先对右操作数求值再对左操作数求值，变量j的值将是$2-1=1$。

### 5.18.4 规定了求值顺序的操作符

在C++17之前，语言标准只为逻辑与操作符（&&）、逻辑或操作符（||）、逗号操作符（,）和条件操作符（?:）规定了操作数的求值顺序。其中逻辑与操作符（&&）、逻辑或操作符（||）、逗号操作符（,），都是二元操作符，带有两个操作数，求值顺序为先左后右，即先对左操作数求值，再对右操作数求值，然后计算表达式的值。条件操作符（?:）是一个三元操作符，带有三个操作数，求值顺序为先对最左边的操作数求值，若该值为真，则对中间的操作数求值，并将其作为整个表达式的值，最右边的操作数将被忽略；若该值为假，则对最右边的操作数求值，并将其作为整个表达式的值，中间的操作数将被忽略。

从C++17开始，语言标准为更多的操作符规定了操作数的求值顺序。对于下标操作符（[]）、间接成员访问操作符（->）、函数操作符（()）、流插入操作符（<<）、流提取操作符（>>）、直接成员指针解引用操作符（.\*）、间接成员指针解引用操作符（->\*）等，编译器都会先对左操作数求值，再对右操作数求值。其中函数操作符（()）的左操作数是一个函数，总是先于其实参被求值的。而对于赋值操作符（=），则是先对右操作数求值，再对左操作数求值。

语言标准对操作符求值顺序的规定只是问题的一个方面，作为程序代码的编写者，尽量规避操作数求值顺序对计算结果的影响，则是问题的另一个方面，比如将一个复杂的表达式，分解为多个单独执行的语句，不失为一种好的设计。另外，逻辑与（&&）和逻辑或（||）操作符具有短路特性。如果被短路掉的恰恰是应该被求值的操作数，将可能引发逻辑错误。例如：

```c++
if (foo() && bar()) {
    ...
}
```

如果foo函数的返回值为假，则bar函数并不会被调用，但这也许并非程序设计者的本意。更好的做法是：

```c++
bool f, b;
f = foo();
b = bar();
if (f && b) {
    ...
}
```

保证foo函数和bar函数都会被调用，而条件判断逻辑与之前并没有不同。

### 5.18.5 指数级复杂度

前面给出的递归法求斐波那契数列的函数，其执行效率差强人意。为了计算fibonacci(5)需要先计算fibonacci(4)和fibonacci(3)，而在计算fibonacci(4)的过程中也需要计算fibonacci(3)。这显然是一种重复，其结果就是，在求取斐波那契数列第$n$项的过程中，共发生$2^{n-2}$次递归调用。为了求第20项，需要递归调用fibonacci函数$2^{18}=262144$次。为了求第30项，需要递归调用fibonacci函数$2^{28}=268435456$次。指数级别的时间复杂度，会令斐波那契数列的求值过程，很快陷入失控状态。