## 5.19 递归与循环

很多问题，比如计算阶乘或者求斐波那契数列，既可以用递归的方法解决，也可以用循环的方法解决。在特定情况下，往往会更青睐其中的一种，而弃绝另一种。首先对二者做一番比较：

- 递归与循环都是基于一个控制语句，递归基于选择语句，而循环基于循环语句
- 递归与循环都涉及迭代，递归通过调用自身实现迭代，循环则在循环体中实现迭代
- 递归与循环都需要终止条件，递归在终止条件满足时返回，而循环在终止条件满足时退出循环
- 递归的终止条件通常是问题的基本解，而循环的终止条件则取决于计数器或哨兵值
- 递归与循环都存在无限持续的可能，无限递归往往是因为问题没有收敛于基本解，而无限循环则源于使循环继续的条件恒为真

### 5.19.1 循环法求阶乘

循环法求阶乘的理论依据：

- $0!=1$

- $n!=n(n-1)!=n(n-1)(n-2)!=n(n-1)(n-2)\cdots0!=n(n-1)(n-2)\cdots1$

...\Ch05\LoopFactorial\LoopFactorial.cpp：

```c++
// LoopFactorial.cpp
//
// 循环法求阶乘
//
#include <iostream>
#include <iomanip>
using namespace std;

long factorial(int number); // 函数原型

int main() {
    // 计算从0到10的阶乘
    for (int number{ 0 }; number <= 10; ++number) {
        cout << setw(2) << number << "! = " << factorial(number) << endl;
    }
}

// 循环法求阶乘的函数
long factorial(int number) {
    long result;

    // 阶乘迭代循环
    for (result = 1; number > 0; --number) {
        result *= number;
    }

    return result;
}
```

编译运行：

```
 0! = 1
 1! = 1
 2! = 2
 3! = 6
 4! = 24
 5! = 120
 6! = 720
 7! = 5040
 8! = 40320
 9! = 362880
10! = 3628800
```

程序代码的第19到28行定义了一个循环法求阶乘的函数。第23到25行使用一个循环语句，完成阶乘计算的迭代过程。将从number开始，然后是number-1，再然后是number-2，一直到1的值，全部累乘到变量result中。循环的终止条件是number的值减到0。循环法的关键不在于每次迭代都生成母问题的子问题，并逐渐收敛于基本解。相反，它以number的值大于0作为循环继续的条件，直至该条件不满足，结束迭代过程。

### 5.19.2 循环法求斐波那契数列

循环法求斐波那契数列的理论依据：

- 前两项的值依次为0和1
- 后续各项均为其前两项的和

...\Ch05\LoopFibonacci\LoopFibonacci.cpp：

```c++
// LoopFibonacci.cpp
//
// 循环法求斐波那契数列
//
#include <iostream>
using namespace std;

long fibonacci(long number); // 函数原型

int main() {
    // 显示斐波那契数列的前10项
    for (int number{ 1 }; number <= 10; ++number) {
        cout << fibonacci(number) << ' ';
    }

    cout << "... ";

    // 显示更多项
    cout << fibonacci(20) << " ... " << fibonacci(30) << " ... " << fibonacci(35) << endl;
}

// 循环法求斐波那契数列的函数
long fibonacci(long number) {
    long fibonacci_n_2{ 0 }, fibonacci_n_1{ 1 }, fibonacci_n;

    for (long n = 1; n <= number; ++n) {
        if (n == 1) {
            fibonacci_n = fibonacci_n_2;
        }
        else if (n == 2) {
            fibonacci_n = fibonacci_n_1;
        }
        else {
            fibonacci_n = fibonacci_n_1 + fibonacci_n_2;
            fibonacci_n_2 = fibonacci_n_1;
            fibonacci_n_1 = fibonacci_n;
        }
    }

    return fibonacci_n;
}
```

编译运行：

```
0 1 1 2 3 5 8 13 21 34 ... 4181 ... 514229 ... 5702887
```

程序代码的第23到41行定义了一个循环法求斐波那契数列的函数。第26到38行使用一个循环语句，完成斐波那契数列从第1项到第number项的迭代过程。第1项和第2项直接取0和1，其它各项取前两项的和。注意每次迭代的前一项和当前项，将成为下次迭代的前两项和前一项。循环终止的条件是已得到第number项的值。

使用循环法求斐波那契数列的效率要比使用递归法高很多。在求取斐波那契数列第$n$项的过程中，迭代循环体中的代码共执行$n$次。线性级别的时间复杂度远优于指数级别时间复杂度。

### 5.19.3 递归的缺点

递归的特点是不断地调用函数自身，这会消耗大量的处理机时间。每次递归调用，都会创建递归函数中包括参数在内的所有数据的一份拷贝，这会消耗大量的内存空间。而循环通常只在一次函数调用中完成，对处理机时间和内存空间的消耗远小于递归。

### 5.19.4 递归和循环的选择

如果使用递归能令算法明显变得简单直观，令程序更易于理解和调试，且在时空性能方面付出的代价不大，那么就选择递归。避免在任何追求性能的场合使用递归。不排除某些问题只能使用递归方案解决的可能性。