## 5.20 对象自然案例

创建现有类（Cipher）创建一个对象，用于对信息的加密和解密，该类实现了维吉尼亚密码。

### 5.20.1 密码学

密码学已有数千年的历史，在互联网时代更是至关重要。每一天，密码学都在幕后默默地工作着，确保互联网通信的私密性和安全性。例如，大多数网站都通过HTTPS协议，加密和解密浏览器与Web服务器间的交互。

### 5.20.2 凯撒密码

凯撒大帝用一种简单的字符替换技术加密军事通信。这种方法后来被称为凯撒密码。具体做法是，将通信报文中的每个字母，替换为其在字母表中向后数第三个字母：

- $A\leftarrow D$
- $B\leftarrow E$
- $C\leftarrow F$
- ······
- $X\leftarrow A$
- $Y\leftarrow B$
- $Z\leftarrow C$

假设有如下明文：

- $Caesar\ Cipher$

加密后得到的密文如下：

- $Fdhvdu\ Flskhu$

明文加密后得到了密文，密文解密后还原成明文。

凯撒密码和其它密码技术体验，参见：https://cryptii.com/pipes/caesar-cipher。

cryptii项目的开源实现，参见：https://github.com/cryptii/cryptii。

### 5.20.3 维吉尼亚密码

凯撒密码的缺陷在于很容易找到规律。比如字母“e”在英语中的使用频率最高。只要从密文中统计出出现频率最高的字母，它的明文极可能就是“e”，计算二者在字母表中的位置差，进而推导出其它密文字母对应的明文字母，完成破解。

比凯撒密码更安全的是维吉尼亚密码。它是一种密钥替换密码。明文字母和密文字母的对应关系，由密钥决定。密钥由发文方随机生成，并被告知给受信任的收文方。持有密钥的发文方，用密钥加密明文得到密文，交给收文方。持有同样密钥的收文方，用密钥解密得到的密文，还原成明文。没有密钥的第三方，即使窃得密文，也会因没有密钥，而无法将其解密为明文。

有关维吉尼亚密码的具体细节，参见：https://zh.wikipedia.org/wiki/维吉尼亚密码。

在这种密码系统中，密钥必须由字母组成，比如“XMWUJBVYHXZ”，密钥中字母的个数没有限制，加密和解密使用相同的密钥，这种加密方式称为对称加密。作为信息的发送者，必须以安全的方式，比如面对面，将密钥交给信息的接收者。

#### 5.20.3.1 使用现成的Cipher类

...\Ch05\UseCipher\UseCipher.cpp：

```c++
// UseCipher.cpp
//
// 使用维吉尼亚密码加密和解密一段文本
// 
#include <iostream>
#include <string>
using namespace std;

#include "cipher.h"

int main() {
    // 输入明文
    cout << "明文：";
    string plainText;
    getline(cin, plainText);

    // 输入密钥
    cout << "密钥：";
    string secretKey;
    getline(cin, secretKey);

    Cipher cipher; // 使用现成的Cipher类创建一个对象

    // 使用密钥加密明文得到密文
    string cipherText{ cipher.encrypt(plainText, secretKey) };
    cout << "密文：" << cipherText << endl;

    // 使用密钥解密密文得到明文
    cout << "明文：" << cipher.decrypt(cipherText, secretKey) << endl;
}
```

编译运行：

```
明文：Welcome to Modern C++ application development with C++20!
密钥：XMWUJBVYHXZ
密文：Tqhwxnz rv Jnaqnh L++ bknsfbxfeiw eztlinmyahc xdro Z++20!
明文：Welcome to Modern C++ application development with C++20!
```

程序代码的第15和20行，从用户处读取待加密的明文文本，和用于加解密的密钥字符串。而后在第22行，用定义在头文件"cipher.h"中的Cipher类创建了一个对象。借助该对象的方法，结合之前用户输入的密钥，在第25和29行，实现对明文的加密和对密文的解密。

#### 5.20.3.2 Cipher类的成员函数

Cipher类提供了两个关键的成员函数：

- encrypt：接收代表明文和密钥的字符串参数，用维吉尼亚密码对明文进行加密，返回包含密文的字符串
  ```c++
  string encrypt(const string& plainText, const string& secretKey);
  ```
- decrypt：接收代表密文和密钥的字符串参数，用维吉尼亚密码对密文进行解密，返回包含明文的字符串
  ```c++
  string decrypt(const string& cipherText, const string& secretKey);
  ```