## 7.1 导读

指针、内置的基于指针的数组和字符串（亦称C风格的字符串或C字符串），都继承自C语言。

### 7.1.1 现代C++对指针的淡化

指针很强大，但用起来颇具挑战性，且极易出错。因此自C++11开始，历经C++14、C++17和C++20，语言标准不断地增加新特性，以削弱甚至消除对指针的依赖。新开发的软件项目应优先考虑以下几点：

- 能使用引用或智能指针的地方，就不要使用指针
- 能使用array和vector的地方，就不要使用内置的基于指针的数组
- 能使用string和string_view的地方，就不要使用基于指针的字符串

### 7.1.2 有时仍然需要指针

在大量已部署的传统C++代码中，充斥着指针、基于指针的数组和基于指针的字符串，例如：

- 链表、队列、堆栈或树等数据结构，要求在使用过程中能够动态伸缩。当然，也可以直接使用标准库中现成的动态容器，如vector等
- 可执行程序的入口函数main，以基于指针的字符串形式，接收来自命令行的参数
- 函数用空指针参数或返回空指针，表示某种特殊的逻辑，如忽略此参数或发生错误，引用不可能为空

### 7.1.3 和指针有关的C++核心准则

C++核心准则鼓励程序员尽一切可能，避免使用指针、基于指针的数组和基于指针的字符串，以使程序代码更安全、更健壮。例如，一些准则建议使用引用而非指针，实现向函数传引用。

### 7.1.4 C++20为规避指针而新增的特性

如果不得不使用指针，特别是基于指针的数组，那么结合C++20的这两个新增特性，会使程序更加安全：

- 借助to_array函数，可以将一个基于指针的数组，转换为一个array对象
- span提供了一种更安全的方式，将基于指针的数组，作为传递给函数的参数
  - 可以与基于范围的for循环一起使用，既能方便地处理数组中的元素，又不会有下标越界的风险
  - 支持迭代，适用于标准库中的容器处理算法，如accumulate、sort等
  - 可以与array和vector一起工作

总之一句话，能不用指针就不用指针，即使不得不用，也要尽可能发挥to_array和span的优势。

### 7.1.5 其它概念

指针的声明与初始化。

取地址（&）和解引用（*）操作符。

通过指针向函数传递引用。

内置的基于指针的数组，及其与指针的关系。

为指针和它指向的数据使用const。

通过sizeof操作符，计算特定类型或特定类型值的字节数。

指针表达式和指针算术。

旧式C++代码中广泛使用的C字符串。

为了获取命令行参数，而不得不使用基于指针的数组和基于指针的字符串。