## 11.1 导读

借助操作符重载，可以使C++现有的操作符，也能用于自定义类的对象。

典型的操作符重载例子是“<<”和“>>”，它们即可用于左移位和右移位，也可用于插入到输出流和从输入流提取。

C++内置了很多操作符重载，比如“+”操作符，既能用于整数的相加，也能用于浮点数的相加，甚至可以用于字符串的相加。

通过自定义操作符重载，可令这些操作符支持更多的数据类型，特别是那些自己定义的数据类型。

编译器会根据操作数的类型，选择正确的重载版本。

通过被重载的操作符能完成的操作，也能通过显式的函数调用完成，但前者显然更加自然、方便。

### 11.1.1 string类的重载操作符

除“+”外，string类还支持更多的重载操作符。对于一个重要的标准库类而言，操作符重载的价值不容小觑。

### 11.1.2 动态内存管理和智能指针

动态内存管理使程序能在运行时而非编译时，获得对象所需的额外内存，并在不需要时予以释放，以利它用。

动态内存管理的潜在问题，集中体现在，由于某种原因分配了内存却没有释放，形成内存泄漏。

智能指针的价值在于，它能自动释放不再被使用的动态内存，避免内存泄漏。

与资源获取即初始化（RAII）策略相结合，智能指针能够避免那些十分隐蔽的内存泄漏问题。

### 11.1.3 MyArray

创建一个自定义的MyArray类，通过重载操作符和其它机制，解决原生的基于指针的数组所存在的各种问题。

为MyArray类定义五个特殊的成员函数：

- 拷贝构造函数
- 拷贝赋值操作符函数
- 移动构造函数
- 移动赋值操作符函数
- 析构函数

有时，缺省构造函数也被视为一种特殊的成员函数。

拷贝语义和移动语义，告诉编译器何时可以将资源从一个对象移动到另一个对象，以避免昂贵的资源复制。

MyArray类使用了智能指针和RAII。

为MyArray类重载一系列一元和二元操作符：

- 赋值操作符（=）
- 等于操作符（==）
- 不等于操作符（!=）
- 下标操作符（[]）
- 递增操作符（++）
- 加法赋值操作符（+=）
- 流提取操作符（>>）
- 流插入操作符（<<）

为MyArray类定义一个类型转换操作符，使MyArray对象可被转换为bool值，用true和false表示容器是否为空。

面向对象技术的精髓在于，构建、使用和重用有价值的类，并于用户、组织，乃至整个C++开源社区分享。

### 11.1.4 三路比较操作符

C++20新增了三路比较操作符（<=>），亦形象地称为太空船操作符。

### 11.1.5 类型转换

类型转换操作符和类型转换构造函数。

通过explicit关键字解决隐式类型转换的潜在问题。