## 12.1 导读

C++之父，本贾尼·斯特劳斯特卢普（Bjarne Stroustrup）在他的网站上维护了一份清单，其中罗列了约150个完全或部分使用C++语言开发的应用程序和系统。下面是其中的一部分：

- macOS、Windows等主流操作系统的组件
- Visual C++、GNU C++、Clang C++等主流C++语言编译器
- Amazon.com
- 脸书（Facebook）中需要高性能和高可靠性的组件
- 彭博社（Bloomberg）的实时金融信息系统
- 奥多比（Adobe）的许多创作、图形和多媒体应用程序
- MySQL、MongoDB等多种数据库系统
- 包括火星车在内的，许多美国国家航空航天局（NASA）项目

The Programming Languages Beacon提供了另一份清单，囊括了超过100个完全或部分使用C++语言开发的应用程序和系统。

许多使用C++语言开发的应用程序和系统的规模都十分庞大。以下是一些关于代码量的统计：

- 波音787飞机的航电设备和联机支持系统：650万行代码
- 波音787飞机的飞行软件系统：1400万行代码
- F-35战斗机软件系统：2400万行代码
- 位于瑞士日内瓦的大型强子对撞机软件系统：5000万行代码
- 脸书（Facebook）软件系统：6200万行代码
- 普通高端汽车软件系统：1~3亿行代码
- 自动驾驶汽车软件系统：10亿行代码

遍布全球乃至太空的，如此海量的C++代码，如何确保其在开发过程中，尽可能地消除错误和潜在的隐患，并在被部署到实际产品中以后，仍能处理可能出现的各种问题，绝非人力所及，这需要一套规则和自动化的机制。

### 12.1.1 异常和异常处理

异常表示程序在执行过程中遇到了问题，这些问题可能来自：

- try块中显式触及的代码
- 对其它函数的调用
- 操作符错误，如new无法获得额外的内存

异常处理有助于编写健壮的、高度容错的程序，它能捕捉到偶然发生的问题，并采取如下行动：

- 解决问题，继续执行
- 恰当清理，体面退出
- 放弃一切，突然终止

### 12.1.2 异常处理、栈展开和重新抛出

通过throw语句抛出异常，并在try-catch结构中捕获和处理异常。

无异常和有异常发生时的控制流程。

捕获并重新抛出异常。

如何处理在特定作用域内未被捕获的异常。

将异常记录到文件或数据库中，以便开发或维护人员审查。

### 12.1.3 何时使用异常和异常安全保证

并非所有的错误都适合通过异常来处理。何时应该使用异常？何时不应使用异常？如何在自己的代码中提供异常安全保证？是完全没有保证？还是通过noexcept保证不抛出任何异常？

### 12.1.4 构造和析构函数中的异常

为什么构造函数中错误要通过异常表达？为什么不要从析构函数中抛出异常？如何借助函数try块，捕获构造函数成员初始化列表中的异常？

### 12.1.5 动态内存分配失败

通过new操作符分配动态内存时，会以bad_alloc类型的异常，表示所发生的错误。在引入bad_alloc之前，大量历史遗留的C++代码，是如何处理动态内存分配失败的？

### 12.1.6 标准库异常继承结构和自定义异常类

C++标准库提供了一组具有继承关系的异常类。从这些异常类可以派生出自定义的异常类。通过引用捕获异常的价值，在于对派生类异常的捕获。

### 12.1.7 异常并未得到广泛应用

使用异常处理的程序，通常需要占用更多内存。一些组织出于这个原因，禁止在代码中使用异常。一些库则提供双接口，当错误发生时，一个版本抛出异常，而另一个版本则设置错误码。

### 12.1.8 契约

契约将在C++20之后的语言标准中发布。前置条件、后置条件和断言，作为运行时测试的契约来实现。如果这些条件失败，即违约。默认情况下，代码会立即终止，这有助于更快地发现错误，并在开发过程中消除它们，有望为最终部署创建更加健壮的代码。可以使用https://godbolt.org的GCC实验性契约，测试示例代码。

### 12.1.9 借助异常将错误处理与程序逻辑分离

异常处理提供了一种处理错误的标准机制。通过try-catch结构，将成功和失败的执行路径截然分开，使代码更容易阅读和维护。异常一旦发生，就不能轻易地被忽略。忽略一个异常，通常会导致程序终止运行。

### 12.1.10 通过返回值表达错误

在不使用异常的情况下，函数通常会在执行成功和失败时返回不同的值，以使调用者得以为后续操作规划不同的分支。这种做法的问题在于：

- 对返回值的判断完全依赖于调用者的代码，检测错误并非强制性的，疏于检测将成为一个潜藏的Bug
- 某些情况下，可能无法通过返回值表达错误，比如在构造函数或被重载的操作符函数中发生错误

通过异常处理错误，将有助于解决这些问题。