## 13.1 导读

标准库定义了一套基于模板且功能强大的可重用组件，实现了许多常见的数据结构，和用于处理这些数据结构的算法。这部分内容在历史上曾被称为标准模板库（Standard Template Library，STL）。但现在，它们已经成为C++标准库的一部分。

### 13.1.1 容器、迭代器和算法

标准库的三大关键组件：容器（模板化数据结构）、迭代器和算法。其中涉及到容器、容器适配器和近似容器。

### 13.1.2 容器的通用成员函数

每种容器都有一套属于的自己的成员函数，其中的一部分在每种容器中都有定义。这些容器包括但不限于array、vector、list和deque等等。

### 13.1.3 迭代器

迭代器用于操作容器中的元素，并具有类似指针的属性，可被视为指向容器中元素的指针。迭代器之于容器，和指针之于内置数组一样，可通过标准库算法操作。在通过标准库算法操作容器时，迭代器具有良好的表现力，有时甚至可将多行代码精简为一条语句。

### 13.1.4 算法

标准库算法是执行常见数据操作的函数模板，包括但不限于查找、排序、拷贝、转换和比较等等。其中：

- 90个算法函数被声明在\<algorithm\>头文件的std名字空间中。在std::ranges名字空间中，为其中的82个算法函数，提供了针对范围的重载版本
- 11个算法函数被声明在\<numeric\>头文件的std名字空间中
- 14个算法函数被声明在\<memory\>头文件的std名字空间中。在std::ranges名字空间中，为其提供了针对范围的重载版本
- 另有2个算法函数被声明在\<cstdlib\>头文件中

C++11和C++20增加了很多算法函数，C++17增加了少数几个算法函数。

有关算法函数的完整列表，参见：https://zh.cppreference.com/w/cpp/algorithm。

容器、迭代器和算法的关系：

- 大多数算法都使用迭代器访问容器中的元素
- 每种算法对可被其使用的迭代器都有最低要求
- 不同容器所支持迭代器的能力不尽相同，有的弱一些，有的强一些
- 一个算法能否用于一种容器，取决于该容器所支持的迭代器，能够满足该算法对迭代器的最低要求
- 迭代器封装了遍历和访问容器中元素的机制，即使是明显不同的容器，也能使用完全相同的算法
- 借助迭代器，可以自行定义算法，处理多种容器中的元素

### 13.1.5 范围

范围是自C++20标准引入的新特性。

任何容器，只要提供了代表其开始和终止位置的迭代器，就可被视为范围。

相比传统风格的std::copy函数，现代风格的std::ranges::copy函数，更有助于利用范围简化代码。

### 13.1.6 自定义模板化数据结构

下面是一些流行的数据结构：

- 链表：逻辑上被排成一排的元素的集合。在任意位置插入和删除元素的效率都很高
- 堆栈：只能在结构的一端，即栈顶，压入和弹出元素
- 队列：向队尾压入元素，从队首弹出元素
- 二叉树：非线性的分级结构，用于排序、查找和消重

借助指针把离散的元素对象仔细地编制在一起，但基于指针的代码复杂且容易出错，即使是最轻微的遗漏和疏忽，都可能导致严重的内存错误或泄漏，而编译器对此无法给出任何预警。

明智的做法是尽量使用C++标准库提供的现成容器，而不是重新发明轮子。标准库的容器、迭代器和算法足以满足大多数应用对数据结构的需求。使用成熟且在性能和灵活性方面经过充分构思和设计的代码，有助于节省大量的编码、测试和调试时间。