## 14.1 导读

关于标准库算法的参考，侧重于常见的容器操作，如：

- 针对元素的填充、生成、删除、查找、替换、比较、排序等
- 针对容器的复制、合并、交换、比较、求最值、确定边界、数学运算、集合运算等

标准库提供了许多预包装的，模板化的算法函数，包括：

- 90个算法函数被声明在\<algorithm\>头文件的std名字空间中。在std::ranges名字空间中，为其中的82个算法函数，提供了针对范围的重载版本
- 11个算法函数被声明在\<numeric\>头文件的std名字空间中
- 14个算法函数被声明在\<memory\>头文件的std名字空间中。在std::ranges名字空间中，为其提供了针对范围的重载版本
- 另有2个算法函数被声明在\<cstdlib\>头文件中

C++11和C++20增加了很多算法函数，C++17增加了少数几个算法函数。

有关算法函数的完整列表，参见：https://zh.cppreference.com/w/cpp/algorithm。

### 14.1.1 最低算法要求和概念

标准库的算法指定了最低要求，据此可以确定能将这些算法用于哪些容器、迭代器和函数。

std::ranges名字空间中基于范围的算法，用概念表达了它们的最低要求。

### 14.1.2 对比现代和传统的范围算法

自C++20以后引入的所谓现代范围算法，定义在std::ranges名字空间中，而在此之前的传统范围算法，则定义在std名字空间中。传统范围算法所针对的范围，由起始迭代器和终止迭代器界定。起始迭代器表示范围中第一个元素的位置，终止迭代器表示范围中最后一个元素的下一个位置。

### 14.1.3 lambda表达式、函数指针和函数对象

重新审视lambda表达式并关注更多细节。许多标准库算法都可以接受一个lambda表达式、函数指针或者函数对象作为参数。对相对简单的小任务而言，使用lambda表达式最为方便。lambda表达式通常可与函数指针或函数对象互换使用。函数对象是由重载了函数操作符的类所创建的对象，可被当作函数使用。任何声称可以接受一个函数作为参数的标准库算法，都可以同时接受lambda表达式、函数指针和函数对象。

### 14.1.4 视图和面向惰性求值的函数式编程

象许多现代编程语言一样，C++也支持函数式编程（Functional-Style Programming）。

使用C++20的\<ranges\>库进行函数式编程。

### 14.1.5 并行算法

C++17为\<algorithm\>头文件中的69个算法函数，增加了支持并行的重载版本，以充分发挥多核处理器的优势，提升程序的运行性能。

### 14.1.6 范围前瞻

截至C++20，并不是所有的算法函数，都在std::ranges名字空间中，拥有针对范围的重载版本。C++23将完成这一部分更新。开源项目rangesnext，提供了一些针对拟议更新的实现。